@@ -798,9 +798,6 @@ ags_bulk_member_finalize(GObject *gobject)
   g_free(bulk_member->plugin_name);
   g_free(bulk_member->control_port);
 
-  g_free(bulk_member->control_port);
-
-
   if(bulk_member->conversion != NULL){
     g_object_unref(bulk_member->conversion);
   }
@@ -1063,7 +1060,7 @@ ags_bulk_member_set_label(AgsBulkMember *bulk_member,
   }
 
 
-  bulk_member->widget_label = label;
+  bulk_member->widget_label = g_strdup(label);
 }
 
 void
