@@ -31,12 +31,14 @@ void ags_generic_preferences_class_init(AgsGenericPreferencesClass *generic_pref
 void ags_generic_preferences_connectable_interface_init(AgsConnectableInterface *connectable);
 void ags_generic_preferences_applicable_interface_init(AgsApplicableInterface *applicable);
 void ags_generic_preferences_init(AgsGenericPreferences *generic_preferences);
+static void ags_generic_preferences_finalize(GObject *gobject);
+
 void ags_generic_preferences_connect(AgsConnectable *connectable);
 void ags_generic_preferences_disconnect(AgsConnectable *connectable);
+
 void ags_generic_preferences_set_update(AgsApplicable *applicable, gboolean update);
 void ags_generic_preferences_apply(AgsApplicable *applicable);
 void ags_generic_preferences_reset(AgsApplicable *applicable);
-static void ags_generic_preferences_finalize(GObject *gobject);
 
 GtkListStore* ags_generic_preferences_create_segmentation();
 
@@ -58,7 +60,7 @@ ags_generic_preferences_get_type(void)
   static volatile gsize g_define_type_id__volatile = 0;
 
   if(g_once_init_enter (&g_define_type_id__volatile)){
-    GType ags_type_generic_preferences;
+    GType ags_type_generic_preferences = 0;
 
     static const GTypeInfo ags_generic_preferences_info = {
       sizeof (AgsGenericPreferencesClass),
@@ -96,7 +98,7 @@ ags_generic_preferences_get_type(void)
 				AGS_TYPE_APPLICABLE,
 				&ags_applicable_interface_info);
 
-    g_once_init_leave (&g_define_type_id__volatile, ags_type_generic_preferences);
+    g_once_init_leave(&g_define_type_id__volatile, ags_type_generic_preferences);
   }
 
   return g_define_type_id__volatile;
@@ -204,6 +206,7 @@ ags_generic_preferences_init(AgsGenericPreferences *generic_preferences)
 static void
 ags_generic_preferences_finalize(GObject *gobject)
 {
+  /* call parent */
   G_OBJECT_CLASS(ags_generic_preferences_parent_class)->finalize(gobject);
 }
 
@@ -401,7 +404,7 @@ ags_generic_preferences_create_segmentation()
  *
  * Returns: a new #AgsGenericPreferences
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 AgsGenericPreferences*
 ags_generic_preferences_new()
