@@ -32,13 +32,13 @@ void ags_output_listing_editor_class_init(AgsOutputListingEditorClass *output_li
 void ags_output_listing_editor_connectable_interface_init(AgsConnectableInterface *connectable);
 void ags_output_listing_editor_applicable_interface_init(AgsApplicableInterface *applicable);
 void ags_output_listing_editor_init(AgsOutputListingEditor *output_listing_editor);
+
 void ags_output_listing_editor_connect(AgsConnectable *connectable);
 void ags_output_listing_editor_disconnect(AgsConnectable *connectable);
+
 void ags_output_listing_editor_set_update(AgsApplicable *applicable, gboolean update);
 void ags_output_listing_editor_apply(AgsApplicable *applicable);
 void ags_output_listing_editor_reset(AgsApplicable *applicable);
-void ags_output_listing_editor_destroy(GtkObject *object);
-void ags_output_listing_editor_show(GtkWidget *widget);
 
 /**
  * SECTION:ags_output_listing_editor
@@ -59,7 +59,7 @@ ags_output_listing_editor_get_type(void)
   static volatile gsize g_define_type_id__volatile = 0;
 
   if(g_once_init_enter (&g_define_type_id__volatile)){
-    GType ags_type_output_listing_editor;
+    GType ags_type_output_listing_editor = 0;
 
     static const GTypeInfo ags_output_listing_editor_info = {
       sizeof (AgsOutputListingEditorClass),
@@ -98,7 +98,7 @@ ags_output_listing_editor_get_type(void)
 				AGS_TYPE_APPLICABLE,
 				&ags_applicable_interface_info);
 
-    g_once_init_leave (&g_define_type_id__volatile, ags_type_output_listing_editor);
+    g_once_init_leave(&g_define_type_id__volatile, ags_type_output_listing_editor);
   }
 
   return g_define_type_id__volatile;
@@ -294,7 +294,7 @@ ags_output_listing_editor_reset(AgsApplicable *applicable)
  *
  * Creates new pad editors or destroys them.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_output_listing_editor_add_children(AgsOutputListingEditor *output_listing_editor,
@@ -306,12 +306,6 @@ ags_output_listing_editor_add_children(AgsOutputListingEditor *output_listing_ed
 
   AgsChannel *channel;
 
-  AgsMutexManager *mutex_manager;
-
-  pthread_mutex_t *application_mutex;
-  pthread_mutex_t *audio_mutex;
-  pthread_mutex_t *channel_mutex;
-
   if(nth_channel == 0 &&
      output_listing_editor->child != NULL){
     vbox = output_listing_editor->child;
@@ -322,17 +316,6 @@ ags_output_listing_editor_add_children(AgsOutputListingEditor *output_listing_ed
   if(audio == NULL){
     return;
   }
-  
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
-
-  /* lookup audio mutex */
-  pthread_mutex_lock(application_mutex);
-  
-  audio_mutex = ags_mutex_manager_lookup(mutex_manager,
-					 audio);
-  
-  pthread_mutex_unlock(application_mutex);
 
   /* instantiate pad editor vbox */
   if(nth_channel == 0){
@@ -344,35 +327,22 @@ ags_output_listing_editor_add_children(AgsOutputListingEditor *output_listing_ed
   }
 
   /* get current channel */
-  if(output_listing_editor->channel_type == AGS_TYPE_OUTPUT){
-    pthread_mutex_lock(audio_mutex);
-
-    channel = audio->output;
-
-    pthread_mutex_unlock(audio_mutex);
-
-    channel = ags_channel_nth(channel,
-			      nth_channel);
+  if(g_type_is_a(output_listing_editor->channel_type, AGS_TYPE_OUTPUT)){
+    g_object_get(audio,
+		 "output", &channel,
+		 NULL);
+  }else if(g_type_is_a(output_listing_editor->channel_type, AGS_TYPE_INPUT)){
+    g_object_get(audio,
+		 "input", &channel,
+		 NULL); 
   }else{
-    pthread_mutex_lock(audio_mutex);
-
-    channel = audio->input;
-
-    pthread_mutex_unlock(audio_mutex);
-    
-    channel = ags_channel_nth(channel,
-			      nth_channel);
+    channel = NULL;
   }
   
+  channel = ags_channel_nth(channel,
+			    nth_channel);
+  
   while(channel != NULL){
-    /* lookup channel mutex */
-    pthread_mutex_lock(application_mutex);
-
-    channel_mutex = ags_mutex_manager_lookup(mutex_manager,
-					     channel);
-    
-    pthread_mutex_unlock(application_mutex);
-
     /* instantiate pad editor */
     pad_editor = ags_pad_editor_new(NULL);
 
@@ -395,11 +365,9 @@ ags_output_listing_editor_add_children(AgsOutputListingEditor *output_listing_ed
     }
 
     /* iterate */
-    pthread_mutex_lock(channel_mutex);
-      
-    channel = channel->next_pad;
-
-    pthread_mutex_unlock(channel_mutex);
+    g_object_get(channel,
+		 "next-pad", &channel,
+		 NULL);
   }
 }
 
@@ -407,11 +375,11 @@ ags_output_listing_editor_add_children(AgsOutputListingEditor *output_listing_ed
  * ags_output_listing_editor_new:
  * @channel_type: the channel type to represent
  *
- * Creates an #AgsOutputListingEditor
+ * Create a new instance of #AgsOutputListingEditor
  *
- * Returns: a new #AgsOutputListingEditor
+ * Returns: the new #AgsOutputListingEditor
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 AgsOutputListingEditor*
 ags_output_listing_editor_new(GType channel_type)
