@@ -1,5 +1,5 @@
 /* GSequencer - Advanced GTK Sequencer
- * Copyright (C) 2005-2017 Joël Krähemann
+ * Copyright (C) 2005-2018 Joël Krähemann
  *
  * This file is part of GSequencer.
  *
@@ -25,9 +25,10 @@
 
 #include <gtk/gtk.h>
 
-#include <ags/X/ags_property_listing_editor.h>
+#include <ags/libags.h>
+#include <ags/libags-audio.h>
 
-#include <ags/audio/ags_audio.h>
+#include <ags/X/ags_property_listing_editor.h>
 
 #define AGS_TYPE_OUTPUT_LISTING_EDITOR                (ags_output_listing_editor_get_type())
 #define AGS_OUTPUT_LISTING_EDITOR(obj)                (G_TYPE_CHECK_INSTANCE_CAST((obj), AGS_TYPE_OUTPUT_LISTING_EDITOR, AgsOutputListingEditor))
