@@ -42,8 +42,10 @@ void ags_pad_editor_get_property(GObject *gobject,
 				 guint prop_id,
 				 GValue *value,
 				 GParamSpec *param_spec);
+
 void ags_pad_editor_connect(AgsConnectable *connectable);
 void ags_pad_editor_disconnect(AgsConnectable *connectable);
+
 void ags_pad_editor_set_update(AgsApplicable *applicable, gboolean update);
 void ags_pad_editor_apply(AgsApplicable *applicable);
 void ags_pad_editor_reset(AgsApplicable *applicable);
@@ -70,7 +72,7 @@ ags_pad_editor_get_type(void)
   static volatile gsize g_define_type_id__volatile = 0;
 
   if(g_once_init_enter (&g_define_type_id__volatile)){
-    GType ags_type_pad_editor;
+    GType ags_type_pad_editor = 0;
 
     static const GTypeInfo ags_pad_editor_info = {
       sizeof (AgsPadEditorClass),
@@ -108,7 +110,7 @@ ags_pad_editor_get_type(void)
 				AGS_TYPE_APPLICABLE,
 				&ags_applicable_interface_info);
 
-    g_once_init_leave (&g_define_type_id__volatile, ags_type_pad_editor);
+    g_once_init_leave(&g_define_type_id__volatile, ags_type_pad_editor);
   }
 
   return g_define_type_id__volatile;
@@ -388,7 +390,7 @@ ags_pad_editor_reset(AgsApplicable *applicable)
  *
  * Is called as channel gets modified.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_pad_editor_set_channel(AgsPadEditor *pad_editor, AgsChannel *channel)
@@ -408,35 +410,17 @@ ags_pad_editor_set_channel(AgsPadEditor *pad_editor, AgsChannel *channel)
     
     AgsChannel *next_pad;
 
-    AgsMutexManager *mutex_manager;
-
     gchar *str;
     
     guint pad;
     guint i;
 
-    pthread_mutex_t *application_mutex;
-    pthread_mutex_t *channel_mutex;
-    
-    mutex_manager = ags_mutex_manager_get_instance();
-    application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
-
-    /* lookup channel mutex */
-    pthread_mutex_lock(application_mutex);
-
-    channel_mutex = ags_mutex_manager_lookup(mutex_manager,
-					     channel);
-    
-    pthread_mutex_unlock(application_mutex);
-
     /* get some channel fields */
-    pthread_mutex_lock(channel_mutex);
+    g_object_get(channel,
+		 "pad", &pad,
+		 "next-pad", &next_pad,
+		 NULL);
     
-    next_pad = channel->next_pad;
-    pad = channel->pad;
-
-    pthread_mutex_unlock(channel_mutex);
-
     /* set label */
     str = g_strdup_printf("%s: %u",
 			  i18n("pad"),
@@ -451,14 +435,6 @@ ags_pad_editor_set_channel(AgsPadEditor *pad_editor, AgsChannel *channel)
 		      GTK_WIDGET(pad_editor->line_editor));
 
     while(channel != next_pad){
-      /* lookup channel mutex */
-      pthread_mutex_lock(application_mutex);
-
-      channel_mutex = ags_mutex_manager_lookup(mutex_manager,
-					       channel);
-    
-      pthread_mutex_unlock(application_mutex);
-      
       /* instantiate line editor */
       line_editor = ags_line_editor_new(NULL);
       line_editor->editor_type_count = pad_editor->editor_type_count;
@@ -478,11 +454,9 @@ ags_pad_editor_set_channel(AgsPadEditor *pad_editor, AgsChannel *channel)
 			 0);
 
       /* iterate */
-      pthread_mutex_lock(channel_mutex);
-      
-      channel = channel->next;
-
-      pthread_mutex_unlock(channel_mutex);
+      g_object_get(channel,
+		   "next", &channel,
+		   NULL);
     }
   }else{
     gtk_expander_set_label(pad_editor->line_editor_expander,
@@ -492,13 +466,13 @@ ags_pad_editor_set_channel(AgsPadEditor *pad_editor, AgsChannel *channel)
 
 /**
  * ags_pad_editor_new:
- * @channel: the channel to edit
+ * @channel: the #AgsChannel to edit
  *
- * Creates an #AgsPadEditor
+ * Create a new instance of #AgsPadEditor
  *
- * Returns: a new #AgsPadEditor
+ * Returns: the new #AgsPadEditor
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 AgsPadEditor*
 ags_pad_editor_new(AgsChannel *channel)
