@@ -35,7 +35,7 @@
 #define AGS_IS_PREFERENCES_CLASS(class)     (G_TYPE_CHECK_CLASS_TYPE((class), AGS_TYPE_PREFERENCES))
 #define AGS_PREFERENCES_GET_CLASS(obj)      (G_TYPE_INSTANCE_GET_CLASS((obj), AGS_TYPE_PREFERENCES, AgsPreferencesClass))
 
-#define AGS_PREFERENCES_DEFAULT_FILENAME "ags_current.xml\0"
+#define AGS_PREFERENCES_DEFAULT_FILENAME "ags_current.xml"
 
 typedef struct _AgsPreferences AgsPreferences;
 typedef struct _AgsPreferencesClass AgsPreferencesClass;
