@@ -36,12 +36,13 @@ void ags_sequencer_editor_class_init(AgsSequencerEditorClass *sequencer_editor);
 void ags_sequencer_editor_connectable_interface_init(AgsConnectableInterface *connectable);
 void ags_sequencer_editor_applicable_interface_init(AgsApplicableInterface *applicable);
 void ags_sequencer_editor_init(AgsSequencerEditor *sequencer_editor);
+
 void ags_sequencer_editor_connect(AgsConnectable *connectable);
 void ags_sequencer_editor_disconnect(AgsConnectable *connectable);
+
 void ags_sequencer_editor_set_update(AgsApplicable *applicable, gboolean update);
 void ags_sequencer_editor_apply(AgsApplicable *applicable);
 void ags_sequencer_editor_reset(AgsApplicable *applicable);
-static void ags_sequencer_editor_finalize(GObject *gobject);
 
 /**
  * SECTION:ags_sequencer_editor
@@ -61,7 +62,7 @@ ags_sequencer_editor_get_type(void)
   static volatile gsize g_define_type_id__volatile = 0;
 
   if(g_once_init_enter (&g_define_type_id__volatile)){
-    GType ags_type_sequencer_editor;
+    GType ags_type_sequencer_editor = 0;
 
     static const GTypeInfo ags_sequencer_editor_info = {
       sizeof (AgsSequencerEditorClass),
@@ -99,7 +100,7 @@ ags_sequencer_editor_get_type(void)
 				AGS_TYPE_APPLICABLE,
 				&ags_applicable_interface_info);
 
-    g_once_init_leave (&g_define_type_id__volatile, ags_type_sequencer_editor);
+    g_once_init_leave(&g_define_type_id__volatile, ags_type_sequencer_editor);
   }
 
   return g_define_type_id__volatile;
@@ -108,15 +109,7 @@ ags_sequencer_editor_get_type(void)
 void
 ags_sequencer_editor_class_init(AgsSequencerEditorClass *sequencer_editor)
 {
-  GObjectClass *gobject;
-  GtkWidgetClass *widget;
-
   ags_sequencer_editor_parent_class = g_type_class_peek_parent(sequencer_editor);
-
-  /* GtkObjectClass */
-  gobject = (GObjectClass *) sequencer_editor;
-
-  gobject->finalize = ags_sequencer_editor_finalize;
 }
 
 void
@@ -173,16 +166,9 @@ ags_sequencer_editor_init(AgsSequencerEditor *sequencer_editor)
 		   GTK_FILL, GTK_FILL,
 		   0, 0);
 
-#ifdef AGS_WITH_CORE_AUDIO
-  gtk_combo_box_text_append_text(sequencer_editor->backend,
-				 "core-audio");
-#endif
-
-#ifdef AGS_WITH_JACK
   gtk_combo_box_text_append_text(sequencer_editor->backend,
 				 "jack");
-#endif
-
+  
 #ifdef AGS_WITH_ALSA
   gtk_combo_box_text_append_text(sequencer_editor->backend,
 				 "alsa");
@@ -217,28 +203,28 @@ ags_sequencer_editor_init(AgsSequencerEditor *sequencer_editor)
 		   0, 0);
 
   /* jack */
-  sequencer_editor->source_hbox = NULL;
-  sequencer_editor->add_source = NULL;
-  sequencer_editor->remove_source = NULL;
+  sequencer_editor->jack_hbox = NULL;
+  sequencer_editor->add_jack = NULL;
+  sequencer_editor->remove_jack = NULL;
 
-  sequencer_editor->source_hbox = (GtkHBox *) gtk_hbox_new(FALSE,
+  sequencer_editor->jack_hbox = (GtkHBox *) gtk_hbox_new(FALSE,
 							 0);
   gtk_table_attach(table,
-		   GTK_WIDGET(sequencer_editor->source_hbox),
+		   GTK_WIDGET(sequencer_editor->jack_hbox),
 		   2, 3,
 		   1, 2,
 		   GTK_FILL, GTK_FILL,
 		   0, 0);
 
-  sequencer_editor->add_source = (GtkButton *) gtk_button_new_from_stock(GTK_STOCK_ADD);
-  gtk_box_pack_start((GtkBox *) sequencer_editor->source_hbox,
-		     (GtkWidget *) sequencer_editor->add_source,
+  sequencer_editor->add_jack = (GtkButton *) gtk_button_new_from_stock(GTK_STOCK_ADD);
+  gtk_box_pack_start((GtkBox *) sequencer_editor->jack_hbox,
+		     (GtkWidget *) sequencer_editor->add_jack,
 		     FALSE, FALSE,
 		     0);
   
-  sequencer_editor->remove_source = (GtkButton *) gtk_button_new_from_stock(GTK_STOCK_REMOVE);
-  gtk_box_pack_start((GtkBox *) sequencer_editor->source_hbox,
-		     (GtkWidget *) sequencer_editor->remove_source,
+  sequencer_editor->remove_jack = (GtkButton *) gtk_button_new_from_stock(GTK_STOCK_REMOVE);
+  gtk_box_pack_start((GtkBox *) sequencer_editor->jack_hbox,
+		     (GtkWidget *) sequencer_editor->remove_jack,
 		     FALSE, FALSE,
 		     0);
   
@@ -273,11 +259,11 @@ ags_sequencer_editor_connect(AgsConnectable *connectable)
 		   G_CALLBACK(ags_sequencer_editor_card_changed_callback), sequencer_editor);
 
   /* add / remove jack */
-  g_signal_connect(G_OBJECT(sequencer_editor->add_source), "clicked",
-		   G_CALLBACK(ags_sequencer_editor_add_source_callback), sequencer_editor);
+  g_signal_connect(G_OBJECT(sequencer_editor->add_jack), "clicked",
+		   G_CALLBACK(ags_sequencer_editor_add_jack_callback), sequencer_editor);
 
-  g_signal_connect(G_OBJECT(sequencer_editor->remove_source), "clicked",
-		   G_CALLBACK(ags_sequencer_editor_remove_source_callback), sequencer_editor);
+  g_signal_connect(G_OBJECT(sequencer_editor->remove_jack), "clicked",
+		   G_CALLBACK(ags_sequencer_editor_remove_jack_callback), sequencer_editor);
 }
 
 void
@@ -307,27 +293,19 @@ ags_sequencer_editor_disconnect(AgsConnectable *connectable)
 		      NULL);
 
   /* add / remove jack */
-  g_object_disconnect(G_OBJECT(sequencer_editor->add_source),
+  g_object_disconnect(G_OBJECT(sequencer_editor->add_jack),
 		      "any_signal::clicked",
-		      G_CALLBACK(ags_sequencer_editor_add_source_callback),
+		      G_CALLBACK(ags_sequencer_editor_add_jack_callback),
 		      sequencer_editor,
 		      NULL);
 
-  g_object_disconnect(G_OBJECT(sequencer_editor->remove_source),
+  g_object_disconnect(G_OBJECT(sequencer_editor->remove_jack),
 		      "any_signal::clicked",
-		      G_CALLBACK(ags_sequencer_editor_remove_source_callback),
+		      G_CALLBACK(ags_sequencer_editor_remove_jack_callback),
 		      sequencer_editor,
 		      NULL);
 }
 
-static void
-ags_sequencer_editor_finalize(GObject *gobject)
-{
-  //TODO:JK: implement me
-  
-  G_OBJECT_CLASS(ags_sequencer_editor_parent_class)->finalize(gobject);
-}
-
 void
 ags_sequencer_editor_set_update(AgsApplicable *applicable, gboolean update)
 {
@@ -353,12 +331,10 @@ ags_sequencer_editor_apply(AgsApplicable *applicable)
   char *device, *str;
 
   gint nth;
-  gboolean use_core_audio, use_jack, use_alsa, use_oss;
+  gboolean use_jack, use_alsa, use_oss;
   
   GValue value =  {0,};
 
-  pthread_mutex_t *application_mutex;
-
   sequencer_editor = AGS_SEQUENCER_EDITOR(applicable);
   midi_preferences = (AgsMidiPreferences *) gtk_widget_get_ancestor(GTK_WIDGET(sequencer_editor),
 								      AGS_TYPE_MIDI_PREFERENCES);
@@ -379,11 +355,7 @@ ags_sequencer_editor_apply(AgsApplicable *applicable)
   g_list_free(list);
   
   /* backend */
-#ifdef AGS_WITH_CORE_AUDIO
-  use_core_audio = TRUE;
-#else
   use_jack = TRUE;
-#endif
   use_alsa = FALSE;
   use_oss = FALSE;
 
@@ -395,10 +367,6 @@ ags_sequencer_editor_apply(AgsApplicable *applicable)
 
   if(backend != NULL){
     if(!g_ascii_strncasecmp(backend,
-			    "core-audio",
-			    11)){
-      use_core_audio = TRUE;
-    }else if(!g_ascii_strncasecmp(backend,
 			    "jack",
 			    5)){
       use_jack = TRUE;
@@ -430,12 +398,7 @@ ags_sequencer_editor_apply(AgsApplicable *applicable)
   /* handle */
   g_message("%s", device);
 
-  if(use_core_audio){
-    ags_config_set_value(config,
-			 sequencer_group,
-			 "device",
-			 device);
-  }else if(use_jack){
+  if(use_jack){
     ags_config_set_value(config,
 			 sequencer_group,
 			 "device",
@@ -480,9 +443,7 @@ ags_sequencer_editor_reset(AgsApplicable *applicable)
 
   backend = NULL;
   
-  if(AGS_IS_CORE_AUDIO_MIDIIN(sequencer)){
-    backend = "core-audio";
-  }else if(AGS_IS_JACK_MIDIIN(sequencer)){
+  if(AGS_IS_JACK_MIDIIN(sequencer)){
     backend = "jack";
   }else if(AGS_IS_MIDIIN(sequencer)){
     if((AGS_MIDIIN_ALSA & (AGS_MIDIIN(sequencer)->flags)) != 0){
@@ -494,13 +455,6 @@ ags_sequencer_editor_reset(AgsApplicable *applicable)
 
   if(backend != NULL){
     if(!g_ascii_strncasecmp(backend,
-			    "core-audio",
-			    5)){
-      gtk_combo_box_set_active(GTK_COMBO_BOX(sequencer_editor->backend),
-			       0);
-      
-      ags_sequencer_editor_load_core_audio_card(sequencer_editor);
-    }else if(!g_ascii_strncasecmp(backend,
 			    "jack",
 			    5)){
       gtk_combo_box_set_active(GTK_COMBO_BOX(sequencer_editor->backend),
@@ -606,136 +560,57 @@ ags_sequencer_editor_add_source(AgsSequencerEditor *sequencer_editor,
   AgsWindow *window;
   AgsPreferences *preferences;
 
-  AgsCoreAudioServer *core_audio_server;
-  AgsCoreAudioMidiin *core_audio_midiin;
-
   AgsJackServer *jack_server;
   AgsJackMidiin *jack_midiin;
 
-  AgsMutexManager *mutex_manager;
   AgsThread *main_loop;
   AgsThread *sequencer_thread;
 
   AgsApplicationContext *application_context;
 
-  GObject *sequencer;
-
-  GType server_type;
-
-  GList *distributed_manager;
+  GList *sound_server;
   GList *card_name, *card_uri;
 
-  gchar *backend;
-
-  gboolean use_core_audio, use_jack;
-  gboolean initial_sequencer;
-
-  pthread_mutex_t *application_mutex;
-  pthread_mutex_t *mutex;
-
   preferences = (AgsPreferences *) gtk_widget_get_ancestor(GTK_WIDGET(sequencer_editor),
 							   AGS_TYPE_PREFERENCES);
   window = AGS_WINDOW(preferences->window);
 
   application_context = (AgsApplicationContext *) window->application_context;
-  application_mutex = window->application_mutex;
-
-  mutex_manager = ags_mutex_manager_get_instance();
-  
-  core_audio_server = NULL;
-  core_audio_midiin = NULL;
-
-  jack_server = NULL;
-  jack_midiin = NULL;
-
-  sequencer = NULL;
-
-  server_type = G_TYPE_NONE;
-  
-  backend = NULL;
-
-  use_core_audio = FALSE;
-  use_jack = FALSE;
-
-  initial_sequencer = FALSE;
-
-  /* determine backend */
-  backend = gtk_combo_box_text_get_active_text(sequencer_editor->backend);
-
-  if(backend != NULL){
-    if(!g_ascii_strncasecmp(backend,
-			    "core-audio",
-			    11)){
-      server_type = AGS_TYPE_CORE_AUDIO_SERVER;
-      
-      use_core_audio = TRUE;
-    }else if(!g_ascii_strncasecmp(backend,
-			    "jack",
-			    5)){
-      server_type = AGS_TYPE_JACK_SERVER;
-
-      use_jack = TRUE;
-    }
-  }
   
   /* create sequencer */
-  pthread_mutex_lock(application_mutex);
+  sound_server = ags_sound_provider_get_sound_server(AGS_SOUND_PROVIDER(application_context));
 
-  distributed_manager = ags_sound_provider_get_distributed_manager(AGS_SOUND_PROVIDER(application_context));
-
-  if((distributed_manager = ags_list_util_find_type(distributed_manager,
-						    server_type)) != NULL){
-    if(use_core_audio){
-      core_audio_server = AGS_CORE_AUDIO_SERVER(distributed_manager->data);
-
-      sequencer = 
-	core_audio_midiin = (AgsCoreAudioMidiin *) ags_distributed_manager_register_sequencer(AGS_DISTRIBUTED_MANAGER(core_audio_server),
-											      FALSE);
-    }else if(use_jack){
-      jack_server = AGS_JACK_SERVER(distributed_manager->data);
-
-      sequencer = 
-	jack_midiin = (AgsJackMidiin *) ags_distributed_manager_register_sequencer(AGS_DISTRIBUTED_MANAGER(jack_server),
-										   FALSE);
-    }
+  if((sound_server = ags_list_util_find_type(sound_server,
+						    AGS_TYPE_JACK_SERVER)) != NULL){
+    jack_server = AGS_JACK_SERVER(sound_server->data);
   }else{
     g_warning("distributed manager not found");
-
-    pthread_mutex_unlock(application_mutex);
     
     return;
   }
   
-  pthread_mutex_unlock(application_mutex);
+  jack_midiin = (AgsJackMidiin *) ags_sound_server_register_sequencer(AGS_SOUND_SERVER(jack_server),
+								      FALSE);
 
-  if(sequencer == NULL){
+  if(jack_midiin == NULL){
     return;
   }
-
-  pthread_mutex_lock(application_mutex);
-
-  mutex = ags_mutex_manager_lookup(mutex_manager,
-				   (GObject *) sequencer);
   
 
   /* add new */
-  main_loop = (AgsThread *) application_context->main_loop;
+  g_object_get(application_context,
+	       "main-loop", &main_loop,
+	       NULL);
   
-  if(ags_sound_provider_get_sequencer(AGS_SOUND_PROVIDER(application_context)) == NULL){
-    initial_sequencer = TRUE;
-  }
-
-  sequencer_editor->sequencer = sequencer;
+  sequencer_editor->sequencer = jack_midiin;
 
   ags_sound_provider_set_sequencer(AGS_SOUND_PROVIDER(application_context),
 				   g_list_append(ags_sound_provider_get_sequencer(AGS_SOUND_PROVIDER(application_context)),
-						 sequencer));
-  
-  pthread_mutex_unlock(application_mutex);  
+						 jack_midiin));
     
-  g_object_ref(sequencer);
+  g_object_ref(jack_midiin);
 
-  sequencer_thread = (AgsThread *) ags_sequencer_thread_new(sequencer);
+  sequencer_thread = (AgsThread *) ags_sequencer_thread_new(jack_midiin);
   sequencer_editor->sequencer_thread = (GObject *) sequencer_thread;
   
   ags_thread_add_child_extended(main_loop,
@@ -743,15 +618,11 @@ ags_sequencer_editor_add_source(AgsSequencerEditor *sequencer_editor,
 				TRUE, TRUE);
 
   /*  */
-  pthread_mutex_lock(mutex);
-
   card_name = NULL;
   card_uri = NULL;
-  ags_sequencer_list_cards(AGS_SEQUENCER(sequencer),
+  ags_sequencer_list_cards(AGS_SEQUENCER(jack_midiin),
 			   &card_uri, &card_name);
 
-  pthread_mutex_unlock(mutex);
-
   gtk_list_store_clear(GTK_LIST_STORE(gtk_combo_box_get_model(GTK_COMBO_BOX(sequencer_editor->card))));
 
   while(card_uri != NULL){
@@ -772,124 +643,49 @@ ags_sequencer_editor_remove_source(AgsSequencerEditor *sequencer_editor,
   AgsPreferences *preferences;
   GtkDialog *dialog;
  
-  AgsCoreAudioServer *core_audio_server;
-  AgsCoreAudioMidiin *core_audio_midiin;
-
-  AgsJackServer *jack_server;
   AgsJackMidiin *jack_midiin;
 
   AgsApplicationContext *application_context;
   AgsThread *main_loop;
 
-  GObject *server;
-  GObject *sequencer;
-
-  GType server_type;
-
-  GList *machine, *machine_start;
-  GList *distributed_manager;
-  GList *list;
+  GList *sound_server;
+  GList *sequencer;
   GList *card_id;
 
-  gchar *backend;
-  
-  gboolean use_core_audio, use_pulse, use_jack;
-
-  pthread_mutex_t *application_mutex;
-
   preferences = (AgsPreferences *) gtk_widget_get_ancestor(GTK_WIDGET(sequencer_editor),
 							   AGS_TYPE_PREFERENCES);
   window = AGS_WINDOW(preferences->window);
   application_context = (AgsApplicationContext *) window->application_context;
-
-  application_mutex = window->application_mutex;
-
-  core_audio_server = NULL;
-  core_audio_midiin = NULL;
   
-  jack_server = NULL;
-  jack_midiin = NULL;
-
-  server = NULL;
-  sequencer = NULL;
-
-  server_type = G_TYPE_NONE;
+  /* create sequencer */
+  g_object_get(application_context,
+	       "main-loop", &main_loop,
+	       NULL);
   
-  backend = NULL;
+  sound_server = ags_sound_provider_get_sound_server(AGS_SOUND_PROVIDER(application_context));
 
-  use_core_audio = FALSE;
-  use_pulse = FALSE;
-  use_jack = FALSE;
-
-  /* determine backend */
-  backend = gtk_combo_box_text_get_active_text(sequencer_editor->backend);
-
-  if(backend != NULL){
-    if(!g_ascii_strncasecmp(backend,
-			    "core-audio",
-			    6)){
-      server_type = AGS_TYPE_CORE_AUDIO_SERVER;
-      
-      use_core_audio = TRUE;
-    }else if(!g_ascii_strncasecmp(backend,
-			    "jack",
-			    5)){
-      server_type = AGS_TYPE_JACK_SERVER;
-
-      use_jack = TRUE;
-    }
-  }
-  
-  /* destroy sequencer */
-  pthread_mutex_lock(application_mutex);
-
-  main_loop = application_context->main_loop;
-  distributed_manager = ags_sound_provider_get_distributed_manager(AGS_SOUND_PROVIDER(application_context));
-
-  if((distributed_manager = ags_list_util_find_type(distributed_manager,
-						    server_type)) != NULL){
-    if(use_core_audio){
-      server = 
-	core_audio_server = AGS_CORE_AUDIO_SERVER(distributed_manager->data);
-    }else if(use_jack){
-      server = 
-	jack_server = AGS_JACK_SERVER(distributed_manager->data);
-    }
-  }else{
+  if((sound_server = ags_list_util_find_type(sound_server,
+						    AGS_TYPE_JACK_SERVER)) == NULL){
     g_warning("distributed manager not found");
-
-    pthread_mutex_unlock(application_mutex);
     
     return;
   }
 
-  list = ags_sound_provider_get_sequencer(AGS_SOUND_PROVIDER(application_context));
+  sequencer = ags_sound_provider_get_sequencer(AGS_SOUND_PROVIDER(application_context));
+  jack_midiin = NULL;
   
-  while(list != NULL){
-    if(use_core_audio){
-      if(AGS_IS_CORE_AUDIO_MIDIIN(list->data) &&
-	 !g_ascii_strcasecmp(ags_sequencer_get_device(AGS_SEQUENCER(list->data)),
-			     device)){
-	sequencer = 
-	  core_audio_midiin = list->data;
-	break;
-      }
-    }else if(use_jack){
-      if(AGS_IS_JACK_MIDIIN(list->data) &&
-	 !g_ascii_strcasecmp(ags_sequencer_get_device(AGS_SEQUENCER(list->data)),
-			     device)){
-	sequencer = 
-	  jack_midiin = list->data;
-	break;
-      }
+  while(sequencer != NULL){
+    if(AGS_IS_JACK_MIDIIN(sequencer->data) &&
+       !g_ascii_strcasecmp(ags_sequencer_get_device(AGS_SEQUENCER(sequencer->data)),
+			   device)){
+      jack_midiin = sequencer->data;
+      break;
     }
     
-    list = list->next;
+    sequencer = sequencer->next;
   }
 
-  pthread_mutex_unlock(application_mutex);
-
-  if(sequencer == NULL){
+  if(jack_midiin == NULL){
     return;
   }
   
@@ -899,25 +695,28 @@ ags_sequencer_editor_remove_source(AgsSequencerEditor *sequencer_editor,
 			   -1);
 
 #if 0
-  if(server != NULL){
-    ags_distributed_manager_unregister_sequencer(AGS_DISTRIBUTED_MANAGER(server),
-						 sequencer);
+  if((sound_server = ags_list_util_find_type(sound_server,
+						    AGS_TYPE_JACK_SERVER)) != NULL){
+    ags_sound_server_unregister_sequencer(AGS_SOUND_SERVER(sound_server->data),
+						 jack_midiin);
+  }
+#endif
+
+  /* remove */
+  if(jack_midiin == sequencer_editor->sequencer){
+    sequencer_editor->sequencer = NULL;
+  }
+
+#if 0
+  if(jack_midiin != NULL){
+    ags_sound_provider_set_sequencer(AGS_SOUND_PROVIDER(application_context),
+				     g_list_remove(ags_sound_provider_get_sequencer(AGS_SOUND_PROVIDER(application_context)),
+						   jack_midiin));
+    g_object_unref(jack_midiin);
   }
 #endif
   
-  /* remove */
-  if(sequencer == sequencer_editor->sequencer){
-    sequencer_editor->sequencer = NULL;
-  }
-    
-#if 0  
-  if(sequencer != NULL){
-    ags_sound_provider_set_sequencer(AGS_SOUND_PROVIDER(application_context),
-				     g_list_remove(ags_sound_provider_get_sequencer(AGS_SOUND_PROVIDER(application_context)),
-						   sequencer));
-    g_object_unref(sequencer);
-  }
-  
+#if 0
   if(sequencer_editor->sequencer_thread != NULL){
     ags_thread_stop((AgsThread *) sequencer_editor->sequencer_thread);
 
@@ -929,7 +728,7 @@ ags_sequencer_editor_remove_source(AgsSequencerEditor *sequencer_editor,
     sequencer_editor->sequencer_thread = NULL;
   }
 #endif
-
+  
   /* notify user about restarting GSequencer */
   dialog = gtk_message_dialog_new(preferences,
 				  GTK_DIALOG_MODAL,
@@ -953,8 +752,6 @@ ags_sequencer_editor_add_sequencer(AgsSequencerEditor *sequencer_editor,
 
   AgsApplicationContext *application_context;
 
-  pthread_mutex_t *application_mutex;
-
   if(sequencer == NULL ||
      AGS_IS_JACK_MIDIIN(sequencer)){
     return;
@@ -962,9 +759,10 @@ ags_sequencer_editor_add_sequencer(AgsSequencerEditor *sequencer_editor,
   
   preferences = (AgsPreferences *) gtk_widget_get_ancestor(GTK_WIDGET(sequencer_editor),
 							   AGS_TYPE_PREFERENCES);
+
   window = AGS_WINDOW(preferences->window);
+
   application_context = (AgsApplicationContext *) window->application_context;
-  application_mutex = window->application_mutex;
 
   if(AGS_IS_MIDIIN(sequencer)){
     if((AGS_MIDIIN_ALSA & (AGS_MIDIIN(sequencer)->flags)) != 0){
@@ -981,13 +779,12 @@ ags_sequencer_editor_add_sequencer(AgsSequencerEditor *sequencer_editor,
   }
   
   /*  */
-  pthread_mutex_lock(application_mutex);
-
-  main_loop = (AgsThread *) application_context->main_loop;
+  g_object_get(application_context,
+	       "main-loop", &main_loop,
+	       NULL);
 
   if(g_list_find(ags_sound_provider_get_sequencer(AGS_SOUND_PROVIDER(application_context)),
 		 sequencer) != NULL){
-    pthread_mutex_unlock(application_mutex);  
   
     return;
   }
@@ -997,8 +794,6 @@ ags_sequencer_editor_add_sequencer(AgsSequencerEditor *sequencer_editor,
   ags_sound_provider_set_sequencer(AGS_SOUND_PROVIDER(application_context),
 				   g_list_append(ags_sound_provider_get_sequencer(AGS_SOUND_PROVIDER(application_context)),
 						 sequencer));
-
-  pthread_mutex_unlock(application_mutex);  
     
   g_object_ref(sequencer);
 
@@ -1023,22 +818,20 @@ ags_sequencer_editor_remove_sequencer(AgsSequencerEditor *sequencer_editor,
 
   AgsApplicationContext *application_context;
 
-  pthread_mutex_t *application_mutex;
-
   if(AGS_IS_JACK_MIDIIN(sequencer)){
     return;
   }
   
   preferences = (AgsPreferences *) gtk_widget_get_ancestor(GTK_WIDGET(sequencer_editor),
 							   AGS_TYPE_PREFERENCES);
+
   window = AGS_WINDOW(preferences->window);
+
   application_context = (AgsApplicationContext *) window->application_context;  
 
-  application_mutex = window->application_mutex;
-  
-  pthread_mutex_lock(application_mutex);
-
-  main_loop = (AgsThread *) application_context->main_loop;
+  g_object_get(application_context,
+	       "main-loop", &main_loop,
+	       NULL);
     
   if(sequencer == sequencer_editor->sequencer){
     sequencer_editor->sequencer = NULL;
@@ -1066,8 +859,6 @@ ags_sequencer_editor_remove_sequencer(AgsSequencerEditor *sequencer_editor,
   }
 #endif
   
-  pthread_mutex_unlock(application_mutex);
-
   /* notify user about restarting GSequencer */
   dialog = gtk_message_dialog_new(preferences,
 				  GTK_DIALOG_MODAL,
@@ -1079,74 +870,6 @@ ags_sequencer_editor_remove_sequencer(AgsSequencerEditor *sequencer_editor,
   gtk_widget_show_all(dialog);
 }
 
-void
-ags_sequencer_editor_load_core_audio_card(AgsSequencerEditor *sequencer_editor)
-{
-  AgsWindow *window;
-  AgsPreferences *preferences;
-
-  AgsCoreAudioMidiin *core_audio_midiin;
-
-  AgsApplicationContext *application_context;
-
-  GList *distributed_manager;
-  GList *sequencer;
-  GList *card_id;
-
-  pthread_mutex_t *application_mutex;
-
-  preferences = (AgsPreferences *) gtk_widget_get_ancestor(GTK_WIDGET(sequencer_editor),
-							   AGS_TYPE_PREFERENCES);
-  window = AGS_WINDOW(preferences->window);
-  application_context = (AgsApplicationContext *) window->application_context;
-
-  application_mutex = window->application_mutex;
-
-  /* create sequencer */
-  pthread_mutex_lock(application_mutex);
-
-  distributed_manager = ags_sound_provider_get_distributed_manager(AGS_SOUND_PROVIDER(application_context));
-
-  if((distributed_manager = ags_list_util_find_type(distributed_manager,
-						    AGS_TYPE_CORE_AUDIO_SERVER)) == NULL){
-    g_warning("distributed manager not found");
-
-    pthread_mutex_unlock(application_mutex);
-
-    return;
-  }
-
-  sequencer = ags_sound_provider_get_sequencer(AGS_SOUND_PROVIDER(application_context));
-  core_audio_midiin = NULL;
-  
-  while(sequencer != NULL){
-    if(AGS_IS_CORE_AUDIO_MIDIIN(sequencer->data)){
-      core_audio_midiin = sequencer->data;
-      break;
-    }
-    
-    sequencer = sequencer->next;
-  }
-
-  card_id = NULL;
-  ags_sequencer_list_cards(AGS_SEQUENCER(core_audio_midiin),
-			   &card_id, NULL);
-
-  gtk_list_store_clear(GTK_LIST_STORE(gtk_combo_box_get_model(GTK_COMBO_BOX(sequencer_editor->card))));
-
-  while(card_id != NULL){
-    if(card_id->data != NULL){
-      gtk_combo_box_text_append_text(sequencer_editor->card,
-				     card_id->data);
-    }
-    
-    card_id = card_id->next;
-  }
-
-  /*  */
-  pthread_mutex_unlock(application_mutex);
-}
-
 void
 ags_sequencer_editor_load_jack_card(AgsSequencerEditor *sequencer_editor)
 {
@@ -1157,30 +880,24 @@ ags_sequencer_editor_load_jack_card(AgsSequencerEditor *sequencer_editor)
 
   AgsApplicationContext *application_context;
 
-  GList *distributed_manager;
+  GList *sound_server;
   GList *sequencer;
   GList *card_id;
 
-  pthread_mutex_t *application_mutex;
-
   preferences = (AgsPreferences *) gtk_widget_get_ancestor(GTK_WIDGET(sequencer_editor),
 							   AGS_TYPE_PREFERENCES);
+
   window = AGS_WINDOW(preferences->window);
+
   application_context = (AgsApplicationContext *) window->application_context;
 
-  application_mutex = window->application_mutex;
-
   /* create sequencer */
-  pthread_mutex_lock(application_mutex);
+  sound_server = ags_sound_provider_get_sound_server(AGS_SOUND_PROVIDER(application_context));
 
-  distributed_manager = ags_sound_provider_get_distributed_manager(AGS_SOUND_PROVIDER(application_context));
-
-  if((distributed_manager = ags_list_util_find_type(distributed_manager,
+  if((sound_server = ags_list_util_find_type(sound_server,
 						    AGS_TYPE_JACK_SERVER)) == NULL){
     g_warning("distributed manager not found");
 
-    pthread_mutex_unlock(application_mutex);
-
     return;
   }
 
@@ -1210,9 +927,6 @@ ags_sequencer_editor_load_jack_card(AgsSequencerEditor *sequencer_editor)
     
     card_id = card_id->next;
   }
-
-  /*  */
-  pthread_mutex_unlock(application_mutex);
 }
 
 void
@@ -1228,18 +942,12 @@ ags_sequencer_editor_load_alsa_card(AgsSequencerEditor *sequencer_editor)
   GList *list;
   GList *card_id;
 
-  pthread_mutex_t *application_mutex;
-
   preferences = (AgsPreferences *) gtk_widget_get_ancestor(GTK_WIDGET(sequencer_editor),
 							   AGS_TYPE_PREFERENCES);
   window = AGS_WINDOW(preferences->window);
   application_context = (AgsApplicationContext *) window->application_context;
-
-  application_mutex = window->application_mutex;
   
   /*  */
-  pthread_mutex_lock(application_mutex);
-
   midiin = g_object_new(AGS_TYPE_MIDIIN,
 			NULL);
   midiin->flags &= (~AGS_MIDIIN_OSS);
@@ -1272,8 +980,6 @@ ags_sequencer_editor_load_alsa_card(AgsSequencerEditor *sequencer_editor)
   /* add new */
   ags_sequencer_editor_add_sequencer(sequencer_editor,
 				     (GObject *) midiin);
-
-  pthread_mutex_unlock(application_mutex);
 }
 
 void
@@ -1288,19 +994,15 @@ ags_sequencer_editor_load_oss_card(AgsSequencerEditor *sequencer_editor)
 
   GList *list;
   GList *card_id;
-
-  pthread_mutex_t *application_mutex;
   
   preferences = (AgsPreferences *) gtk_widget_get_ancestor(GTK_WIDGET(sequencer_editor),
 							   AGS_TYPE_PREFERENCES);
-  window = AGS_WINDOW(preferences->window);
-  application_context = (AgsApplicationContext *) window->application_context;
 
-  application_mutex = window->application_mutex;
+  window = AGS_WINDOW(preferences->window);
+
+  application_context = (AgsApplicationContext *) window->application_context;
   
   /*  */  
-  pthread_mutex_lock(application_mutex);
-
   midiin = g_object_new(AGS_TYPE_MIDIIN,
 			NULL);
   midiin->flags &= (~AGS_MIDIIN_ALSA);
@@ -1333,18 +1035,16 @@ ags_sequencer_editor_load_oss_card(AgsSequencerEditor *sequencer_editor)
   /* add new */
   ags_sequencer_editor_add_sequencer(sequencer_editor,
 				     (GObject *) midiin);
-
-  pthread_mutex_unlock(application_mutex);
 }
 
 /**
  * ags_sequencer_editor_new:
  *
- * Creates an #AgsSequencerEditor
+ * Create a new instance of #AgsSequencerEditor
  *
- * Returns: a new #AgsSequencerEditor
+ * Returns: the new #AgsSequencerEditor
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 AgsSequencerEditor*
 ags_sequencer_editor_new()
