@@ -1,5 +1,5 @@
 /* GSequencer - Advanced GTK Sequencer
- * Copyright (C) 2005-2017 Joël Krähemann
+ * Copyright (C) 2005-2018 Joël Krähemann
  *
  * This file is part of GSequencer.
  *
@@ -60,7 +60,7 @@ ags_soundcard_editor_get_type(void)
   static volatile gsize g_define_type_id__volatile = 0;
 
   if(g_once_init_enter (&g_define_type_id__volatile)){
-    GType ags_type_soundcard_editor;
+    GType ags_type_soundcard_editor = 0;
 
     static const GTypeInfo ags_soundcard_editor_info = {
       sizeof (AgsSoundcardEditorClass),
@@ -98,7 +98,7 @@ ags_soundcard_editor_get_type(void)
 				AGS_TYPE_APPLICABLE,
 				&ags_applicable_interface_info);
 
-    g_once_init_leave (&g_define_type_id__volatile, ags_type_soundcard_editor);
+    g_once_init_leave(&g_define_type_id__volatile, ags_type_soundcard_editor);
   }
 
   return g_define_type_id__volatile;
@@ -220,35 +220,35 @@ ags_soundcard_editor_init(AgsSoundcardEditor *soundcard_editor)
 		   GTK_FILL, GTK_FILL,
 		   0, 0);
 
-  /* sink */
-  soundcard_editor->sink_hbox = NULL;
-  soundcard_editor->add_sink = NULL;
-  soundcard_editor->remove_sink = NULL;
+  /* port */
+  soundcard_editor->port_hbox = NULL;
+  soundcard_editor->add_port = NULL;
+  soundcard_editor->remove_port = NULL;
 
-  soundcard_editor->sink_hbox = (GtkHBox *) gtk_hbox_new(FALSE,
+  soundcard_editor->port_hbox = (GtkHBox *) gtk_hbox_new(FALSE,
 							 0);
   gtk_table_attach(table,
-		   GTK_WIDGET(soundcard_editor->sink_hbox),
+		   GTK_WIDGET(soundcard_editor->port_hbox),
 		   2, 3,
 		   1, 2,
 		   GTK_FILL, GTK_FILL,
 		   0, 0);
 
-  soundcard_editor->add_sink = (GtkButton *) gtk_button_new_from_stock(GTK_STOCK_ADD);
-  gtk_box_pack_start((GtkBox *) soundcard_editor->sink_hbox,
-		     (GtkWidget *) soundcard_editor->add_sink,
+  soundcard_editor->add_port = (GtkButton *) gtk_button_new_from_stock(GTK_STOCK_ADD);
+  gtk_box_pack_start((GtkBox *) soundcard_editor->port_hbox,
+		     (GtkWidget *) soundcard_editor->add_port,
 		     FALSE, FALSE,
 		     0);
   
-  soundcard_editor->remove_sink = (GtkButton *) gtk_button_new_from_stock(GTK_STOCK_REMOVE);
-  gtk_box_pack_start((GtkBox *) soundcard_editor->sink_hbox,
-		     (GtkWidget *) soundcard_editor->remove_sink,
+  soundcard_editor->remove_port = (GtkButton *) gtk_button_new_from_stock(GTK_STOCK_REMOVE);
+  gtk_box_pack_start((GtkBox *) soundcard_editor->port_hbox,
+		     (GtkWidget *) soundcard_editor->remove_port,
 		     FALSE, FALSE,
 		     0);
-  
-  /* audio channels */
+
+  /* capability */
   label = (GtkLabel *) g_object_new(GTK_TYPE_LABEL,
-				    "label", i18n("audio channels"),
+				    "label", i18n("capability"),
 				    "xalign", 0.0,
 				    NULL);
   gtk_table_attach(table,
@@ -258,13 +258,47 @@ ags_soundcard_editor_init(AgsSoundcardEditor *soundcard_editor)
 		   GTK_FILL, GTK_FILL,
 		   0, 0);
 
+  soundcard_editor->capability = (GtkComboBoxText *) gtk_combo_box_text_new();
+  gtk_table_attach(table,
+		   GTK_WIDGET(soundcard_editor->capability),
+		   1, 2,
+		   2, 3,
+		   GTK_FILL, GTK_FILL,
+		   0, 0);
+
+  gtk_combo_box_text_append_text(soundcard_editor->capability,
+				 "playback");
+
+  gtk_combo_box_text_append_text(soundcard_editor->capability,
+				 "capture");
+
+#if 0
+  gtk_combo_box_text_append_text(soundcard_editor->capability,
+				 "duplex");
+#endif
+  
+  gtk_combo_box_set_active(GTK_COMBO_BOX(soundcard_editor->capability),
+			   0);
+  
+  /* audio channels */
+  label = (GtkLabel *) g_object_new(GTK_TYPE_LABEL,
+				    "label", i18n("audio channels"),
+				    "xalign", 0.0,
+				    NULL);
+  gtk_table_attach(table,
+		   GTK_WIDGET(label),
+		   0, 1,
+		   3, 4,
+		   GTK_FILL, GTK_FILL,
+		   0, 0);
+
   soundcard_editor->audio_channels = (GtkSpinButton *) gtk_spin_button_new_with_range(1.0, 24.0, 1.0);
   gtk_spin_button_set_value(soundcard_editor->audio_channels,
 			    2);
   gtk_table_attach(table,
 		   GTK_WIDGET(soundcard_editor->audio_channels),
 		   1, 2,
-		   2, 3,
+		   3, 4,
 		   GTK_FILL, GTK_FILL,
 		   0, 0);
 
@@ -276,7 +310,7 @@ ags_soundcard_editor_init(AgsSoundcardEditor *soundcard_editor)
   gtk_table_attach(table,
 		   GTK_WIDGET(label),
 		   0, 1,
-		   3, 4,
+		   4, 5,
 		   GTK_FILL, GTK_FILL,
 		   0, 0);
 
@@ -285,7 +319,7 @@ ags_soundcard_editor_init(AgsSoundcardEditor *soundcard_editor)
   gtk_table_attach(table,
 		   GTK_WIDGET(soundcard_editor->samplerate),
 		   1, 2,
-		   3, 4,
+		   4, 5,
 		   GTK_FILL, GTK_FILL,
 		   0, 0);
 
@@ -297,7 +331,7 @@ ags_soundcard_editor_init(AgsSoundcardEditor *soundcard_editor)
   gtk_table_attach(table,
 		   GTK_WIDGET(label),
 		   0, 1,
-		   4, 5,
+		   5, 6,
 		   GTK_FILL, GTK_FILL,
 		   0, 0);
 
@@ -306,7 +340,7 @@ ags_soundcard_editor_init(AgsSoundcardEditor *soundcard_editor)
   gtk_table_attach(table,
 		   GTK_WIDGET(soundcard_editor->buffer_size),
 		   1, 2,
-		   4, 5,
+		   5, 6,
 		   GTK_FILL, GTK_FILL,
 		   0, 0);
 
@@ -318,7 +352,7 @@ ags_soundcard_editor_init(AgsSoundcardEditor *soundcard_editor)
   gtk_table_attach(table,
 		   GTK_WIDGET(label),
 		   0, 1,
-		   5, 6,
+		   6, 7,
 		   GTK_FILL, GTK_FILL,
 		   0, 0);
 
@@ -326,7 +360,7 @@ ags_soundcard_editor_init(AgsSoundcardEditor *soundcard_editor)
   gtk_table_attach(table,
 		   GTK_WIDGET(soundcard_editor->format),
 		   1, 2,
-		   5, 6,
+		   6, 7,
 		   GTK_FILL, GTK_FILL,
 		   0, 0);
 
@@ -376,12 +410,12 @@ ags_soundcard_editor_connect(AgsConnectable *connectable)
   g_signal_connect(G_OBJECT(soundcard_editor->card), "changed",
 		   G_CALLBACK(ags_soundcard_editor_card_changed_callback), soundcard_editor);
 
-  /* add / remove sink */
-  g_signal_connect(G_OBJECT(soundcard_editor->add_sink), "clicked",
-		   G_CALLBACK(ags_soundcard_editor_add_sink_callback), soundcard_editor);
+  /* add / remove port */
+  g_signal_connect(G_OBJECT(soundcard_editor->add_port), "clicked",
+		   G_CALLBACK(ags_soundcard_editor_add_port_callback), soundcard_editor);
 
-  g_signal_connect(G_OBJECT(soundcard_editor->remove_sink), "clicked",
-		   G_CALLBACK(ags_soundcard_editor_remove_sink_callback), soundcard_editor);
+  g_signal_connect(G_OBJECT(soundcard_editor->remove_port), "clicked",
+		   G_CALLBACK(ags_soundcard_editor_remove_port_callback), soundcard_editor);
 
   /* presets */
   g_signal_connect(G_OBJECT(soundcard_editor->audio_channels), "changed",
@@ -423,16 +457,16 @@ ags_soundcard_editor_disconnect(AgsConnectable *connectable)
 		      soundcard_editor,
 		      NULL);
 
-  /* add / remove sink */
-  g_object_disconnect(G_OBJECT(soundcard_editor->add_sink),
+  /* add / remove port */
+  g_object_disconnect(G_OBJECT(soundcard_editor->add_port),
 		      "any_signal::clicked",
-		      G_CALLBACK(ags_soundcard_editor_add_sink_callback),
+		      G_CALLBACK(ags_soundcard_editor_add_port_callback),
 		      soundcard_editor,
 		      NULL);
 
-  g_object_disconnect(G_OBJECT(soundcard_editor->remove_sink),
+  g_object_disconnect(G_OBJECT(soundcard_editor->remove_port),
 		      "any_signal::clicked",
-		      G_CALLBACK(ags_soundcard_editor_remove_sink_callback),
+		      G_CALLBACK(ags_soundcard_editor_remove_port_callback),
 		      soundcard_editor,
 		      NULL);
 
@@ -492,6 +526,7 @@ ags_soundcard_editor_apply(AgsApplicable *applicable)
 
   gchar *soundcard_group;
   gchar *backend;
+  gchar *capability;
   char *device, *str;
 
   gint nth;
@@ -503,8 +538,6 @@ ags_soundcard_editor_apply(AgsApplicable *applicable)
   
   GValue value =  {0,};
 
-  pthread_mutex_t *application_mutex;
-
   soundcard_editor = AGS_SOUNDCARD_EDITOR(applicable);
   audio_preferences = (AgsAudioPreferences *) gtk_widget_get_ancestor(GTK_WIDGET(soundcard_editor),
 								      AGS_TYPE_AUDIO_PREFERENCES);
@@ -545,7 +578,7 @@ ags_soundcard_editor_apply(AgsApplicable *applicable)
     if(!g_ascii_strncasecmp(backend,
 			    "core-audio",
 			    11)){
-      use_pulse = TRUE;
+      use_core_audio = TRUE;
     }else if(!g_ascii_strncasecmp(backend,
 			    "pulse",
 			    6)){
@@ -565,6 +598,12 @@ ags_soundcard_editor_apply(AgsApplicable *applicable)
     }
   }
 
+  capability = gtk_combo_box_text_get_active_text(soundcard_editor->capability);
+  ags_config_set_value(config,
+		       soundcard_group,
+		       "capability",
+		       capability);
+
   /* buffer size */
   buffer_size = gtk_spin_button_get_value(soundcard_editor->buffer_size);
   str = g_strdup_printf("%u",
@@ -682,6 +721,7 @@ ags_soundcard_editor_reset(AgsApplicable *applicable)
 
   gchar *backend, *device, *str, *tmp;
 
+  guint capability;
   guint nth;
   guint channels, channels_min, channels_max;
   guint samplerate, samplerate_min, samplerate_max;
@@ -706,19 +746,48 @@ ags_soundcard_editor_reset(AgsApplicable *applicable)
   use_alsa = FALSE;
 
   backend = NULL;
+  capability = 0;
   
   if(AGS_IS_CORE_AUDIO_DEVOUT(soundcard)){
     backend = "core-audio";
+
+    capability = AGS_SOUNDCARD_CAPABILITY_PLAYBACK;
   }else if(AGS_IS_PULSE_DEVOUT(soundcard)){
     backend = "pulse";
+
+    capability = AGS_SOUNDCARD_CAPABILITY_PLAYBACK;
   }else if(AGS_IS_JACK_DEVOUT(soundcard)){
     backend = "jack";
+
+    capability = AGS_SOUNDCARD_CAPABILITY_PLAYBACK;
   }else if(AGS_IS_DEVOUT(soundcard)){
     if((AGS_DEVOUT_ALSA & (AGS_DEVOUT(soundcard)->flags)) != 0){
       backend = "alsa";
     }else if((AGS_DEVOUT_OSS & (AGS_DEVOUT(soundcard)->flags)) != 0){
       backend = "oss";
     }
+
+    capability = AGS_SOUNDCARD_CAPABILITY_PLAYBACK;
+  }else if(AGS_IS_CORE_AUDIO_DEVIN(soundcard)){
+    backend = "core-audio";
+
+    capability = AGS_SOUNDCARD_CAPABILITY_CAPTURE;
+  }else if(AGS_IS_PULSE_DEVIN(soundcard)){
+    backend = "pulse";
+
+    capability = AGS_SOUNDCARD_CAPABILITY_CAPTURE;
+  }else if(AGS_IS_JACK_DEVIN(soundcard)){
+    backend = "jack";
+
+    capability = AGS_SOUNDCARD_CAPABILITY_CAPTURE;
+  }else if(AGS_IS_DEVIN(soundcard)){
+    if((AGS_DEVIN_ALSA & (AGS_DEVIN(soundcard)->flags)) != 0){
+      backend = "alsa";
+    }else if((AGS_DEVIN_OSS & (AGS_DEVIN(soundcard)->flags)) != 0){
+      backend = "oss";
+    }
+
+    capability = AGS_SOUNDCARD_CAPABILITY_CAPTURE;
   }
 
   if(backend != NULL){
@@ -735,6 +804,10 @@ ags_soundcard_editor_reset(AgsApplicable *applicable)
       gtk_combo_box_set_active(GTK_COMBO_BOX(soundcard_editor->backend),
 			       0);
       
+      gtk_combo_box_set_active(soundcard_editor->capability,
+			       0);
+      gtk_widget_set_sensitive(soundcard_editor->capability,
+			       FALSE);
       //      ags_soundcard_editor_load_pulse_card(soundcard_editor);
     }else if(!g_ascii_strncasecmp(backend,
 			    "jack",
@@ -831,6 +904,28 @@ ags_soundcard_editor_reset(AgsApplicable *applicable)
     g_list_free_full(card_name_start,
 		     g_free);
   }
+
+  /* capability */
+  switch(capability){
+  case AGS_SOUNDCARD_CAPABILITY_PLAYBACK:
+    {
+      gtk_combo_box_set_active(GTK_COMBO_BOX(soundcard_editor->capability),
+			       0);
+    }
+    break;
+  case AGS_SOUNDCARD_CAPABILITY_CAPTURE:
+    {
+      gtk_combo_box_set_active(GTK_COMBO_BOX(soundcard_editor->capability),
+			       1);
+    }
+    break;
+  case AGS_SOUNDCARD_CAPABILITY_DUPLEX:
+    {
+      gtk_combo_box_set_active(GTK_COMBO_BOX(soundcard_editor->capability),
+			       2);
+    }
+    break;
+  }
   
   /*  */
   ags_soundcard_get_presets(AGS_SOUNDCARD(soundcard),
@@ -909,7 +1004,7 @@ ags_soundcard_editor_reset(AgsApplicable *applicable)
 }
 
 void
-ags_soundcard_editor_add_sink(AgsSoundcardEditor *soundcard_editor,
+ags_soundcard_editor_add_port(AgsSoundcardEditor *soundcard_editor,
 			      gchar *device)
 {
   AgsWindow *window;
@@ -926,7 +1021,6 @@ ags_soundcard_editor_add_sink(AgsSoundcardEditor *soundcard_editor,
 
   AgsNotifySoundcard *notify_soundcard;
   
-  AgsMutexManager *mutex_manager;
   AgsThread *main_loop;
   AgsThread *soundcard_thread;
   AgsThread *export_thread;
@@ -937,25 +1031,20 @@ ags_soundcard_editor_add_sink(AgsSoundcardEditor *soundcard_editor,
   
   GType server_type;
   
-  GList *distributed_manager;
+  GList *sound_server;
   GList *card_name, *card_uri;
 
   gchar *backend;
-  
+
   gboolean use_core_audio, use_pulse, use_jack;
+  gboolean is_output;
   gboolean initial_soundcard;
-  
-  pthread_mutex_t *application_mutex;
-  pthread_mutex_t *mutex;
 
   preferences = (AgsPreferences *) gtk_widget_get_ancestor(GTK_WIDGET(soundcard_editor),
 							   AGS_TYPE_PREFERENCES);
   window = AGS_WINDOW(preferences->window);
 
   application_context = (AgsApplicationContext *) window->application_context;
-  application_mutex = window->application_mutex;
-
-  mutex_manager = ags_mutex_manager_get_instance();
 
   core_audio_server = NULL;
   core_audio_devout = NULL;
@@ -976,6 +1065,8 @@ ags_soundcard_editor_add_sink(AgsSoundcardEditor *soundcard_editor,
   use_pulse = FALSE;
   use_jack = FALSE;
 
+  is_output = (gtk_combo_box_get_active(soundcard_editor->capability) == 0) ? TRUE: FALSE;
+  
   initial_soundcard = FALSE;
 
   /* determine backend */
@@ -1004,53 +1095,44 @@ ags_soundcard_editor_add_sink(AgsSoundcardEditor *soundcard_editor,
   }
   
   /* create soundcard */
-  pthread_mutex_lock(application_mutex);
+  sound_server = ags_sound_provider_get_sound_server(AGS_SOUND_PROVIDER(application_context));
 
-  distributed_manager = ags_sound_provider_get_distributed_manager(AGS_SOUND_PROVIDER(application_context));
-
-  if((distributed_manager = ags_list_util_find_type(distributed_manager,
-						    server_type)) != NULL){
+  if((sound_server = ags_list_util_find_type(sound_server,
+					     server_type)) != NULL){
     if(use_core_audio){
-      core_audio_server = AGS_CORE_AUDIO_SERVER(distributed_manager->data);
+      core_audio_server = AGS_CORE_AUDIO_SERVER(sound_server->data);
 
       soundcard = 
-	core_audio_devout = (AgsCoreAudioDevout *) ags_distributed_manager_register_soundcard(AGS_DISTRIBUTED_MANAGER(core_audio_server),
-											      TRUE);
+	core_audio_devout = (AgsCoreAudioDevout *) ags_sound_server_register_soundcard(AGS_SOUND_SERVER(core_audio_server),
+										       is_output);
     }else if(use_pulse){
-      pulse_server = AGS_PULSE_SERVER(distributed_manager->data);
+      pulse_server = AGS_PULSE_SERVER(sound_server->data);
 
       soundcard = 
-	pulse_devout = (AgsPulseDevout *) ags_distributed_manager_register_soundcard(AGS_DISTRIBUTED_MANAGER(pulse_server),
-										     TRUE);
+	pulse_devout = (AgsPulseDevout *) ags_sound_server_register_soundcard(AGS_SOUND_SERVER(pulse_server),
+									      is_output);
     }else if(use_jack){
-      jack_server = AGS_JACK_SERVER(distributed_manager->data);
+      jack_server = AGS_JACK_SERVER(sound_server->data);
 
       soundcard = 
-	jack_devout = (AgsJackDevout *) ags_distributed_manager_register_soundcard(AGS_DISTRIBUTED_MANAGER(jack_server),
-										   TRUE);
+	jack_devout = (AgsJackDevout *) ags_sound_server_register_soundcard(AGS_SOUND_SERVER(jack_server),
+									    is_output);
     }
   }else{
-    g_warning("distributed manager not found");
-
-    pthread_mutex_unlock(application_mutex);
+    g_warning("sound server not found");
     
     return;
   }
-  
-  pthread_mutex_unlock(application_mutex);
 
   if(soundcard == NULL){
     return;
   }
-
-  pthread_mutex_lock(application_mutex);
-
-  mutex = ags_mutex_manager_lookup(mutex_manager,
-				   (GObject *) soundcard);
   
 
   /* add new */
-  main_loop = (AgsThread *) application_context->main_loop;
+  g_object_get(application_context,
+	       "main-loop", &main_loop,
+	       NULL);
   
   if(ags_sound_provider_get_soundcard(AGS_SOUND_PROVIDER(application_context)) == NULL){
     initial_soundcard = TRUE;
@@ -1073,23 +1155,22 @@ ags_soundcard_editor_add_sink(AgsSoundcardEditor *soundcard_editor,
     while(machine != NULL){
       g_object_ref(G_OBJECT(AGS_MACHINE(machine->data)->audio));
   
-      list = ags_soundcard_get_audio(AGS_SOUNDCARD(soundcard));
+      list = ags_sound_provider_get_audio(AGS_SOUND_PROVIDER(application_context));
       list = g_list_prepend(list,
 			    AGS_MACHINE(machine->data)->audio);
-      ags_soundcard_set_audio(AGS_SOUNDCARD(soundcard),
-			      list);
+      ags_sound_provider_set_audio(AGS_SOUND_PROVIDER(application_context),
+				   list);
 
       machine = machine->next;
     }
 
     g_list_free(machine_start);
   }
-  
-  pthread_mutex_unlock(application_mutex);  
-    
+      
   g_object_ref(soundcard);
 
-  soundcard_thread = (AgsThread *) ags_soundcard_thread_new(soundcard);
+  soundcard_thread = (AgsThread *) ags_soundcard_thread_new(soundcard,
+							    ags_soundcard_get_capability(AGS_SOUNDCARD(soundcard)));
   soundcard_editor->soundcard_thread = (GObject *) soundcard_thread;
   
   ags_thread_add_child_extended(main_loop,
@@ -1123,18 +1204,12 @@ ags_soundcard_editor_add_sink(AgsSoundcardEditor *soundcard_editor,
 				(AgsThread *) export_thread,
 				TRUE, TRUE);
 
-  pthread_mutex_unlock(application_mutex);
-
   /*  */
-  pthread_mutex_lock(mutex);
-
   card_name = NULL;
   card_uri = NULL;
   ags_soundcard_list_cards(AGS_SOUNDCARD(soundcard),
 			   &card_uri, &card_name);
 
-  pthread_mutex_unlock(mutex);
-
   gtk_list_store_clear(GTK_LIST_STORE(gtk_combo_box_get_model(GTK_COMBO_BOX(soundcard_editor->card))));
 
   while(card_uri != NULL){
@@ -1148,7 +1223,7 @@ ags_soundcard_editor_add_sink(AgsSoundcardEditor *soundcard_editor,
 }
 
 void
-ags_soundcard_editor_remove_sink(AgsSoundcardEditor *soundcard_editor,
+ags_soundcard_editor_remove_port(AgsSoundcardEditor *soundcard_editor,
 				 gchar *device)
 {
   AgsWindow *window;
@@ -1173,7 +1248,7 @@ ags_soundcard_editor_remove_sink(AgsSoundcardEditor *soundcard_editor,
   GType server_type;
 
   GList *machine, *machine_start;
-  GList *distributed_manager;
+  GList *sound_server;
   GList *list;
   GList *card_id;
 
@@ -1181,14 +1256,12 @@ ags_soundcard_editor_remove_sink(AgsSoundcardEditor *soundcard_editor,
   
   gboolean use_core_audio, use_pulse, use_jack;
 
-  pthread_mutex_t *application_mutex;
-
   preferences = (AgsPreferences *) gtk_widget_get_ancestor(GTK_WIDGET(soundcard_editor),
 							   AGS_TYPE_PREFERENCES);
-  window = AGS_WINDOW(preferences->window);
-  application_context = (AgsApplicationContext *) window->application_context;
 
-  application_mutex = window->application_mutex;
+  window = AGS_WINDOW(preferences->window);
+
+  application_context = (AgsApplicationContext *) window->application_context;
 
   core_audio_server = NULL;
   core_audio_devout = NULL;
@@ -1236,27 +1309,26 @@ ags_soundcard_editor_remove_sink(AgsSoundcardEditor *soundcard_editor,
   }
   
   /* destroy soundcard */
-  pthread_mutex_lock(application_mutex);
+  g_object_get(application_context,
+	       "main-loop", &main_loop,
+	       NULL);
 
-  main_loop = application_context->main_loop;
-  distributed_manager = ags_sound_provider_get_distributed_manager(AGS_SOUND_PROVIDER(application_context));
+  sound_server = ags_sound_provider_get_sound_server(AGS_SOUND_PROVIDER(application_context));
 
-  if((distributed_manager = ags_list_util_find_type(distributed_manager,
+  if((sound_server = ags_list_util_find_type(sound_server,
 						    server_type)) != NULL){
     if(use_core_audio){
       server = 
-	core_audio_server = AGS_CORE_AUDIO_SERVER(distributed_manager->data);
+	core_audio_server = AGS_CORE_AUDIO_SERVER(sound_server->data);
     }else if(use_pulse){
       server = 
-	pulse_server = AGS_PULSE_SERVER(distributed_manager->data);
+	pulse_server = AGS_PULSE_SERVER(sound_server->data);
     }else if(use_jack){
       server = 
-	jack_server = AGS_JACK_SERVER(distributed_manager->data);
+	jack_server = AGS_JACK_SERVER(sound_server->data);
     }
   }else{
-    g_warning("distributed manager not found");
-
-    pthread_mutex_unlock(application_mutex);
+    g_warning("sound server not found");
     
     return;
   }
@@ -1293,8 +1365,6 @@ ags_soundcard_editor_remove_sink(AgsSoundcardEditor *soundcard_editor,
     list = list->next;
   }
 
-  pthread_mutex_unlock(application_mutex);
-
   if(soundcard == NULL){
     return;
   }
@@ -1316,8 +1386,8 @@ ags_soundcard_editor_remove_sink(AgsSoundcardEditor *soundcard_editor,
     machine = machine->next;
   }
 
-  ags_soundcard_set_audio(AGS_SOUNDCARD(soundcard),
-			  NULL);
+  ags_sound_provider_set_audio(AGS_SOUND_PROVIDER(application_context),
+			       NULL);
   g_list_free(machine_start);
 #endif
   
@@ -1328,7 +1398,7 @@ ags_soundcard_editor_remove_sink(AgsSoundcardEditor *soundcard_editor,
 
 #if 0
   if(server != NULL){
-    ags_distributed_manager_unregister_soundcard(AGS_DISTRIBUTED_MANAGER(server),
+    ags_sound_server_unregister_soundcard(AGS_SOUND_SERVER(server),
 						 soundcard);
   }
 #endif
@@ -1385,8 +1455,6 @@ ags_soundcard_editor_add_soundcard(AgsSoundcardEditor *soundcard_editor,
   AgsApplicationContext *application_context;
 
   gboolean initial_soundcard;
-  
-  pthread_mutex_t *application_mutex;
 
   if(soundcard == NULL ||
      AGS_IS_CORE_AUDIO_DEVOUT(soundcard) ||
@@ -1403,10 +1471,10 @@ ags_soundcard_editor_add_soundcard(AgsSoundcardEditor *soundcard_editor,
 
   preferences = (AgsPreferences *) gtk_widget_get_ancestor(GTK_WIDGET(soundcard_editor),
 							   AGS_TYPE_PREFERENCES);
+
   window = AGS_WINDOW(preferences->window);
   
   application_context = (AgsApplicationContext *) window->application_context;
-  application_mutex = window->application_mutex;
 
   initial_soundcard = FALSE;
   
@@ -1425,14 +1493,12 @@ ags_soundcard_editor_add_soundcard(AgsSoundcardEditor *soundcard_editor,
   }
   
   /*  */
-  pthread_mutex_lock(application_mutex);
-
-  main_loop = (AgsThread *) application_context->main_loop;
+  g_object_get(application_context,
+	       "main-loop", &main_loop,
+	       NULL);
 
   if(g_list_find(ags_sound_provider_get_soundcard(AGS_SOUND_PROVIDER(application_context)),
 		 soundcard) != NULL){
-    pthread_mutex_unlock(application_mutex);  
-  
     soundcard_editor->flags &= (~AGS_SOUNDCARD_EDITOR_BLOCK_ADD);
     
     return;
@@ -1459,11 +1525,11 @@ ags_soundcard_editor_add_soundcard(AgsSoundcardEditor *soundcard_editor,
     while(machine != NULL){
       g_object_ref(G_OBJECT(AGS_MACHINE(machine->data)->audio));
   
-      list = ags_soundcard_get_audio(AGS_SOUNDCARD(soundcard));
+      list = ags_sound_provider_get_audio(AGS_SOUND_PROVIDER(application_context));
       list = g_list_prepend(list,
 			    AGS_MACHINE(machine->data)->audio);
-      ags_soundcard_set_audio(AGS_SOUNDCARD(soundcard),
-			      list);
+      ags_sound_provider_set_audio(AGS_SOUND_PROVIDER(application_context),
+				   list);
 
       g_object_set(AGS_MACHINE(machine->data)->audio,
 		   "soundcard", soundcard,
@@ -1476,11 +1542,10 @@ ags_soundcard_editor_add_soundcard(AgsSoundcardEditor *soundcard_editor,
     g_list_free(machine_start);
   }
   
-  pthread_mutex_unlock(application_mutex);  
-  
   g_object_ref(soundcard);
 
-  soundcard_thread = (AgsThread *) ags_soundcard_thread_new(soundcard);
+  soundcard_thread = (AgsThread *) ags_soundcard_thread_new(soundcard,
+							    ags_soundcard_get_capability(AGS_SOUNDCARD(soundcard)));
   soundcard_editor->soundcard_thread = (GObject *) soundcard_thread;
   
   ags_thread_add_child_extended(main_loop,
@@ -1528,8 +1593,6 @@ ags_soundcard_editor_remove_soundcard(AgsSoundcardEditor *soundcard_editor,
 
   GList *machine, *machine_start;
 
-  pthread_mutex_t *application_mutex;
-
   if(soundcard == NULL ||
      AGS_IS_CORE_AUDIO_DEVOUT(soundcard) ||
      AGS_IS_PULSE_DEVOUT(soundcard) ||
@@ -1539,14 +1602,14 @@ ags_soundcard_editor_remove_soundcard(AgsSoundcardEditor *soundcard_editor,
   
   preferences = (AgsPreferences *) gtk_widget_get_ancestor(GTK_WIDGET(soundcard_editor),
 							   AGS_TYPE_PREFERENCES);
+
   window = AGS_WINDOW(preferences->window);
+
   application_context = (AgsApplicationContext *) window->application_context;  
 
-  application_mutex = window->application_mutex;
-  
-  pthread_mutex_lock(application_mutex);
-
-  main_loop = (AgsThread *) application_context->main_loop;
+  g_object_get(application_context,
+	       "main-loop", &main_loop,
+	       NULL);
 
   /* remove AgsSoundcard from AgsAudio */
 #if 0
@@ -1565,8 +1628,8 @@ ags_soundcard_editor_remove_soundcard(AgsSoundcardEditor *soundcard_editor,
     machine = machine->next;
   }
 
-  ags_soundcard_set_audio(AGS_SOUNDCARD(soundcard),
-			  NULL);
+  ags_sound_provider_set_audio(AGS_SOUND_PROVIDER(application_context),
+			       NULL);
   g_list_free(machine_start);
 #endif
   
@@ -1597,8 +1660,6 @@ ags_soundcard_editor_remove_soundcard(AgsSoundcardEditor *soundcard_editor,
   }
 #endif
 
-  pthread_mutex_unlock(application_mutex);
-
   /* notify user about restarting GSequencer */
   dialog = gtk_message_dialog_new(preferences,
 				  GTK_DIALOG_MODAL,
@@ -1620,28 +1681,22 @@ ags_soundcard_editor_load_core_audio_card(AgsSoundcardEditor *soundcard_editor)
 
   AgsApplicationContext *application_context;
 
-  GList *distributed_manager;
+  GList *sound_server;
   GList *soundcard;
   GList *card_id;
 
-  pthread_mutex_t *application_mutex;
-
   preferences = (AgsPreferences *) gtk_widget_get_ancestor(GTK_WIDGET(soundcard_editor),
 							   AGS_TYPE_PREFERENCES);
+
   window = AGS_WINDOW(preferences->window);
+
   application_context = (AgsApplicationContext *) window->application_context;
 
-  application_mutex = window->application_mutex;
-
   /* create soundcard */
-  pthread_mutex_lock(application_mutex);
+  sound_server = ags_sound_provider_get_sound_server(AGS_SOUND_PROVIDER(application_context));
 
-  distributed_manager = ags_sound_provider_get_distributed_manager(AGS_SOUND_PROVIDER(application_context));
-
-  if(distributed_manager == NULL){
-    g_warning("distributed manager not found");
-
-    pthread_mutex_unlock(application_mutex);
+  if(sound_server == NULL){
+    g_warning("sound server not found");
 
     return;
   }
@@ -1679,7 +1734,6 @@ ags_soundcard_editor_load_core_audio_card(AgsSoundcardEditor *soundcard_editor)
   
   gtk_widget_set_sensitive((GtkWidget *) soundcard_editor->samplerate,
 			   FALSE);
-  pthread_mutex_unlock(application_mutex);
 }
 
 void
@@ -1692,28 +1746,22 @@ ags_soundcard_editor_load_pulse_card(AgsSoundcardEditor *soundcard_editor)
 
   AgsApplicationContext *application_context;
 
-  GList *distributed_manager;
+  GList *sound_server;
   GList *soundcard;
   GList *card_id;
 
-  pthread_mutex_t *application_mutex;
-
   preferences = (AgsPreferences *) gtk_widget_get_ancestor(GTK_WIDGET(soundcard_editor),
 							   AGS_TYPE_PREFERENCES);
+
   window = AGS_WINDOW(preferences->window);
+
   application_context = (AgsApplicationContext *) window->application_context;
 
-  application_mutex = window->application_mutex;
-
   /* create soundcard */
-  pthread_mutex_lock(application_mutex);
+  sound_server = ags_sound_provider_get_sound_server(AGS_SOUND_PROVIDER(application_context));
 
-  distributed_manager = ags_sound_provider_get_distributed_manager(AGS_SOUND_PROVIDER(application_context));
-
-  if(distributed_manager == NULL){
-    g_warning("distributed manager not found");
-
-    pthread_mutex_unlock(application_mutex);
+  if(sound_server == NULL){
+    g_warning("sound server not found");
 
     return;
   }
@@ -1751,7 +1799,6 @@ ags_soundcard_editor_load_pulse_card(AgsSoundcardEditor *soundcard_editor)
   
   gtk_widget_set_sensitive((GtkWidget *) soundcard_editor->samplerate,
 			   FALSE);
-  pthread_mutex_unlock(application_mutex);
 }
 
 void
@@ -1764,28 +1811,22 @@ ags_soundcard_editor_load_jack_card(AgsSoundcardEditor *soundcard_editor)
 
   AgsApplicationContext *application_context;
 
-  GList *distributed_manager;
+  GList *sound_server;
   GList *soundcard;
   GList *card_id;
 
-  pthread_mutex_t *application_mutex;
-
   preferences = (AgsPreferences *) gtk_widget_get_ancestor(GTK_WIDGET(soundcard_editor),
 							   AGS_TYPE_PREFERENCES);
+
   window = AGS_WINDOW(preferences->window);
+
   application_context = (AgsApplicationContext *) window->application_context;
 
-  application_mutex = window->application_mutex;
-
   /* create soundcard */
-  pthread_mutex_lock(application_mutex);
+  sound_server = ags_sound_provider_get_sound_server(AGS_SOUND_PROVIDER(application_context));
 
-  distributed_manager = ags_sound_provider_get_distributed_manager(AGS_SOUND_PROVIDER(application_context));
-
-  if(distributed_manager == NULL){
-    g_warning("distributed manager not found");
-
-    pthread_mutex_unlock(application_mutex);
+  if(sound_server == NULL){
+    g_warning("sound server not found");
 
     return;
   }
@@ -1823,7 +1864,6 @@ ags_soundcard_editor_load_jack_card(AgsSoundcardEditor *soundcard_editor)
   
   gtk_widget_set_sensitive((GtkWidget *) soundcard_editor->samplerate,
 			   FALSE);
-  pthread_mutex_unlock(application_mutex);
 }
 
 void
@@ -1839,8 +1879,6 @@ ags_soundcard_editor_load_alsa_card(AgsSoundcardEditor *soundcard_editor)
   GList *list;
   GList *card_id;
 
-  pthread_mutex_t *application_mutex;
-
   if((AGS_SOUNDCARD_EDITOR_BLOCK_LOAD & (soundcard_editor->flags)) != 0){
     return;
   }
@@ -1849,14 +1887,12 @@ ags_soundcard_editor_load_alsa_card(AgsSoundcardEditor *soundcard_editor)
 
   preferences = (AgsPreferences *) gtk_widget_get_ancestor(GTK_WIDGET(soundcard_editor),
 							   AGS_TYPE_PREFERENCES);
+
   window = AGS_WINDOW(preferences->window);
+
   application_context = (AgsApplicationContext *) window->application_context;
 
-  application_mutex = window->application_mutex;
-  
   /*  */
-  pthread_mutex_lock(application_mutex);
-
   devout = g_object_new(AGS_TYPE_DEVOUT,
 			NULL);
   devout->flags &= (~AGS_DEVOUT_OSS);
@@ -1903,8 +1939,6 @@ ags_soundcard_editor_load_alsa_card(AgsSoundcardEditor *soundcard_editor)
   gtk_widget_set_sensitive((GtkWidget *) soundcard_editor->samplerate,
 			   TRUE);
 
-  pthread_mutex_unlock(application_mutex);
-
   soundcard_editor->flags &= (~AGS_SOUNDCARD_EDITOR_BLOCK_LOAD);
 }
 
@@ -1920,19 +1954,15 @@ ags_soundcard_editor_load_oss_card(AgsSoundcardEditor *soundcard_editor)
 
   GList *list;
   GList *card_id;
-
-  pthread_mutex_t *application_mutex;
   
   preferences = (AgsPreferences *) gtk_widget_get_ancestor(GTK_WIDGET(soundcard_editor),
 							   AGS_TYPE_PREFERENCES);
+
   window = AGS_WINDOW(preferences->window);
+
   application_context = (AgsApplicationContext *) window->application_context;
-
-  application_mutex = window->application_mutex;
   
-  /*  */  
-  pthread_mutex_lock(application_mutex);
-
+  /*  */
   devout = g_object_new(AGS_TYPE_DEVOUT,
 			NULL);
   devout->flags &= (~AGS_DEVOUT_ALSA);
@@ -1978,18 +2008,16 @@ ags_soundcard_editor_load_oss_card(AgsSoundcardEditor *soundcard_editor)
 
   gtk_widget_set_sensitive((GtkWidget *) soundcard_editor->samplerate,
 			   TRUE);
-
-  pthread_mutex_unlock(application_mutex);
 }
 
 /**
  * ags_soundcard_editor_new:
  *
- * Creates an #AgsSoundcardEditor
+ * Create a new instance of #AgsSoundcardEditor
  *
- * Returns: a new #AgsSoundcardEditor
+ * Returns: the new #AgsSoundcardEditor
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 AgsSoundcardEditor*
 ags_soundcard_editor_new()
@@ -2001,4 +2029,3 @@ ags_soundcard_editor_new()
   
   return(soundcard_editor);
 }
-
