@@ -1,5 +1,5 @@
 /* GSequencer - Advanced GTK Sequencer
- * Copyright (C) 2005-2017 Joël Krähemann
+ * Copyright (C) 2005-2018 Joël Krähemann
  *
  * This file is part of GSequencer.
  *
@@ -59,7 +59,7 @@ ags_automation_edit_box_get_type(void)
   static volatile gsize g_define_type_id__volatile = 0;
 
   if(g_once_init_enter (&g_define_type_id__volatile)){
-    GType ags_type_automation_edit_box;
+    GType ags_type_automation_edit_box = 0;
 
     static const GTypeInfo ags_automation_edit_box_info = {
       sizeof (AgsAutomationEditBoxClass),
@@ -77,7 +77,7 @@ ags_automation_edit_box_get_type(void)
 							  "AgsAutomationEditBox", &ags_automation_edit_box_info,
 							  0);
 
-    g_once_init_leave (&g_define_type_id__volatile, ags_type_automation_edit_box);
+    g_once_init_leave(&g_define_type_id__volatile, ags_type_automation_edit_box);
   }
 
   return g_define_type_id__volatile;
@@ -108,7 +108,7 @@ ags_automation_edit_box_class_init(AgsAutomationEditBoxClass *automation_edit_bo
    *
    * The fixed height of a edit.
    * 
-   * Since: 1.3.0
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_uint("fixed-edit-height",
 				 "fixed edit height",
@@ -207,7 +207,7 @@ GType ags_automation_edit_box_child_type(GtkContainer *container)
  * 
  * Returns: the new #AgsAutomationEditBox instance
  * 
- * Since: 1.3.0
+ * Since: 2.0.0
  */
 AgsAutomationEditBox*
 ags_automation_edit_box_new()
