@@ -62,10 +62,12 @@ ags_ramp_acceleration_dialog_port_callback(GtkComboBox *combo_box,
   AgsMachine *machine;
 
   AgsAudio *audio;
+  AgsChannel *channel;
   
+  AgsPluginPort *plugin_port;
+
+  GList *start_port, *port;
 
-  GList *start_list_automation, *list_automation;
-  
   gchar *specifier;
 
   window = AGS_WINDOW(ramp_acceleration_dialog->main_window);
@@ -79,53 +81,155 @@ ags_ramp_acceleration_dialog_port_callback(GtkComboBox *combo_box,
   
   audio = machine->audio;
 
-  g_object_get(audio,
-	       "automation", &start_list_automation,
-	       NULL);
-  
   /* specifier */
-  list_automation = NULL;
-  
   specifier = gtk_combo_box_text_get_active_text(ramp_acceleration_dialog->port);
 
-  if(specifier != NULL &&
-     strlen(specifier) > 0){
-    list_automation = ags_automation_find_specifier(start_list_automation,
-						    specifier);
+  switch(gtk_notebook_get_current_page(automation_editor->notebook)){
+  case 0:
+    {
+      port = ags_audio_collect_all_audio_ports_by_specifier_and_context(machine->audio,
+									specifier,
+									TRUE);
+      start_port = port;
+      
+      port = ags_audio_collect_all_audio_ports_by_specifier_and_context(machine->audio,
+									specifier,
+									FALSE);
+
+      if(start_port != NULL){
+	start_port = g_list_concat(start_port,
+				   port);
+      }else{
+	start_port = port;
+      }
+
+      port = start_port;
+    }
+    break;
+  case 1:
+    {
+      g_object_get(machine->audio,
+		   "output", &channel,
+		   NULL);
+
+      start_port = NULL;
+      
+      while(channel != NULL){
+	port = ags_channel_collect_all_channel_ports_by_specifier_and_context(channel,
+									      specifier,
+									      TRUE);
+	if(start_port != NULL){
+	  start_port = g_list_concat(start_port,
+				     port);
+	}else{
+	  start_port = port;
+	}
+      
+	port = ags_channel_collect_all_channel_ports_by_specifier_and_context(channel,
+									      specifier,
+									      FALSE);
+
+	if(start_port != NULL){
+	  start_port = g_list_concat(start_port,
+				     port);
+	}else{
+	  start_port = port;
+	}
+
+	/* iterate */
+	g_object_get(channel,
+		     "next", &channel,
+		     NULL);
+      }
+    }
+    break;
+  case 2:
+    {
+      g_object_get(machine->audio,
+		   "input", &channel,
+		   NULL);
+
+      start_port = NULL;
+      
+      while(channel != NULL){
+	port = ags_channel_collect_all_channel_ports_by_specifier_and_context(channel,
+									      specifier,
+									      TRUE);
+	if(start_port != NULL){
+	  start_port = g_list_concat(start_port,
+				     port);
+	}else{
+	  start_port = port;
+	}
+      
+	port = ags_channel_collect_all_channel_ports_by_specifier_and_context(channel,
+									      specifier,
+									      FALSE);
+
+	if(start_port != NULL){
+	  start_port = g_list_concat(start_port,
+				     port);
+	}else{
+	  start_port = port;
+	}
+
+	/* iterate */
+	g_object_get(channel,
+		     "next", &channel,
+		     NULL);
+      }
+    }
+    break;
   }
-    
+  
   /* reset range */
-  if(list_automation != NULL){
-    AgsAutomation *automation;
-
-    guint steps;
-    gdouble lower, upper;
+  plugin_port = NULL;
     
-    automation = AGS_AUTOMATION(list_automation->data);
+  if(start_port != NULL){
+    g_object_get(start_port->data,
+		 "plugin-port", &plugin_port,
+		 NULL);      
 
-    g_object_get(automation,
-		 "lower", &lower,
-		 "upper", &upper,
-		 "steps", &steps,
+    g_list_free(start_port);
+  }
+
+  if(plugin_port != NULL){
+    gint scale_steps;
+    gdouble lower, upper;
+
+    GValue *upper_value, *lower_value;
+    
+    /* get some fields */
+    g_object_get(plugin_port,
+		 "upper-value", &upper_value,
+		 "lower-value", &lower_value,
+		 "scale-steps", &scale_steps,
 		 NULL);
     
+    upper = g_value_get_float(upper_value);
+    lower = g_value_get_float(lower_value);
+
+    if(scale_steps == -1){
+      scale_steps = AGS_AUTOMATION_EDIT_DEFAULT_HEIGHT;
+    }
+    
     gtk_spin_button_set_range(ramp_acceleration_dialog->ramp_y0,
 			      lower,
 			      upper);
     gtk_spin_button_set_increments(ramp_acceleration_dialog->ramp_y0,
-				   (upper - lower) / steps,
-				   (upper - lower) / steps);
+				   (upper - lower) / scale_steps,
+				   (upper - lower) / scale_steps);
     
     gtk_spin_button_set_range(ramp_acceleration_dialog->ramp_y1,
 			      lower,
 			      upper);
     gtk_spin_button_set_increments(ramp_acceleration_dialog->ramp_y1,
-				   (upper - lower) / steps,
-				   (upper - lower) / steps);
+				   (upper - lower) / scale_steps,
+				   (upper - lower) / scale_steps);
     
     gtk_spin_button_set_range(ramp_acceleration_dialog->ramp_step_count,
 			      0.0,
-			      steps);
+			      scale_steps);
   }else{
     gtk_spin_button_set_range(ramp_acceleration_dialog->ramp_y0,
 			      0.0,
@@ -145,8 +249,6 @@ ags_ramp_acceleration_dialog_port_callback(GtkComboBox *combo_box,
 			      0.0,
 			      0.0);
   }
-
-  g_list_free(start_list_automation);
 }
 
 void
