@@ -43,9 +43,11 @@ void ags_ladspa_bridge_get_property(GObject *gobject,
 				    guint prop_id,
 				    GValue *value,
 				    GParamSpec *param_spec);
+void ags_ladspa_bridge_finalize(GObject *gobject);
+
 void ags_ladspa_bridge_connect(AgsConnectable *connectable);
 void ags_ladspa_bridge_disconnect(AgsConnectable *connectable);
-void ags_ladspa_bridge_finalize(GObject *gobject);
+
 gchar* ags_ladspa_bridge_get_version(AgsPlugin *plugin);
 void ags_ladspa_bridge_set_version(AgsPlugin *plugin, gchar *version);
 gchar* ags_ladspa_bridge_get_build_id(AgsPlugin *plugin);
@@ -84,7 +86,7 @@ ags_ladspa_bridge_get_type(void)
   static volatile gsize g_define_type_id__volatile = 0;
 
   if(g_once_init_enter (&g_define_type_id__volatile)){
-    GType ags_type_ladspa_bridge;
+    GType ags_type_ladspa_bridge = 0;
 
     static const GTypeInfo ags_ladspa_bridge_info = {
       sizeof(AgsLadspaBridgeClass),
@@ -122,7 +124,7 @@ ags_ladspa_bridge_get_type(void)
 				AGS_TYPE_PLUGIN,
 				&ags_plugin_interface_info);
 
-    g_once_init_leave (&g_define_type_id__volatile, ags_type_ladspa_bridge);
+    g_once_init_leave(&g_define_type_id__volatile, ags_type_ladspa_bridge);
   }
 
   return g_define_type_id__volatile;
@@ -150,7 +152,7 @@ ags_ladspa_bridge_class_init(AgsLadspaBridgeClass *ladspa_bridge)
    *
    * The plugins filename.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   param_spec =  g_param_spec_string("filename",
 				    i18n_pspec("the object file"),
@@ -166,7 +168,7 @@ ags_ladspa_bridge_class_init(AgsLadspaBridgeClass *ladspa_bridge)
    *
    * The effect's name.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   param_spec =  g_param_spec_string("effect",
 				    i18n_pspec("the effect"),
@@ -182,7 +184,7 @@ ags_ladspa_bridge_class_init(AgsLadspaBridgeClass *ladspa_bridge)
    *
    * The effect's index.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   param_spec =  g_param_spec_ulong("index",
 				   i18n_pspec("index of effect"),
@@ -235,7 +237,7 @@ ags_ladspa_bridge_init(AgsLadspaBridge *ladspa_bridge)
 			 G_CALLBACK(ags_ladspa_bridge_parent_set_callback), (gpointer) ladspa_bridge);
 
   audio = AGS_MACHINE(ladspa_bridge)->audio;
-  audio->flags |= (AGS_AUDIO_SYNC);
+  ags_audio_set_flags(audio, (AGS_AUDIO_SYNC));
   
   ladspa_bridge->flags = 0;
 
@@ -385,6 +387,20 @@ ags_ladspa_bridge_get_property(GObject *gobject,
   }
 }
 
+void
+ags_ladspa_bridge_finalize(GObject *gobject)
+{
+  AgsLadspaBridge *ladspa_bridge;
+
+  ladspa_bridge = (AgsLadspaBridge *) gobject;
+  
+  g_free(ladspa_bridge->filename);
+  g_free(ladspa_bridge->effect);
+
+  /* call parent */
+  G_OBJECT_CLASS(ags_ladspa_bridge_parent_class)->finalize(gobject);
+}
+
 void
 ags_ladspa_bridge_connect(AgsConnectable *connectable)
 {
@@ -405,20 +421,6 @@ ags_ladspa_bridge_disconnect(AgsConnectable *connectable)
   ags_ladspa_bridge_parent_connectable_interface->disconnect(connectable);
 }
 
-void
-ags_ladspa_bridge_finalize(GObject *gobject)
-{
-  AgsLadspaBridge *ladspa_bridge;
-
-  ladspa_bridge = (AgsLadspaBridge *) gobject;
-  
-  g_free(ladspa_bridge->filename);
-  g_free(ladspa_bridge->effect);
-
-  /* call parent */
-  G_OBJECT_CLASS(ags_ladspa_bridge_parent_class)->finalize(gobject);
-}
-
 gchar*
 ags_ladspa_bridge_get_version(AgsPlugin *plugin)
 {
@@ -520,9 +522,9 @@ ags_ladspa_bridge_launch_task(AgsFileLaunch *file_launch, AgsLadspaBridge *ladsp
     recall = AGS_MACHINE(ladspa_bridge)->audio->input->recall;
     
     while((recall = ags_recall_template_find_type(recall, AGS_TYPE_RECALL_LADSPA)) != NULL){
-      if(!g_strcmp0(AGS_RECALL_LADSPA(recall->data)->filename,
+      if(!g_strcmp0(AGS_RECALL(recall->data)->filename,
 		  ladspa_bridge->filename) &&
-	 !g_strcmp0(AGS_RECALL_LADSPA(recall->data)->effect,
+	 !g_strcmp0(AGS_RECALL(recall->data)->effect,
 		    ladspa_bridge->effect)){
 	break;
       }
@@ -636,7 +638,7 @@ ags_ladspa_bridge_load(AgsLadspaBridge *ladspa_bridge)
  *
  * Returns: a new #AgsLadspaBridge
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 AgsLadspaBridge*
 ags_ladspa_bridge_new(GObject *soundcard,
@@ -644,19 +646,14 @@ ags_ladspa_bridge_new(GObject *soundcard,
 		      gchar *effect)
 {
   AgsLadspaBridge *ladspa_bridge;
-  GValue value = {0,};
 
   ladspa_bridge = (AgsLadspaBridge *) g_object_new(AGS_TYPE_LADSPA_BRIDGE,
 						   NULL);
 
-  if(soundcard != NULL){
-    g_value_init(&value, G_TYPE_OBJECT);
-    g_value_set_object(&value, soundcard);
-    g_object_set_property(G_OBJECT(AGS_MACHINE(ladspa_bridge)->audio),
-			  "soundcard", &value);
-    g_value_unset(&value);
-  }
-
+  g_object_set(AGS_MACHINE(ladspa_bridge)->audio,
+	       "output-soundcard", soundcard,
+	       NULL);
+  
   g_object_set(ladspa_bridge,
 	       "filename", filename,
 	       "effect", effect,
