@@ -1,5 +1,5 @@
 /* GSequencer - Advanced GTK Sequencer
- * Copyright (C) 2005-2017 Joël Krähemann
+ * Copyright (C) 2005-2018 Joël Krähemann
  *
  * This file is part of GSequencer.
  *
@@ -19,21 +19,17 @@
 
 #include <ags/audio/ags_input.h>
 
-#include <ags/object/ags_application_context.h>
-#include <ags/object/ags_connectable.h>
+#include <ags/libags.h>
 
-#include <ags/thread/ags_mutex_manager.h>
-
-#include <ags/file/ags_file_link.h>
-
-#include <ags/audio/ags_playable.h>
 #include <ags/audio/ags_recycling.h>
 #include <ags/audio/ags_audio_signal.h>
 #include <ags/audio/ags_synth_generator.h>
 
 #include <ags/audio/file/ags_audio_file_link.h>
+#include <ags/audio/file/ags_audio_container.h>
 #include <ags/audio/file/ags_audio_file.h>
-#include <ags/audio/file/ags_ipatch.h>
+#include <ags/audio/file/ags_sound_container.h>
+#include <ags/audio/file/ags_sound_resource.h>
 
 #include <ags/i18n.h>
 
@@ -80,7 +76,7 @@ ags_input_get_type (void)
   static volatile gsize g_define_type_id__volatile = 0;
 
   if(g_once_init_enter (&g_define_type_id__volatile)){
-    GType ags_type_input;
+    GType ags_type_input = 0;
 
     static const GTypeInfo ags_input_info = {
       sizeof (AgsInputClass),
@@ -109,7 +105,7 @@ ags_input_get_type (void)
 				AGS_TYPE_CONNECTABLE,
 				&ags_connectable_interface_info);
 
-    g_once_init_leave (&g_define_type_id__volatile, ags_type_input);
+    g_once_init_leave(&g_define_type_id__volatile, ags_type_input);
   }
 
   return g_define_type_id__volatile;
@@ -137,7 +133,7 @@ ags_input_class_init(AgsInputClass *input)
    *
    * The file containing audio data.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_object("file-link",
 				   i18n_pspec("file link assigned to"),
@@ -151,15 +147,14 @@ ags_input_class_init(AgsInputClass *input)
   /**
    * AgsInput:synth-generator:
    *
-   * An optional synth generator that might be used.
+   * The synth generators assigned with this input.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
-  param_spec = g_param_spec_object("synth-generator",
-				   i18n_pspec("the synth generator"),
-				   i18n_pspec("The synth generator to be used"),
-				   AGS_TYPE_SYNTH_GENERATOR,
-				   G_PARAM_READABLE | G_PARAM_WRITABLE);
+  param_spec = g_param_spec_pointer("synth-generator",
+				    i18n_pspec("the synth generator"),
+				    i18n_pspec("The synth generator to be used"),
+				    G_PARAM_READABLE | G_PARAM_WRITABLE);
   g_object_class_install_property(gobject,
 				  PROP_SYNTH_GENERATOR,
 				  param_spec);
@@ -180,6 +175,7 @@ void
 ags_input_init(AgsInput *input)
 {
   input->file_link = NULL;
+
   input->synth_generator = NULL;
 }
 
@@ -217,23 +213,15 @@ ags_input_set_property(GObject *gobject,
     break;
   case PROP_SYNTH_GENERATOR:
     {
-      AgsFileLink *synth_generator;
+      AgsSynthGenerator *synth_generator;
 
-      synth_generator = (AgsFileLink *) g_value_get_object(value);
+      synth_generator = (AgsSynthGenerator *) g_value_get_pointer(value);
 
-      if(input->synth_generator == (GObject *) synth_generator){
+      if(g_list_find(input->synth_generator, synth_generator) != NULL){
 	return;
       }
       
-      if(input->synth_generator != NULL){
-	g_object_unref(G_OBJECT(input->synth_generator));
-      }
-
-      if(synth_generator != NULL){
-	g_object_ref(G_OBJECT(synth_generator));
-      }
-
-      input->synth_generator = (GObject *) synth_generator;
+      ags_input_add_synth_generator(input, synth_generator);
     }
     break;
   default:
@@ -260,7 +248,7 @@ ags_input_get_property(GObject *gobject,
     break;
   case PROP_SYNTH_GENERATOR:
     {
-      g_value_set_object(value, input->synth_generator);
+      g_value_set_pointer(value, g_list_copy(input->synth_generator));
     }
     break;
   default:
@@ -274,6 +262,8 @@ ags_input_dispose(GObject *gobject)
 {
   AgsInput *input;
 
+  GList *list, *list_next;
+  
   input = AGS_INPUT(gobject);
 
   /* file link */
@@ -287,9 +277,18 @@ ags_input_dispose(GObject *gobject)
 
   /* synth generator */
   if(input->synth_generator != NULL){
-    g_object_run_dispose(G_OBJECT(input->synth_generator));
+    list = input->synth_generator;
+    
+    while(list != NULL){
+      list_next = list->next;
 
-    g_object_unref(G_OBJECT(input->synth_generator));
+      g_object_run_dispose(list->data);
+
+      list = list_next;
+    }
+
+    g_list_free_full(input->synth_generator,
+		     g_object_unref);
     
     input->synth_generator = NULL;
   }
@@ -311,9 +310,8 @@ ags_input_finalize(GObject *gobject)
   }
 
   /* synth generator */
-  if(input->synth_generator != NULL){
-    g_object_unref(G_OBJECT(input->synth_generator));
-  }
+  g_list_free_full(input->synth_generator,
+		   g_object_unref);
 
   /* finalize */
   G_OBJECT_CLASS(ags_input_parent_class)->finalize(gobject);
@@ -331,6 +329,376 @@ ags_input_disconnect(AgsConnectable *connectable)
   ags_input_parent_connectable_interface->disconnect(connectable);
 }
 
+/**
+ * ags_input_is_active:
+ * @input: the #AgsInput
+ * @recycling_context: the #AgsRecyclingContext to check
+ * 
+ * Check if @input is active and needs processing.
+ * 
+ * Returns: %TRUE if has a need to be processed, else %FALSE
+ * 
+ * Since: 2.0.0
+ */
+gboolean
+ags_input_is_active(AgsInput *input,
+		    GObject *recycling_context)
+{
+  AgsChannel *channel;
+  AgsRecycling *first_recycling, *last_recycling;
+  AgsRecycling *recycling, *end_region;
+  AgsAudioSignal *audio_signal;
+  AgsRecyclingContext *active_context;
+  
+  GList *list_start, *list;
+  
+  pthread_mutex_t *channel_mutex;
+  pthread_mutex_t *recycling_mutex;
+  pthread_mutex_t *audio_signal_mutex;
+  
+  if(!AGS_IS_INPUT(input) ||
+     !AGS_IS_RECYCLING_CONTEXT(recycling_context)){
+    return(FALSE);
+  }
+  
+  channel = input;
+
+  /* get channel mutex */
+  pthread_mutex_lock(ags_channel_get_class_mutex());
+  
+  channel_mutex = channel->obj_mutex;
+  
+  pthread_mutex_unlock(ags_channel_get_class_mutex());
+    
+  /* get recycling */
+  pthread_mutex_lock(channel_mutex);
+  
+  first_recycling = channel->first_recycling;
+  last_recycling = channel->last_recycling;
+
+  pthread_mutex_unlock(channel_mutex);
+  
+  if(first_recycling == NULL){
+    return(FALSE);
+  }
+
+  /* get active recycling context */
+  active_context = recycling_context;
+  
+  /* get recycling mutex */
+  pthread_mutex_lock(ags_recycling_get_class_mutex());
+  
+  recycling_mutex = last_recycling->obj_mutex;
+  
+  pthread_mutex_unlock(ags_recycling_get_class_mutex());
+
+  /* get end region */  
+  pthread_mutex_lock(recycling_mutex);
+
+  end_region = last_recycling->next;
+
+  pthread_mutex_lock(recycling_mutex);
+
+  /* check if active */
+  recycling = first_recycling;
+
+  while(recycling != end_region){
+    /* get recycling mutex */
+    pthread_mutex_lock(ags_recycling_get_class_mutex());
+  
+    recycling_mutex = recycling->obj_mutex;
+  
+    pthread_mutex_unlock(ags_recycling_get_class_mutex());
+    
+    /* get audio signal */
+    pthread_mutex_lock(recycling_mutex);
+
+    list = 
+      list_start = g_list_copy(recycling->audio_signal);
+
+    pthread_mutex_unlock(recycling_mutex);  
+
+    while(list != NULL){
+      AgsRecallID *current_recall_id;
+      AgsRecyclingContext *current_recycling_context;
+
+      pthread_mutex_t *current_recall_id_mutex;
+
+      audio_signal = list->data;
+
+      /* get audio signal mutex */
+      pthread_mutex_lock(ags_audio_signal_get_class_mutex());
+  
+      audio_signal_mutex = audio_signal->obj_mutex;
+  
+      pthread_mutex_unlock(ags_audio_signal_get_class_mutex());
+
+      /* get recall id */
+      pthread_mutex_lock(audio_signal_mutex);
+      
+      current_recall_id = audio_signal->recall_id;
+
+      pthread_mutex_unlock(audio_signal_mutex);
+      
+      if(current_recall_id == NULL){
+	list = list->next;
+	
+	continue; 
+      }
+
+      /* get recall id mutex */
+      pthread_mutex_lock(ags_recall_id_get_class_mutex());
+  
+      current_recall_id_mutex = current_recall_id->obj_mutex;
+  
+      pthread_mutex_unlock(ags_recall_id_get_class_mutex());
+
+      /* get recycling context */
+      pthread_mutex_lock(current_recall_id_mutex);
+      
+      current_recycling_context = current_recall_id->recycling_context;
+
+      pthread_mutex_unlock(current_recall_id_mutex);      
+      
+      if(current_recycling_context == active_context) {
+	g_list_free(list_start);
+	
+	return(TRUE);
+      }
+
+      list = list->next;
+    }
+
+    g_list_free(list_start);
+    
+    /* iterate */
+    pthread_mutex_lock(recycling_mutex);
+
+    recycling = recycling->next;
+    
+    pthread_mutex_unlock(recycling_mutex);  
+  }
+  
+  return(FALSE);
+}
+
+/**
+ * ags_input_next_active:
+ * @input: the #AgsInput
+ * @recycling_context: the #AgsRecyclingContext to check
+ * 
+ * Find next #AgsInput needed to be processed.
+ * 
+ * Returns: next active #AgsInput, else %NULL if non available
+ * 
+ * Since: 2.0.0
+ */
+AgsInput*
+ags_input_next_active(AgsInput *input,
+		      GObject *recycling_context)
+{
+  AgsChannel *channel;
+  AgsRecycling *first_recycling, *last_recycling;
+  AgsRecycling *recycling, *end_region;
+  AgsAudioSignal *audio_signal;  
+  AgsRecyclingContext *active_context;
+
+  GList *list_start, *list;
+  
+  pthread_mutex_t *channel_mutex;
+  pthread_mutex_t *recycling_mutex;
+  pthread_mutex_t *audio_signal_mutex;
+
+  if(!AGS_IS_INPUT(input) ||
+     !AGS_IS_RECYCLING_CONTEXT(recycling_context)){
+    return(NULL);
+  }
+
+  channel = input;
+
+  /* check if active */
+  while(channel != NULL){
+    /* get channel mutex */
+    pthread_mutex_lock(ags_channel_get_class_mutex());
+  
+    channel_mutex = channel->obj_mutex;
+  
+    pthread_mutex_unlock(ags_channel_get_class_mutex());
+    
+    /* get recycling */
+    pthread_mutex_lock(channel_mutex);
+  
+    first_recycling = channel->first_recycling;
+    last_recycling = channel->last_recycling;
+
+    pthread_mutex_unlock(channel_mutex);
+  
+    if(first_recycling == NULL){
+      /* iterate */
+      pthread_mutex_lock(channel_mutex);
+
+      channel = channel->next_pad;
+    
+      pthread_mutex_unlock(channel_mutex);
+      
+      continue;
+    }
+
+    /* get active recycling context */
+    active_context = recycling_context;
+  
+    /* get recycling mutex */
+    pthread_mutex_lock(ags_recycling_get_class_mutex());
+  
+    recycling_mutex = last_recycling->obj_mutex;
+  
+    pthread_mutex_unlock(ags_recycling_get_class_mutex());
+
+    /* get end region */  
+    pthread_mutex_lock(recycling_mutex);
+
+    end_region = last_recycling->next;
+
+    pthread_mutex_lock(recycling_mutex);
+    
+    /* check recycling */
+    recycling = first_recycling;
+    
+    while(recycling != end_region){
+      /* get recycling mutex */
+      pthread_mutex_lock(ags_recycling_get_class_mutex());
+  
+      recycling_mutex = recycling->obj_mutex;
+  
+      pthread_mutex_unlock(ags_recycling_get_class_mutex());
+    
+      /* get audio signal */
+      pthread_mutex_lock(recycling_mutex);
+
+      list = 
+	list_start = g_list_copy(recycling->audio_signal);
+
+      pthread_mutex_unlock(recycling_mutex);  
+
+      /* check audio signal */
+      while(list != NULL){
+	AgsRecallID *current_recall_id;
+	AgsRecyclingContext *current_recycling_context;
+
+	pthread_mutex_t *current_recall_id_mutex;
+      
+	audio_signal = list->data;
+
+	/* get audio signal mutex */
+	pthread_mutex_lock(ags_audio_signal_get_class_mutex());
+  
+	audio_signal_mutex = audio_signal->obj_mutex;
+  
+	pthread_mutex_unlock(ags_audio_signal_get_class_mutex());
+
+	/* get recall id */
+	pthread_mutex_lock(audio_signal_mutex);
+      
+	current_recall_id = audio_signal->recall_id;
+
+	pthread_mutex_unlock(audio_signal_mutex);
+      
+	if(current_recall_id == NULL){
+	  list = list->next;
+	
+	  continue; 
+	}
+
+	/* get recall id mutex */
+	pthread_mutex_lock(ags_recall_id_get_class_mutex());
+  
+	current_recall_id_mutex = current_recall_id->obj_mutex;
+  
+	pthread_mutex_unlock(ags_recall_id_get_class_mutex());
+
+	/* get recycling context */
+	pthread_mutex_lock(current_recall_id_mutex);
+      
+	current_recycling_context = current_recall_id->recycling_context;
+
+	pthread_mutex_unlock(current_recall_id_mutex);      
+      
+	if(current_recycling_context == active_context) {
+	  g_list_free(list_start);
+	
+	  return(channel);
+	}
+
+	list = list->next;
+      }
+
+      g_list_free(list_start);
+
+      /* iterate */
+      pthread_mutex_lock(recycling_mutex);
+
+      recycling = recycling->next;
+    
+      pthread_mutex_unlock(recycling_mutex);  
+    }
+
+    /* iterate */
+    pthread_mutex_lock(channel_mutex);
+
+    channel = channel->next_pad;
+    
+    pthread_mutex_unlock(channel_mutex);
+  }
+  
+  return(NULL);
+}
+
+/**
+ * ags_input_add_synth_generator:
+ * @input: the #AgsInput
+ * @synth_generator: the #AgsSynthGenerator
+ * 
+ * Add @synth_generator to @input.
+ * 
+ * Since: 2.0.0
+ */
+void
+ags_input_add_synth_generator(AgsInput *input,
+			      GObject *synth_generator)
+{
+  if(!AGS_IS_INPUT(input) ||
+     !AGS_IS_SYNTH_GENERATOR(synth_generator)){
+    return;
+  }
+
+  g_object_ref(synth_generator);
+  input->synth_generator = g_list_prepend(input->synth_generator,
+					  synth_generator);
+}
+
+/**
+ * ags_input_remove_synth_generator:
+ * @input: the #AgsInput
+ * @synth_generator: the #AgsSynthGenerator
+ * 
+ * Remove @synth_generator from @input.
+ * 
+ * Since: 2.0.0
+ */
+void
+ags_input_remove_synth_generator(AgsInput *input,
+				 GObject *synth_generator)
+{
+  if(!AGS_IS_INPUT(input) ||
+     !AGS_IS_SYNTH_GENERATOR(synth_generator)){
+    return;
+  }
+
+  input->synth_generator = g_list_remove(input->synth_generator,
+					 synth_generator);
+  g_object_unref(synth_generator);
+}
+
 /**
  * ags_input_open_file:
  * @input: the #AgsInput
@@ -345,7 +713,7 @@ ags_input_disconnect(AgsConnectable *connectable)
  * 
  * Returns: %TRUE if open was successful, else %FALSE
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 gboolean
 ags_input_open_file(AgsInput *input,
@@ -378,8 +746,8 @@ ags_input_open_file(AgsInput *input,
     
     /* open audio file and read audio signal */
     audio_file = ags_audio_file_new(filename,
-				    AGS_CHANNEL(input)->soundcard,
-				    audio_channel, 1);
+				    AGS_CHANNEL(input)->output_soundcard,
+				    audio_channel);
     
     ags_audio_file_open(audio_file);
     ags_audio_file_read_audio_signal(audio_file);
@@ -387,73 +755,26 @@ ags_input_open_file(AgsInput *input,
     audio_signal = audio_file->audio_signal;
     //    g_object_unref(audio_file);
   }else if(ags_ipatch_check_suffix(filename)){
-    AgsIpatch *ipatch;
+    AgsAudioContainer *audio_container;
 
-    AgsPlayable *playable;
-
-    GError *error;
-    
     success = TRUE;
+
+    preset = NULL;
+    instrument = NULL;
     
-    ipatch = g_object_new(AGS_TYPE_IPATCH,
-			  "soundcard", AGS_CHANNEL(input)->soundcard,
-			  "mode", AGS_IPATCH_READ,
-			  "filename", filename,
-			  NULL);
+    /* open audio container and read audio signal */
+    audio_container = ags_audio_container_new(filename,
+					      preset,
+					      instrument,
+					      sample,
+					      AGS_CHANNEL(input)->output_soundcard,
+					      audio_channel);
+    
+    ags_audio_container_open(audio_container);
+    ags_audio_container_read_audio_signal(audio_container);
 
-    playable = AGS_PLAYABLE(ipatch);
-  
-    error = NULL;
-    ags_playable_level_select(playable,
-			      0, filename,
-			      &error);
-
-    /* select first - preset */
-    ipatch->nth_level = 1;
-  
-    error = NULL;
-    ags_playable_level_select(playable,
-			      1, preset,
-			      &error);
-
-    if(error != NULL){
-      g_object_unref(ipatch);
-
-      return(FALSE);
-    }
-
-    /* select second - instrument */
-    ipatch->nth_level = 2;
-
-    error = NULL;
-    ags_playable_level_select(playable,
-			      2, instrument,
-			      &error);
-
-    if(error != NULL){
-      g_object_unref(ipatch);
-
-      return(FALSE);
-    }
-
-    /* select third - sample */
-    ipatch->nth_level = 3;
-
-    error = NULL;
-    ags_playable_level_select(playable,
-			      3, sample,
-			      &error);
-
-    if(error != NULL){
-      g_object_unref(ipatch);
-
-      return(FALSE);
-    }
-
-    /* read audio signal */
-    audio_signal = ags_playable_read_audio_signal(playable,
-						  AGS_CHANNEL(input)->soundcard,
-						  audio_channel, 1);
+    audio_signal = audio_container->audio_signal;
+    //    g_object_unref(audio_container);
   }
 
   if(success){
@@ -483,334 +804,6 @@ ags_input_open_file(AgsInput *input,
   return(success);
 }
 
-/**
- * ags_input_apply_synth:
- * @input: the #AgsInput
- * @oscillator: the oscillator to use
- * @frequency: the frequency to use
- * @phase: the phase to use
- * @volume: the volume to use
- * @n_frames: compute n_frames count of frames
- * 
- * Apply synth using specified parameters to input.
- * 
- * Returns: %TRUE if successful applied, else %FALSE
- * 
- * Since: 1.0.0
- */
-gboolean
-ags_input_apply_synth(AgsInput *input,
-		      guint oscillator,
-		      gdouble frequency,
-		      gdouble phase,
-		      gdouble volume,
-		      guint n_frames)
-{
-  return(ags_input_apply_synth_extended(input,
-					oscillator,
-					frequency,
-					phase,
-					volume,
-					n_frames,
-					0,
-					AGS_INPUT_SYNTH_BASE_NOTE,
-					NULL, NULL,
-					AGS_SYNTH_GENERATOR_COMPUTE_FIXED_LENGTH));
-}
-
-/**
- * ags_input_apply_synth_extended:
- * @input: the #AgsInput
- * @oscillator: the oscillator to use
- * @frequency: the frequency to use
- * @phase: the phase to use
- * @volume: the volume to use
- * @n_frames: compute n_frames count of frames
- * @attack: the attack to use
- * @base_note: the base note to ramp up from
- * @sync_start: safe to be %NULL
- * @sync_end: safe to be %NULL
- * @compute_flags: the flags to use to compute the synth
- * 
- * Apply synth using specified parameters to input.
- * 
- * Returns: %TRUE if successful applied, else %FALSE
- * 
- * Since: 1.0.0
- */
-gboolean
-ags_input_apply_synth_extended(AgsInput *input,
-			       guint oscillator,
-			       gdouble frequency,
-			       gdouble phase,
-			       gdouble volume,
-			       guint n_frames,
-			       guint attack,
-			       gdouble base_note,
-			       AgsComplex *sync_start, AgsComplex *sync_end,
-			       guint compute_flags)
-{
-  AgsAudioSignal *audio_signal;
-
-  if(!AGS_IS_INPUT(input)){
-    return(FALSE);
-  }
-
-  audio_signal = ags_audio_signal_get_template(AGS_CHANNEL(input)->first_recycling->audio_signal);
-
-  if(audio_signal == NULL){
-    audio_signal = g_object_new(AGS_TYPE_AUDIO_SIGNAL,
-				"soundcard", AGS_CHANNEL(input)->soundcard,
-				NULL);
-    
-    /* mark as template */
-    audio_signal->flags |= AGS_AUDIO_SIGNAL_TEMPLATE;
-      
-    ags_audio_signal_stream_resize(audio_signal,
-				   (guint) ceil((attack + n_frames) / audio_signal->buffer_size));
-
-    /* add as template */
-    ags_recycling_add_audio_signal(AGS_CHANNEL(input)->first_recycling,
-				   audio_signal);
-  }
-  
-  /* creat synth generator if needed */
-  if(input->synth_generator == NULL){
-    input->synth_generator = (GObject *) ags_synth_generator_new();
-
-    g_object_set(input->synth_generator,
-		 "samplerate", audio_signal->samplerate,
-		 "buffer-size", audio_signal->buffer_size,
-		 "format", audio_signal->format,
-		 NULL);
-  }
-
-  /* set properties */
-  g_object_set(input->synth_generator,
-	       "n-frames", n_frames,
-	       "attack", attack,
-	       "oscillator", oscillator,
-	       "frequency", frequency,
-	       "phase", phase,
-	       "volume", volume,
-	       NULL);
-
-  /* compute audio signal */
-  ags_synth_generator_compute_with_audio_signal(input->synth_generator,
-						audio_signal,
-						base_note,
-						sync_start, sync_end,
-						compute_flags);
-
-  return(TRUE);
-}
-
-/**
- * ags_input_is_active:
- * @input: the #AgsInput
- * @recycling_context: the #AgsRecyclingContext to check
- * 
- * Check if @input is active and needs processing.
- * 
- * Returns: %TRUE if has a need to be processed, else %FALSE
- * 
- * Since: 1.0.0
- */
-gboolean
-ags_input_is_active(AgsInput *input,
-		    GObject *recycling_context)
-{
-  AgsChannel *channel;
-  AgsRecycling *recycling, *end_region;
-  AgsAudioSignal *audio_signal;
-  AgsRecyclingContext *active_context;
-  
-  AgsMutexManager *mutex_manager;
-
-  GList *list;
-  
-  pthread_mutex_t *application_mutex;
-  pthread_mutex_t *input_mutex;
-  pthread_mutex_t *recycling_mutex;
-  
-  channel = input;
-
-  if(input == NULL ||
-     channel->first_recycling == NULL ||
-     recycling_context == NULL){
-    return(FALSE);
-  }
-
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
-
-  /* lookup mutex */
-  pthread_mutex_lock(application_mutex);
-  
-  input_mutex = ags_mutex_manager_lookup(mutex_manager,
-					 (GObject *) channel);
-  
-  pthread_mutex_unlock(application_mutex);
-
-  /* get recycling */
-  pthread_mutex_lock(input_mutex);
-
-  recycling = channel->first_recycling;
-  end_region = channel->last_recycling->next;
-
-  if(channel->link == NULL){
-    active_context = AGS_RECYCLING_CONTEXT(recycling_context)->parent;
-  }else{
-    active_context = recycling_context;
-  }
-  
-  pthread_mutex_unlock(input_mutex);
-
-  /* check if active */
-  while(recycling != end_region){
-    /* lookup mutex */
-    pthread_mutex_lock(application_mutex);
-  
-    recycling_mutex = ags_mutex_manager_lookup(mutex_manager,
-					       (GObject *) recycling);
-  
-    pthread_mutex_unlock(application_mutex);
-
-    pthread_mutex_lock(recycling_mutex);
-
-    list = recycling->audio_signal;
-
-    while(list != NULL){
-      audio_signal = list->data;
-      
-      if(audio_signal->recall_id != NULL &&
-	 AGS_RECALL_ID(audio_signal->recall_id)->recycling_context != NULL &&
-	 AGS_RECYCLING_CONTEXT(AGS_RECALL_ID(audio_signal->recall_id)->recycling_context)->parent == active_context){
-	pthread_mutex_unlock(recycling_mutex);
-      
-	return(TRUE);
-      }
-
-      list = list->next;
-    }
-
-    /* iterate */
-    recycling = recycling->next;
-    
-    pthread_mutex_unlock(recycling_mutex);  
-  }
-  
-  return(FALSE);
-}
-
-/**
- * ags_input_next_active:
- * @input: the #AgsInput
- * @recycling_context: the #AgsRecyclingContext to check
- * 
- * Find next #AgsInput needed to be processed.
- * 
- * Returns: next active #AgsInput, else %NULL if non available
- * 
- * Since: 1.0.0
- */
-AgsInput*
-ags_input_next_active(AgsInput *input,
-		      GObject *recycling_context)
-{
-  AgsChannel *channel;
-  AgsRecycling *recycling, *end_region;
-  AgsAudioSignal *audio_signal;  
-  AgsRecyclingContext *active_context;
-  
-  AgsMutexManager *mutex_manager;
-
-  GList *list;
-  
-  pthread_mutex_t *application_mutex;
-  pthread_mutex_t *input_mutex;
-  pthread_mutex_t *recycling_mutex;
-
-  channel = input;
-  
-  if(input == NULL ||
-     recycling_context == NULL){
-    return(NULL);
-  }
-
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
-
-  /* check if active */
-  while(channel != NULL){
-    /* lookup mutex */
-    pthread_mutex_lock(application_mutex);
-  
-    input_mutex = ags_mutex_manager_lookup(mutex_manager,
-					   (GObject *) channel);
-  
-    pthread_mutex_unlock(application_mutex);
-
-    /* get recycling */
-    pthread_mutex_lock(input_mutex);
-
-    recycling = channel->first_recycling;
-    end_region = channel->last_recycling->next;
-  
-    if(channel->link == NULL){
-      active_context = AGS_RECYCLING_CONTEXT(recycling_context)->parent;
-    }else{
-      active_context = recycling_context;
-    }
-    
-    pthread_mutex_unlock(input_mutex);
-    
-    /* check recycling */
-    while(recycling != end_region){
-      /* lookup mutex */
-      pthread_mutex_lock(application_mutex);
-  
-      recycling_mutex = ags_mutex_manager_lookup(mutex_manager,
-						 (GObject *) recycling);
-  
-      pthread_mutex_unlock(application_mutex);
-
-      /* check audio signal */
-      pthread_mutex_lock(recycling_mutex);
-
-      list = recycling->audio_signal;
-
-      while(list != NULL){
-	audio_signal = list->data;
-      
-	if(audio_signal->recall_id != NULL &&
-	   AGS_RECALL_ID(audio_signal->recall_id)->recycling_context != NULL &&
-	   AGS_RECYCLING_CONTEXT(AGS_RECALL_ID(audio_signal->recall_id)->recycling_context)->parent == active_context){
-	  pthread_mutex_unlock(recycling_mutex);
-      
-	  return(channel);
-	}
-
-	list = list->next;
-      }
-
-      /* iterate */
-      recycling = recycling->next;
-    
-      pthread_mutex_unlock(recycling_mutex);  
-    }
-
-    /* iterate */
-    pthread_mutex_lock(input_mutex);
-
-    channel = channel->next_pad;
-    
-    pthread_mutex_unlock(input_mutex);
-  }
-  
-  return(NULL);
-}
-
 /**
  * ags_input_new:
  * @audio: the #AgsAudio
@@ -819,7 +812,7 @@ ags_input_next_active(AgsInput *input,
  *
  * Returns: a new #AgsInput
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 AgsInput*
 ags_input_new(GObject *audio)
