@@ -1,5 +1,5 @@
 /* GSequencer - Advanced GTK Sequencer
- * Copyright (C) 2005-2015 Joël Krähemann
+ * Copyright (C) 2005-2018 Joël Krähemann
  *
  * This file is part of GSequencer.
  *
@@ -32,7 +32,6 @@
 #include <ags/i18n.h>
 
 void ags_recall_dependency_class_init(AgsRecallDependencyClass *recall_dependency);
-void ags_recall_dependency_connectable_interface_init(AgsConnectableInterface *connectable);
 void ags_recall_dependency_init(AgsRecallDependency *recall_dependency);
 void ags_recall_dependency_set_property(GObject *gobject,
 					guint prop_id,
@@ -42,14 +41,12 @@ void ags_recall_dependency_get_property(GObject *gobject,
 					guint prop_id,
 					GValue *value,
 					GParamSpec *param_spec);
-void ags_recall_dependency_connect(AgsConnectable *connectable);
-void ags_recall_dependency_disconnect(AgsConnectable *connectable);
 void ags_recall_dependency_dispose(GObject *gobject);
 void ags_recall_dependency_finalize(GObject *gobject);
 
 /**
  * SECTION:ags_recall_dependency
- * @short_description: Object specifing dependency
+ * @short_description: recall dependency
  * @title: AgsRecallDependency
  * @section_id:
  * @include: ags/audio/ags_recall_dependency.h
@@ -58,49 +55,41 @@ void ags_recall_dependency_finalize(GObject *gobject);
  * are resolved during initialization.
  */
 
-static gpointer ags_recall_dependency_parent_class = NULL;
-
 enum{
   PROP_0,
   PROP_DEPENDENCY,
 };
 
+static gpointer ags_recall_dependency_parent_class = NULL;
+
+static pthread_mutex_t ags_recall_dependency_class_mutex = PTHREAD_MUTEX_INITIALIZER;
+
 GType
 ags_recall_dependency_get_type(void)
 {
   static volatile gsize g_define_type_id__volatile = 0;
 
   if(g_once_init_enter (&g_define_type_id__volatile)){
-    GType ags_type_recall_dependency;
+    GType ags_type_recall_dependency = 0;
 
     static const GTypeInfo ags_recall_dependency_info = {
-      sizeof (AgsRecallDependencyClass),
+      sizeof(AgsRecallDependencyClass),
       NULL, /* base_init */
       NULL, /* base_finalize */
       (GClassInitFunc) ags_recall_dependency_class_init,
       NULL, /* class_finalize */
       NULL, /* class_data */
-      sizeof (AgsRecallDependency),
+      sizeof(AgsRecallDependency),
       0,    /* n_preallocs */
       (GInstanceInitFunc) ags_recall_dependency_init,
     };
 
-    static const GInterfaceInfo ags_connectable_interface_info = {
-      (GInterfaceInitFunc) ags_recall_dependency_connectable_interface_init,
-      NULL, /* interface_finalize */
-      NULL, /* interface_data */
-    };
-
     ags_type_recall_dependency = g_type_register_static(G_TYPE_OBJECT,
 							"AgsRecallDependency",
 							&ags_recall_dependency_info,
 							0);
-    
-    g_type_add_interface_static(ags_type_recall_dependency,
-				AGS_TYPE_CONNECTABLE,
-				&ags_connectable_interface_info);
 
-    g_once_init_leave (&g_define_type_id__volatile, ags_type_recall_dependency);
+    g_once_init_leave(&g_define_type_id__volatile, ags_type_recall_dependency);
   }
 
   return g_define_type_id__volatile;
@@ -128,7 +117,7 @@ ags_recall_dependency_class_init(AgsRecallDependencyClass *recall_dependency)
    *
    * The dependency.
    * 
-   * Since: 1.0.0.7
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_object("dependency",
 				   i18n_pspec("dependency of recall"),
@@ -140,18 +129,32 @@ ags_recall_dependency_class_init(AgsRecallDependencyClass *recall_dependency)
 				  param_spec);
 }
 
-void
-ags_recall_dependency_connectable_interface_init(AgsConnectableInterface *connectable)
-{
-  connectable->connect = ags_recall_dependency_connect;
-  connectable->disconnect = ags_recall_dependency_disconnect;
-}
-
 void
 ags_recall_dependency_init(AgsRecallDependency *recall_dependency)
 {
+  pthread_mutex_t *mutex;
+  pthread_mutexattr_t *attr;
+
   recall_dependency->flags = 0;
   
+  /* add recall dependency mutex */
+  recall_dependency->obj_mutexattr = 
+    attr = (pthread_mutexattr_t *) malloc(sizeof(pthread_mutexattr_t));
+  pthread_mutexattr_init(attr);
+  pthread_mutexattr_settype(attr,
+			    PTHREAD_MUTEX_RECURSIVE);
+
+#ifdef __linux__
+  pthread_mutexattr_setprotocol(attr,
+				PTHREAD_PRIO_INHERIT);
+#endif
+
+  recall_dependency->obj_mutex = 
+    mutex = (pthread_mutex_t *) malloc(sizeof(pthread_mutex_t));
+  pthread_mutex_init(mutex,
+		     attr);
+
+  /* dependency */
   recall_dependency->dependency = NULL;
 }
 
@@ -163,7 +166,16 @@ ags_recall_dependency_set_property(GObject *gobject,
 {
   AgsRecallDependency *recall_dependency;
 
+  pthread_mutex_t *recall_dependency_mutex;
+
   recall_dependency = AGS_RECALL_DEPENDENCY(gobject);
+
+  /* get recall dependency mutex */
+  pthread_mutex_lock(ags_recall_dependency_get_class_mutex());
+  
+  recall_dependency_mutex = recall_dependency->obj_mutex;
+  
+  pthread_mutex_unlock(ags_recall_dependency_get_class_mutex());
   
   switch(prop_id){
   case PROP_DEPENDENCY:
@@ -172,7 +184,11 @@ ags_recall_dependency_set_property(GObject *gobject,
       
       dependency = (AgsRecall *) g_value_get_object(value);
       
+      pthread_mutex_lock(recall_dependency_mutex);
+
       if(recall_dependency->dependency == dependency){
+	pthread_mutex_unlock(recall_dependency_mutex);
+	
 	return;
       }
 	
@@ -185,6 +201,8 @@ ags_recall_dependency_set_property(GObject *gobject,
       }
 
       recall_dependency->dependency = dependency;
+
+      pthread_mutex_unlock(recall_dependency_mutex);
     }
     break;
   default:
@@ -201,12 +219,25 @@ ags_recall_dependency_get_property(GObject *gobject,
 {
   AgsRecallDependency *recall_dependency;
 
+  pthread_mutex_t *recall_dependency_mutex;
+
   recall_dependency = AGS_RECALL_DEPENDENCY(gobject);
 
+  /* get recall dependency mutex */
+  pthread_mutex_lock(ags_recall_dependency_get_class_mutex());
+  
+  recall_dependency_mutex = recall_dependency->obj_mutex;
+  
+  pthread_mutex_unlock(ags_recall_dependency_get_class_mutex());
+
   switch(prop_id){
   case PROP_DEPENDENCY:
     {
+      pthread_mutex_lock(recall_dependency_mutex);
+
       g_value_set_object(value, recall_dependency->dependency);
+
+      pthread_mutex_unlock(recall_dependency_mutex);
     }
     break;
   default:
@@ -215,34 +246,6 @@ ags_recall_dependency_get_property(GObject *gobject,
   };
 }
 
-void
-ags_recall_dependency_connect(AgsConnectable *connectable)
-{
-  AgsRecallDependency *recall_dependency;
-
-  recall_dependency = AGS_RECALL_DEPENDENCY(connectable);
-
-  if((AGS_RECALL_DEPENDENCY_CONNECTED & (recall_dependency->flags)) != 0){
-    return;
-  }
-
-  recall_dependency->flags |= AGS_RECALL_DEPENDENCY_CONNECTED;
-}
-
-void
-ags_recall_dependency_disconnect(AgsConnectable *connectable)
-{
-  AgsRecallDependency *recall_dependency;
-
-  recall_dependency = AGS_RECALL_DEPENDENCY(connectable);
-
-  if((AGS_RECALL_DEPENDENCY_CONNECTED & (recall_dependency->flags)) == 0){
-    return;
-  }
-
-  recall_dependency->flags &= (~AGS_RECALL_DEPENDENCY_CONNECTED);
-}
-
 void
 ags_recall_dependency_dispose(GObject *gobject)
 {
@@ -268,6 +271,12 @@ ags_recall_dependency_finalize(GObject *gobject)
 
   recall_dependency = AGS_RECALL_DEPENDENCY(gobject);
 
+  pthread_mutex_destroy(recall_dependency->obj_mutex);
+  free(recall_dependency->obj_mutex);
+
+  pthread_mutexattr_destroy(recall_dependency->obj_mutexattr);
+  free(recall_dependency->obj_mutexattr);
+  
   /* dependency */
   if(recall_dependency->dependency != NULL){
     g_object_unref(recall_dependency->dependency);
@@ -277,30 +286,47 @@ ags_recall_dependency_finalize(GObject *gobject)
   G_OBJECT_CLASS(ags_recall_dependency_parent_class)->finalize(gobject);
 }
 
+/**
+ * ags_recall_dependency_get_class_mutex:
+ * 
+ * Use this function's returned mutex to access mutex fields.
+ *
+ * Returns: the class mutex
+ * 
+ * Since: 2.0.0
+ */
+pthread_mutex_t*
+ags_recall_dependency_get_class_mutex()
+{
+  return(&ags_recall_dependency_class_mutex);
+}
+
 /**
  * ags_recall_dependency_find_dependency:
- * @recall_dependencies: a #GList containing  #AgsRecallDependency
+ * @recall_dependency: a #GList-struct containing  #AgsRecallDependency
  * @dependency: the #AgsRecall depending on
  *
  * Retrieve dependency.
  *
- * Returns: Next match.
+ * Returns: next matching #GList-struct or %NULL.
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 GList*
-ags_recall_dependency_find_dependency(GList *recall_dependencies, GObject *dependency)
+ags_recall_dependency_find_dependency(GList *recall_dependency, GObject *dependency)
 {
-  AgsRecallDependency *recall_dependency;
+  GObject *current_dependency;
 
-  while(recall_dependencies != NULL){
-    recall_dependency = AGS_RECALL_DEPENDENCY(recall_dependencies->data);
+  while(recall_dependency != NULL){
+    g_object_get(recall_dependency->data,
+		 "dependency", &current_dependency,
+		 NULL);
 
-    if(recall_dependency->dependency == dependency){
-      return(recall_dependencies);
+    if(current_dependency == dependency){
+      return(recall_dependency);
     }
 
-    recall_dependencies = recall_dependencies->next;
+    recall_dependency = recall_dependency->next;
   }
 
   return(NULL);
@@ -308,35 +334,53 @@ ags_recall_dependency_find_dependency(GList *recall_dependencies, GObject *depen
 
 /**
  * ags_recall_dependency_find_dependency_by_provider:
- * @recall_dependencies: a #GList containing  #AgsRecallDependency
+ * @recall_dependency: a #GList-struct containing  #AgsRecallDependency
  * @provider: the object providing recall, like #AgsAudio or #AgsChannel
  *
  * Retrieve dependency by provider.
  *
- * Returns: Next match.
+ * Returns: next matching #GList-struct or %NULL.
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 GList*
-ags_recall_dependency_find_dependency_by_provider(GList *recall_dependencies,
+ags_recall_dependency_find_dependency_by_provider(GList *recall_dependency,
 						  GObject *provider)
 {
-  AgsRecallDependency *recall_dependency;
+  GObject *current_dependency;
 
-  while(recall_dependencies != NULL){
-    recall_dependency = AGS_RECALL_DEPENDENCY(recall_dependencies->data);
+  while(recall_dependency != NULL){
+    g_object_get(recall_dependency->data,
+		 "dependency", &current_dependency,
+		 NULL);
 
-    if((AGS_IS_CHANNEL(provider) &&
-	AGS_IS_RECALL_CHANNEL_RUN(recall_dependency->dependency) &&
-	(AGS_RECALL_CHANNEL_RUN(recall_dependency->dependency)->source == AGS_CHANNEL(provider))) ||
-       (AGS_IS_AUDIO(provider) &&
-	AGS_IS_RECALL_AUDIO_RUN(recall_dependency->dependency) &&
-	AGS_RECALL_AUDIO_RUN(recall_dependency->dependency)->recall_audio != NULL &&
-	(AGS_RECALL_AUDIO_RUN(recall_dependency->dependency)->recall_audio->audio == AGS_AUDIO(provider)))){
-      return(recall_dependencies);
+    if(AGS_IS_CHANNEL(provider) &&
+       AGS_IS_RECALL_CHANNEL_RUN(current_dependency)){
+      AgsChannel *channel;
+
+      g_object_get(current_dependency,
+		   "source", &channel,
+		   NULL);
+      
+      if(channel == AGS_CHANNEL(provider)){
+	return(recall_dependency);
+      }
     }
 
-    recall_dependencies = recall_dependencies->next;
+    if(AGS_IS_AUDIO(provider) &&
+       AGS_IS_RECALL_AUDIO_RUN(current_dependency)){
+      AgsAudio *audio;
+
+      g_object_get(current_dependency,
+		   "audio", &audio,
+		   NULL);
+      
+      if(audio == AGS_AUDIO(provider)){
+	return(recall_dependency);
+      }
+    }
+
+    recall_dependency = recall_dependency->next;
   }
 
   return(NULL);
@@ -351,7 +395,7 @@ ags_recall_dependency_find_dependency_by_provider(GList *recall_dependencies,
  *
  * Returns: the #AgsRecall dependency.
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 GObject*
 ags_recall_dependency_resolve(AgsRecallDependency *recall_dependency, AgsRecallID *recall_id)
@@ -359,74 +403,125 @@ ags_recall_dependency_resolve(AgsRecallDependency *recall_dependency, AgsRecallI
   AgsRecallContainer *recall_container;
   AgsRecall *dependency;
   
-  if(recall_dependency->dependency == NULL ||
-     AGS_RECALL(recall_dependency->dependency)->container == NULL){
-    g_message("dependency resolve: container == NULL");
+  g_object_get(recall_dependency,
+	       "dependency", &dependency,
+	       NULL);
+  
+  if(dependency == NULL){
     return(NULL);
   }
 
-  dependency = AGS_RECALL(recall_dependency->dependency);
-  recall_container = AGS_RECALL_CONTAINER(dependency->container);
+  g_object_get(dependency,
+	       "recall-container", &recall_container,
+	       NULL);
+  
+  if(recall_container == NULL){
+    return(NULL);
+  }
   
   if(AGS_IS_RECALL_AUDIO(dependency)){
-    return((GObject *) recall_container->recall_audio);
+    GObject *recall_audio;
+
+    g_object_get(recall_container,
+		 "recall-audio", &recall_audio,
+		 NULL);
+    
+    return(recall_audio);
   }else if(AGS_IS_RECALL_AUDIO_RUN(dependency)){
-    GList *recall_list;
+    AgsRecyclingContext *recycling_context;
+
+    GObject *recall_audio_run;
+    
+    GList *list_start, *list;
 
     if(recall_id == NULL){
       g_message("dependency resolve: recall_id == NULL");
+      
       return(NULL);
     }
 
-    recall_list = ags_recall_find_recycling_context(recall_container->recall_audio_run,
-						    (GObject *) recall_id->recycling_context);
+    g_object_get(recall_id,
+		 "recycling-context", &recycling_context,
+		 NULL);
+    
+    g_object_get(recall_container,
+		 "recall-audio-run", &list_start,
+		 NULL);
+    list = ags_recall_find_recycling_context(list_start,
+					     (GObject *) recycling_context);
 
-    if(recall_list != NULL){
-      return(G_OBJECT(recall_list->data));
-    }else if(recall_id->recycling_context->parent != NULL){
-      recall_list = ags_recall_find_recycling_context(recall_container->recall_audio_run,
-						      (GObject *) recall_id->recycling_context->parent);
+    if(list != NULL){
+      recall_audio_run = list->data;
 
-      if(recall_list != NULL){
-	return(G_OBJECT(recall_list->data));
-      }else{
-	g_warning("dependency not found!");
-      }
+      g_list_free(list_start);
+      
+      return(recall_audio_run);
     }
+
+    g_list_free(list_start);
   }else if(AGS_IS_RECALL_CHANNEL(dependency)){
-    GList *recall_list;
+    AgsChannel *source;
 
-    recall_list = ags_recall_find_provider(recall_container->recall_channel,
-					   (GObject *) AGS_RECALL_CHANNEL(dependency)->source);
+    GObject *recall_channel;
+    
+    GList *list_start, *list;
 
-    if(recall_list != NULL){
-      return(G_OBJECT(recall_list->data));
+    g_object_get(recall_container,
+		 "recall-channel", &list_start,
+		 NULL);
+
+    g_object_get(dependency,
+		 "source", &source,
+		 NULL);
+    
+    list = ags_recall_find_provider(list_start,
+				    (GObject *) source);
+
+    if(list != NULL){
+      recall_channel = list->data;
+
+      g_list_free(list_start);
+      
+      return(recall_channel);
     }
+
+    g_list_free(list_start);
   }else if(AGS_IS_RECALL_CHANNEL_RUN(dependency)){
-    GList *recall_list;
+    AgsRecyclingContext *recycling_context;
+
+    GObject *recall_channel_run;
+    
+    GList *list_start, *list;
 
     if(recall_id == NULL){
       g_message("dependency resolve: recall_id == NULL");
+      
       return(NULL);
     }
 
-    recall_list = ags_recall_find_provider_with_recycling_context(recall_container->recall_channel_run,
-								  (GObject *) AGS_RECALL_CHANNEL_RUN(dependency)->recall_channel->source,
-								  (GObject *) recall_id->recycling_context);
+    g_object_get(recall_id,
+		 "recycling-context", &recycling_context,
+		 NULL);
+    
+    g_object_get(recall_container,
+		 "recall-channel-run", &list_start,
+		 NULL);
+    list = ags_recall_find_recycling_context(list_start,
+					     (GObject *) recycling_context);
 
-    if(recall_list != NULL){
-      return(G_OBJECT(recall_list->data));
-    }else if(recall_id->recycling_context->parent != NULL){
-      recall_list = ags_recall_find_provider_with_recycling_context(recall_container->recall_channel_run,
-								    (GObject *) AGS_RECALL_CHANNEL_RUN(dependency)->recall_channel->source,
-								    (GObject *) recall_id->recycling_context->parent);
+    if(list != NULL){
+      recall_channel_run = list->data;
 
-      if(recall_list != NULL){
-	return(G_OBJECT(recall_list->data));
-      }      
+      g_list_free(list_start);
+      
+      return(recall_channel_run);
     }
+
+    g_list_free(list_start);
   }
 
+  g_warning("dependency not found!");
+
   return(NULL);
 }
 
@@ -434,11 +529,11 @@ ags_recall_dependency_resolve(AgsRecallDependency *recall_dependency, AgsRecallI
  * ags_recall_dependency_new:
  * @dependency: the #AgsRecall depending on
  *
- * Creates a #AgsRecallDependency
+ * Creates an #AgsRecallDependency
  *
- * Returns: a new #AgsRecallDependency
+ * Returns: the new #AgsRecallDependency
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 AgsRecallDependency*
 ags_recall_dependency_new(GObject *dependency)
