@@ -60,7 +60,7 @@ void ags_recall_dssi_run_get_property(GObject *gobject,
 void ags_recall_dssi_run_finalize(GObject *gobject);
 
 void ags_recall_dssi_run_run_init_pre(AgsRecall *recall);
-void ags_recall_dssi_run_feed_input_queue(AgsRecall *recall);
+void ags_recall_dssi_run_run_pre(AgsRecall *recall);
 
 void ags_recall_dssi_run_load_ports(AgsRecallDssiRun *recall_dssi_run);
 
@@ -187,7 +187,7 @@ ags_recall_dssi_run_class_init(AgsRecallDssiRunClass *recall_dssi_run)
   recall = (AgsRecallClass *) recall_dssi_run;
 
   recall->run_init_pre = ags_recall_dssi_run_run_init_pre;
-  recall->feed_input_queue = ags_recall_dssi_run_feed_input_queue;
+  recall->run_pre = ags_recall_dssi_run_run_pre;
 }
 
 void
@@ -548,7 +548,7 @@ ags_recall_dssi_run_run_init_pre(AgsRecall *recall)
 }
 
 void
-ags_recall_dssi_run_feed_input_queue(AgsRecall *recall)
+ags_recall_dssi_run_run_pre(AgsRecall *recall)
 {
   AgsRecallDssi *recall_dssi;
   AgsRecallChannelRun *recall_channel_run;
@@ -584,7 +584,7 @@ ags_recall_dssi_run_feed_input_queue(AgsRecall *recall)
   guint buffer_size;
   guint i, i_stop;
 
-  void (*parent_class_feed_input_queue)(AgsRecall *recall);
+  void (*parent_class_run_pre)(AgsRecall *recall);
 
   void (*select_program)(LADSPA_Handle Instance,
 			 unsigned long Bank,
@@ -604,12 +604,12 @@ ags_recall_dssi_run_feed_input_queue(AgsRecall *recall)
   /* get parent class */
   pthread_mutex_lock(ags_recall_get_class_mutex());
 
-  parent_class_feed_input_queue = AGS_RECALL_CLASS(ags_recall_dssi_run_parent_class)->feed_input_queue;
+  parent_class_run_pre = AGS_RECALL_CLASS(ags_recall_dssi_run_parent_class)->run_pre;
   
   pthread_mutex_unlock(ags_recall_get_class_mutex());
 
   /* call parent */
-  parent_class_feed_input_queue(recall);
+  parent_class_run_pre(recall);
 
   g_object_get(recall,
 	       "recall-id", &recall_id,
@@ -704,7 +704,7 @@ ags_recall_dssi_run_feed_input_queue(AgsRecall *recall)
 		   "x1", &x1,
 		   NULL);
 	
-      if((x1 <= notation_counter &&
+      if((x1 + 1 <= notation_counter &&
 	  !ags_note_test_flags(note->data, AGS_NOTE_FEED)) ||
 	 x0 > notation_counter){
 	recall_dssi_run->note = g_list_remove(recall_dssi_run->note,
@@ -727,7 +727,7 @@ ags_recall_dssi_run_feed_input_queue(AgsRecall *recall)
 		 NULL);
     
     if(audio_signal->stream_current == NULL ||
-       (x1 <= notation_counter &&
+       (x1 + 1 <= notation_counter &&
 	!ags_note_test_flags(note_start->data, AGS_NOTE_FEED)) ||
        x0 > notation_counter){
       //    g_message("done");
