@@ -1,5 +1,5 @@
 /* GSequencer - Advanced GTK Sequencer
- * Copyright (C) 2005-2015 Joël Krähemann
+ * Copyright (C) 2005-2018 Joël Krähemann
  *
  * This file is part of GSequencer.
  *
@@ -25,11 +25,10 @@
 
 #include <ladspa.h>
 
-#include <ags/plugin/ags_ladspa_plugin.h>
+#include <ags/audio/ags_recall_channel.h>
 
-#include <ags/audio/ags_recall.h>
-#include <ags/audio/ags_recall_channel_run.h>
 #include <ags/audio/ags_channel.h>
+#include <ags/audio/ags_recall.h>
 
 #define AGS_TYPE_RECALL_LADSPA                (ags_recall_ladspa_get_type())
 #define AGS_RECALL_LADSPA(obj)                (G_TYPE_CHECK_INSTANCE_CAST((obj), AGS_TYPE_RECALL_LADSPA, AgsRecallLadspa))
@@ -45,18 +44,13 @@ struct _AgsRecallLadspa
 {
   AgsRecallChannel recall_channel;
 
-  gchar *filename;
-  gchar *effect;
-  unsigned long index;
-  
-  AgsLadspaPlugin *plugin;
   LADSPA_Descriptor *plugin_descriptor;
 
-  unsigned long *input_port;
-  unsigned long input_lines;
+  guint *input_port;
+  guint input_lines;
 
-  unsigned long *output_port;
-  unsigned long output_lines;
+  guint *output_port;
+  guint output_lines;
 };
 
 struct _AgsRecallLadspaClass
@@ -70,7 +64,7 @@ void ags_recall_ladspa_load(AgsRecallLadspa *recall_ladspa);
 GList* ags_recall_ladspa_load_ports(AgsRecallLadspa *recall_ladspa);
 void ags_recall_ladspa_load_conversion(AgsRecallLadspa *recall_ladspa,
 				       GObject *port,
-				       gpointer port_descriptor);
+				       gpointer plugin_port);
 
 GList* ags_recall_ladspa_find(GList *recall,
 			      gchar *filename, gchar *effect);
@@ -78,6 +72,6 @@ GList* ags_recall_ladspa_find(GList *recall,
 AgsRecallLadspa* ags_recall_ladspa_new(AgsChannel *source,
 				       gchar *filename,
 				       gchar *effect,
-				       unsigned long index);
+				       guint effect_index);
 
 #endif /*__AGS_RECALL_LADSPA_H__*/
