@@ -1,5 +1,5 @@
 /* GSequencer - Advanced GTK Sequencer
- * Copyright (C) 2005-2017 Joël Krähemann
+ * Copyright (C) 2005-2018 Joël Krähemann
  *
  * This file is part of GSequencer.
  *
@@ -23,7 +23,7 @@
 #include <glib.h>
 #include <glib-object.h>
 
-#include <ags/lib/ags_function.h>
+#include <ags/libags.h>
 
 #define AGS_TYPE_SYNTH_GENERATOR                (ags_synth_generator_get_type())
 #define AGS_SYNTH_GENERATOR(obj)                (G_TYPE_CHECK_INSTANCE_CAST((obj), AGS_TYPE_SYNTH_GENERATOR, AgsSynthGenerator))
@@ -51,32 +51,6 @@ typedef enum{
   AGS_SYNTH_GENERATOR_OSCILLATOR_SQUARE,
 }AgsSynthGeneratorOscillator;
 
-/**
- * AgsSynthGeneratorComputeFlags:
- * @AGS_SYNTH_GENERATOR_COMPUTE_FIXED_LENGTH: fixed length
- * @AGS_SYNTH_GENERATOR_COMPUTE_SYNC: sync
- * @AGS_SYNTH_GENERATOR_COMPUTE_16HZ: 16 Hz
- * @AGS_SYNTH_GENERATOR_COMPUTE_440HZ: 440 Hz
- * @AGS_SYNTH_GENERATOR_COMPUTE_22000HZ: 22000 Hz
- * @AGS_SYNTH_GENERATOR_COMPUTE_LIMIT: limit
- * @AGS_SYNTH_GENERATOR_COMPUTE_NOHZ: no Hertz
- * @AGS_SYNTH_GENERATOR_COMPUTE_FREQUENCY: frequency
- * @AGS_SYNTH_GENERATOR_COMPUTE_NOTE: note
- * 
- * Enum values to specify compute flags.
- */
-typedef enum{
-  AGS_SYNTH_GENERATOR_COMPUTE_FIXED_LENGTH  = 1,
-  AGS_SYNTH_GENERATOR_COMPUTE_SYNC          = 1 <<  1,
-  AGS_SYNTH_GENERATOR_COMPUTE_16HZ          = 1 <<  2,
-  AGS_SYNTH_GENERATOR_COMPUTE_440HZ         = 1 <<  3,
-  AGS_SYNTH_GENERATOR_COMPUTE_22000HZ       = 1 <<  4,
-  AGS_SYNTH_GENERATOR_COMPUTE_LIMIT         = 1 <<  5,
-  AGS_SYNTH_GENERATOR_COMPUTE_NOHZ          = 1 <<  6,
-  AGS_SYNTH_GENERATOR_COMPUTE_FREQUENCY     = 1 <<  7,
-  AGS_SYNTH_GENERATOR_COMPUTE_NOTE          = 1 <<  8,
-}AgsSynthGeneratorComputeFlags;
-
 struct _AgsSynthGenerator
 {
   AgsFunction function;
@@ -88,6 +62,8 @@ struct _AgsSynthGenerator
   guint format;
 
   guint frame_count;
+  guint loop_start;
+  guint loop_end;
 
   gdouble delay;
   guint attack;
@@ -97,6 +73,13 @@ struct _AgsSynthGenerator
   gdouble frequency;
   gdouble phase;
   gdouble volume;
+
+  gboolean sync_relative;
+  AgsComplex **sync_point;
+  guint sync_point_count;
+
+  AgsComplex *damping;
+  AgsComplex *vibration;
   
   GObject *timestamp;
 };
@@ -108,18 +91,9 @@ struct _AgsSynthGeneratorClass
   
 GType ags_synth_generator_get_type();
 
-GObject* ags_synth_generator_compute(AgsSynthGenerator *synth_generator,
-				     gdouble note);
-void ags_synth_generator_compute_with_audio_signal(AgsSynthGenerator *synth_generator,
-						   GObject *audio_signal,
-						   gdouble note,
-						   AgsComplex *sync_start, AgsComplex *sync_end,
-						   guint compute_flags);
-void ags_synth_generator_compute_extended(AgsSynthGenerator *synth_generator,
-					  GObject *audio_signal,
-					  gdouble note,
-					  AgsComplex **sync_point,
-					  guint sync_point_count);
+void ags_synth_generator_compute(AgsSynthGenerator *synth_generator,
+				 GObject *audio_signal,
+				 gdouble note);
 
 AgsSynthGenerator* ags_synth_generator_new();
 
