@@ -1,5 +1,5 @@
 /* GSequencer - Advanced GTK Sequencer
- * Copyright (C) 2005-2015 Joël Krähemann
+ * Copyright (C) 2005-2018 Joël Krähemann
  *
  * This file is part of GSequencer.
  *
@@ -85,7 +85,7 @@ ags_audio_file_link_get_type()
   static volatile gsize g_define_type_id__volatile = 0;
 
   if(g_once_init_enter (&g_define_type_id__volatile)){
-    GType ags_type_audio_file_link;
+    GType ags_type_audio_file_link = 0;
 
     static const GTypeInfo ags_audio_file_link_info = {
       sizeof (AgsAudioFileLinkClass),
@@ -114,7 +114,7 @@ ags_audio_file_link_get_type()
 				AGS_TYPE_PLUGIN,
 				&ags_plugin_interface_info);
 
-    g_once_init_leave (&g_define_type_id__volatile, ags_type_audio_file_link);
+    g_once_init_leave(&g_define_type_id__volatile, ags_type_audio_file_link);
   }
 
   return g_define_type_id__volatile;
@@ -142,7 +142,7 @@ ags_audio_file_link_class_init(AgsAudioFileLinkClass *audio_file_link)
    *
    * The assigned preset.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_string("preset",
 				   i18n_pspec("the preset"),
@@ -158,7 +158,7 @@ ags_audio_file_link_class_init(AgsAudioFileLinkClass *audio_file_link)
    *
    * The assigned instrument.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_string("instrument",
 				   i18n_pspec("the instrument"),
@@ -174,7 +174,7 @@ ags_audio_file_link_class_init(AgsAudioFileLinkClass *audio_file_link)
    *
    * The assigned sample.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_string("sample",
 				   i18n_pspec("the sample"),
@@ -190,7 +190,7 @@ ags_audio_file_link_class_init(AgsAudioFileLinkClass *audio_file_link)
    *
    * The assigned audio channel.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_uint("audio-channel",
 				 i18n_pspec("audio channel to read"),
@@ -207,7 +207,7 @@ ags_audio_file_link_class_init(AgsAudioFileLinkClass *audio_file_link)
    *
    * The assigned timestamp.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_object("timestamp",
 				   i18n_pspec("timestamp"),
@@ -248,7 +248,16 @@ ags_audio_file_link_set_property(GObject *gobject,
 {
   AgsAudioFileLink *audio_file_link;
 
+  pthread_mutex_t *file_link_mutex;
+
   audio_file_link = AGS_AUDIO_FILE_LINK(gobject);
+
+  /* get file link mutex */
+  pthread_mutex_lock(ags_file_link_get_class_mutex());
+  
+  file_link_mutex = AGS_FILE_LINK(audio_file_link)->obj_mutex;
+  
+  pthread_mutex_unlock(ags_file_link_get_class_mutex());
   
   switch(prop_id){
   case PROP_PRESET:
@@ -257,7 +266,11 @@ ags_audio_file_link_set_property(GObject *gobject,
 
       preset = (char *) g_value_get_string(value);
 
+      pthread_mutex_lock(file_link_mutex);
+
       if(preset == audio_file_link->preset){
+	pthread_mutex_unlock(file_link_mutex);
+
 	return;
       }
 
@@ -266,6 +279,8 @@ ags_audio_file_link_set_property(GObject *gobject,
       }
 	
       audio_file_link->preset = g_strdup(preset);
+
+      pthread_mutex_unlock(file_link_mutex);
     }
     break;
   case PROP_INSTRUMENT:
@@ -274,7 +289,11 @@ ags_audio_file_link_set_property(GObject *gobject,
 
       instrument = (char *) g_value_get_string(value);
 
+      pthread_mutex_lock(file_link_mutex);
+
       if(instrument == audio_file_link->instrument){
+	pthread_mutex_unlock(file_link_mutex);
+
 	return;
       }
 
@@ -283,6 +302,8 @@ ags_audio_file_link_set_property(GObject *gobject,
       }
 	
       audio_file_link->instrument = g_strdup(instrument);
+
+      pthread_mutex_unlock(file_link_mutex);
     }
     break;
   case PROP_SAMPLE:
@@ -291,7 +312,11 @@ ags_audio_file_link_set_property(GObject *gobject,
 
       sample = (char *) g_value_get_string(value);
 
+      pthread_mutex_lock(file_link_mutex);
+
       if(sample == audio_file_link->sample){
+	pthread_mutex_unlock(file_link_mutex);
+
 	return;
       }
 
@@ -300,11 +325,17 @@ ags_audio_file_link_set_property(GObject *gobject,
       }
 	
       audio_file_link->sample = g_strdup(sample);
+
+      pthread_mutex_unlock(file_link_mutex);
     }
     break;
   case PROP_AUDIO_CHANNEL:
     {
+      pthread_mutex_lock(file_link_mutex);
+
       audio_file_link->audio_channel = g_value_get_uint(value);
+
+      pthread_mutex_unlock(file_link_mutex);
     }
     break;
   case PROP_TIMESTAMP:
@@ -313,7 +344,11 @@ ags_audio_file_link_set_property(GObject *gobject,
 
       timestamp = (GObject *) g_value_get_object(value);
 
+      pthread_mutex_lock(file_link_mutex);
+
       if((AgsTimestamp *) timestamp == audio_file_link->timestamp){
+	pthread_mutex_unlock(file_link_mutex);
+
 	return;
       }
 
@@ -326,6 +361,8 @@ ags_audio_file_link_set_property(GObject *gobject,
       }
 
       audio_file_link->timestamp = (AgsTimestamp *) timestamp;
+
+      pthread_mutex_unlock(file_link_mutex);
     }
     break;
   default:
@@ -342,32 +379,61 @@ ags_audio_file_link_get_property(GObject *gobject,
 {
   AgsAudioFileLink *audio_file_link;
 
+  pthread_mutex_t *file_link_mutex;
+
   audio_file_link = AGS_AUDIO_FILE_LINK(gobject);
+
+  /* get file link mutex */
+  pthread_mutex_lock(ags_file_link_get_class_mutex());
+  
+  file_link_mutex = AGS_FILE_LINK(audio_file_link)->obj_mutex;
+  
+  pthread_mutex_unlock(ags_file_link_get_class_mutex());
   
   switch(prop_id){
   case PROP_PRESET:
     {
+      pthread_mutex_lock(file_link_mutex);
+
       g_value_set_string(value, audio_file_link->preset);
+
+      pthread_mutex_unlock(file_link_mutex);
     }
     break;
   case PROP_INSTRUMENT:
     {
+      pthread_mutex_lock(file_link_mutex);
+
       g_value_set_string(value, audio_file_link->instrument);
+
+      pthread_mutex_unlock(file_link_mutex);
     }
     break;
   case PROP_SAMPLE:
     {
+      pthread_mutex_lock(file_link_mutex);
+
       g_value_set_string(value, audio_file_link->sample);
+
+      pthread_mutex_unlock(file_link_mutex);
     }
     break;
   case PROP_AUDIO_CHANNEL:
     {
+      pthread_mutex_lock(file_link_mutex);
+
       g_value_set_uint(value, audio_file_link->audio_channel);
+
+      pthread_mutex_unlock(file_link_mutex);
     }
     break;
   case PROP_TIMESTAMP:
     {
+      pthread_mutex_lock(file_link_mutex);
+
       g_value_set_object(value, audio_file_link->timestamp);
+
+      pthread_mutex_unlock(file_link_mutex);
     }
     break;
   default:
@@ -491,7 +557,6 @@ ags_audio_file_link_read_launch(AgsFileLaunch *file_launch,
   xmlChar *filename;
   xmlChar *encoding, *demuxer;
 
-  guint audio_channels;
   guint audio_channel;
   
   node = file_launch->node;
@@ -508,16 +573,13 @@ ags_audio_file_link_read_launch(AgsFileLaunch *file_launch,
 
   if(id_ref != NULL){
     input = (AgsChannel *) id_ref->ref;
-    soundcard = input->soundcard;
+    soundcard = input->output_soundcard;
   }
 
   /* get audio */
   if(input != NULL &&
      input->audio != NULL){
     audio = AGS_AUDIO(input->audio);
-    audio_channels = audio->audio_channels;
-  }else{
-    audio_channels = 1;
   }
   
   /* read file link using URL or embedded */
@@ -533,20 +595,19 @@ ags_audio_file_link_read_launch(AgsFileLaunch *file_launch,
     filename = AGS_FILE_LINK(audio_file_link)->filename;
     audio_file = ags_audio_file_new((gchar *) filename,
 				    soundcard,
-				    0, audio_channels);
+				    audio_channel);
 
     /* open file and read audio signal */
     ags_audio_file_open(audio_file);
     ags_audio_file_read_audio_signal(audio_file);
 
-    /* retrieve audio channel */
-    audio_signal = g_list_nth(audio_file->audio_signal,
-			      audio_channel);
-
-    if(audio_signal == NULL){
+    /* retrieve audio signal */
+    if(audio_file->audio_signal == NULL){
       return;
     }
 
+    audio_signal = audio_file->audio_signal;
+    
     /* set template flag */
     AGS_AUDIO_SIGNAL(audio_signal->data)->flags |= AGS_AUDIO_SIGNAL_TEMPLATE;
 
@@ -623,7 +684,7 @@ ags_audio_file_link_read_launch(AgsFileLaunch *file_launch,
 
 	  audio_file = ags_audio_file_new(NULL,
 					  soundcard,
-					  0, audio_channels);
+					  audio_channel);
 	  data = child->content;
 
 	  ags_audio_file_open_from_data(audio_file, data);
@@ -647,11 +708,11 @@ ags_audio_file_link_read_launch(AgsFileLaunch *file_launch,
 /**
  * ags_audio_file_link_new:
  *
- * Creates an #AgsAudioFileLink
+ * Create a new instance of #AgsAudioFileLink
  *
- * Returns: a new #AgsAudioFileLink
+ * Returns: the new #AgsAudioFileLink
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 AgsAudioFileLink*
 ags_audio_file_link_new()
