@@ -177,13 +177,15 @@ enum{
 static gpointer ags_midi_builder_parent_class = NULL;
 static guint midi_builder_signals[LAST_SIGNAL];
 
+static pthread_mutex_t ags_midi_builder_class_mutex = PTHREAD_MUTEX_INITIALIZER;
+
 GType
 ags_midi_builder_get_type(void)
 {
   static volatile gsize g_define_type_id__volatile = 0;
 
   if(g_once_init_enter (&g_define_type_id__volatile)){
-    GType ags_type_midi_builder;
+    GType ags_type_midi_builder = 0;
 
     static const GTypeInfo ags_midi_builder_info = {
       sizeof (AgsMidiBuilderClass),
@@ -201,7 +203,7 @@ ags_midi_builder_get_type(void)
 						   "AgsMidiBuilder", &ags_midi_builder_info,
 						   0);
 
-    g_once_init_leave (&g_define_type_id__volatile, ags_type_midi_builder);
+    g_once_init_leave(&g_define_type_id__volatile, ags_type_midi_builder);
   }
 
   return g_define_type_id__volatile;
@@ -229,7 +231,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The file to parse data from.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_pointer("file",
 				    i18n_pspec("the file stream"),
@@ -272,7 +274,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::midi-putc signal is emited during putting char to file.
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_builder_signals[MIDI_PUTC] =
     g_signal_new("midi-putc",
@@ -291,7 +293,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::on-error signal is emited during building of event.
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_builder_signals[ON_ERROR] =
     g_signal_new("on-error",
@@ -316,7 +318,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::append-header signal is emited during building of header.
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_builder_signals[APPEND_HEADER] =
     g_signal_new("append-header",
@@ -324,7 +326,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_header),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__UINT_UINT_UINT_UINT_UINT_UINT_UINT,
+		 ags_cclosure_marshal_VOID__UINT_UINT_UINT_UINT_UINT_UINT_UINT,
 		 G_TYPE_NONE, 7,
 		 G_TYPE_UINT, G_TYPE_UINT,
 		 G_TYPE_UINT, G_TYPE_UINT,
@@ -340,7 +342,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::append-track signal is emited during building of track.
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_builder_signals[APPEND_TRACK] =
     g_signal_new("append-track",
@@ -362,7 +364,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::key-on signal is emited during building of event.
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_builder_signals[KEY_ON] =
     g_signal_new("key-on",
@@ -370,7 +372,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_key_on),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__UINT_UINT_UINT_UINT,
+		 ags_cclosure_marshal_VOID__UINT_UINT_UINT_UINT,
 		 G_TYPE_NONE, 4,
 		 G_TYPE_UINT,
 		 G_TYPE_UINT,
@@ -387,7 +389,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::key-off signal is emited during building of event.
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_builder_signals[KEY_OFF] =
     g_signal_new("key-off",
@@ -395,7 +397,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_key_off),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__UINT_UINT_UINT_UINT,
+		 ags_cclosure_marshal_VOID__UINT_UINT_UINT_UINT,
 		 G_TYPE_NONE, 4,
 		 G_TYPE_UINT,
 		 G_TYPE_UINT,
@@ -412,7 +414,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::key-pressure signal is emited during building of event.
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_builder_signals[KEY_PRESSURE] =
     g_signal_new("key-pressure",
@@ -420,7 +422,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_key_pressure),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__UINT_UINT_UINT_UINT,
+		 ags_cclosure_marshal_VOID__UINT_UINT_UINT_UINT,
 		 G_TYPE_NONE, 4,
 		 G_TYPE_UINT,
 		 G_TYPE_UINT,
@@ -437,7 +439,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::change-parameter signal is emited during building of event.
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_builder_signals[CHANGE_PARAMETER] =
     g_signal_new("change-parameter",
@@ -445,7 +447,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_change_parameter),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__UINT_UINT_UINT_UINT,
+		 ags_cclosure_marshal_VOID__UINT_UINT_UINT_UINT,
 		 G_TYPE_NONE, 4,
 		 G_TYPE_UINT,
 		 G_TYPE_UINT,
@@ -462,7 +464,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::change-pitch-bend signal is emited during building of event.
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_builder_signals[CHANGE_PITCH_BEND] =
     g_signal_new("change-pitch-bend",
@@ -470,7 +472,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_change_pitch_bend),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__UINT_UINT_UINT_UINT,
+		 ags_cclosure_marshal_VOID__UINT_UINT_UINT_UINT,
 		 G_TYPE_NONE, 4,
 		 G_TYPE_UINT,
 		 G_TYPE_UINT,
@@ -486,7 +488,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::change-program signal is emited during building of event.
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_builder_signals[CHANGE_PROGRAM] =
     g_signal_new("change-program",
@@ -494,7 +496,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_change_program),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__UINT_UINT_UINT,
+		 ags_cclosure_marshal_VOID__UINT_UINT_UINT,
 		 G_TYPE_NONE, 3,
 		 G_TYPE_UINT,
 		 G_TYPE_UINT,
@@ -509,7 +511,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::change-channel-pressure signal is emited during building of event.
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_builder_signals[CHANGE_PRESSURE] =
     g_signal_new("change-channel-pressure",
@@ -517,7 +519,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_change_pressure),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__UINT_UINT_UINT,
+		 ags_cclosure_marshal_VOID__UINT_UINT_UINT,
 		 G_TYPE_NONE, 3,
 		 G_TYPE_UINT,
 		 G_TYPE_UINT,
@@ -532,7 +534,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::sysex signal is emited during building of event.
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_builder_signals[SYSEX] =
     g_signal_new("sysex",
@@ -540,7 +542,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_sysex),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__UINT_POINTER_UINT,
+		 ags_cclosure_marshal_VOID__UINT_POINTER_UINT,
 		 G_TYPE_NONE, 3,
 		 G_TYPE_UINT,
 		 G_TYPE_POINTER,
@@ -555,7 +557,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::quarter-frame signal is emited during building of event.
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_builder_signals[QUARTER_FRAME] =
     g_signal_new("quarter-frame",
@@ -563,7 +565,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_quarter_frame),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__UINT_UINT_UINT,
+		 ags_cclosure_marshal_VOID__UINT_UINT_UINT,
 		 G_TYPE_NONE, 3,
 		 G_TYPE_UINT,
 		 G_TYPE_UINT,
@@ -577,7 +579,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::song-position signal is emited during building of event.
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_builder_signals[SONG_POSITION] =
     g_signal_new("song-position",
@@ -585,7 +587,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_song_position),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__UINT_UINT,
+		 ags_cclosure_marshal_VOID__UINT_UINT,
 		 G_TYPE_NONE, 2,
 		 G_TYPE_UINT,
 		 G_TYPE_UINT);
@@ -598,7 +600,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::song-select signal is emited during building of event.
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_builder_signals[SONG_SELECT] =
     g_signal_new("song-select",
@@ -606,7 +608,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_song_select),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__UINT_UINT,
+		 ags_cclosure_marshal_VOID__UINT_UINT,
 		 G_TYPE_NONE, 2,
 		 G_TYPE_UINT,
 		 G_TYPE_UINT);
@@ -618,7 +620,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::tune-request signal is emited during building of event.
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_builder_signals[TUNE_REQUEST] =
     g_signal_new("tune-request",
@@ -639,7 +641,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::sequence-number signal is emited during building of event.
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_builder_signals[SEQUENCE_NUMBER] =
     g_signal_new("sequence-number",
@@ -647,7 +649,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_sequence_number),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__UINT_UINT,
+		 ags_cclosure_marshal_VOID__UINT_UINT,
 		 G_TYPE_NONE, 2,
 		 G_TYPE_UINT,
 		 G_TYPE_UINT);
@@ -664,7 +666,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::smtpe signal is emited during building of event.
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_builder_signals[SMTPE] =
     g_signal_new("smtpe",
@@ -672,7 +674,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_smtpe),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__UINT_UINT_UINT_UINT_UINT_UINT,
+		 ags_cclosure_marshal_VOID__UINT_UINT_UINT_UINT_UINT_UINT,
 		 G_TYPE_NONE, 6,
 		 G_TYPE_UINT,
 		 G_TYPE_UINT,
@@ -689,7 +691,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::tempo signal is emited during building of event.
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_builder_signals[TEMPO] =
     g_signal_new("tempo",
@@ -697,7 +699,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_tempo),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__UINT_UINT,
+		 ags_cclosure_marshal_VOID__UINT_UINT,
 		 G_TYPE_NONE, 2,
 		 G_TYPE_UINT,
 		 G_TYPE_UINT);
@@ -713,7 +715,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::time-signature signal is emited during building of event.
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_builder_signals[TIME_SIGNATURE] =
     g_signal_new("time-signature",
@@ -721,7 +723,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_time_signature),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__UINT_UINT_UINT_UINT_UINT,
+		 ags_cclosure_marshal_VOID__UINT_UINT_UINT_UINT_UINT,
 		 G_TYPE_NONE, 5,
 		 G_TYPE_UINT,
 		 G_TYPE_UINT,
@@ -738,7 +740,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    * 
    * The ::key-signature signal is emited during building of event.
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_builder_signals[KEY_SIGNATURE] =
     g_signal_new("key-signature",
@@ -746,7 +748,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_key_signature),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__UINT_UINT_BOOLEAN,
+		 ags_cclosure_marshal_VOID__UINT_UINT_BOOLEAN,
 		 G_TYPE_NONE, 3,
 		 G_TYPE_UINT,
 		 G_TYPE_UINT,
@@ -762,7 +764,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::sequencer-meta-event signal is emited during building of event.
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_builder_signals[SEQUENCER_META_EVENT] =
     g_signal_new("sequencer-meta-event",
@@ -770,7 +772,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_sequencer_meta_event),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__UINT_UINT_UINT_UINT,
+		 ags_cclosure_marshal_VOID__UINT_UINT_UINT_UINT,
 		 G_TYPE_NONE, 4,
 		 G_TYPE_UINT,
 		 G_TYPE_UINT,
@@ -786,7 +788,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
    *
    * The ::text-event signal is emited during building of event.
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   midi_builder_signals[TEXT_EVENT] =
     g_signal_new("text-event",
@@ -794,7 +796,7 @@ ags_midi_builder_class_init(AgsMidiBuilderClass *midi_builder)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(AgsMidiBuilderClass, append_text_event),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_VOID__UINT_STRING_UINT,
+		 ags_cclosure_marshal_VOID__UINT_STRING_UINT,
 		 G_TYPE_NONE, 3,
 		 G_TYPE_UINT,
 		 G_TYPE_STRING,
@@ -806,6 +808,21 @@ ags_midi_builder_init(AgsMidiBuilder *midi_builder)
 {
   midi_builder->flags = 0;
 
+  /* midi builder mutex */
+  midi_builder->obj_mutexattr = (pthread_mutexattr_t *) malloc(sizeof(pthread_mutexattr_t));
+  pthread_mutexattr_init(midi_builder->obj_mutexattr);
+  pthread_mutexattr_settype(midi_builder->obj_mutexattr,
+			    PTHREAD_MUTEX_RECURSIVE);
+
+#ifdef __linux__
+  pthread_mutexattr_setprotocol(midi_builder->obj_mutexattr,
+				PTHREAD_PRIO_INHERIT);
+#endif
+
+  midi_builder->obj_mutex =  (pthread_mutex_t *) malloc(sizeof(pthread_mutex_t));
+  pthread_mutex_init(midi_builder->obj_mutex,
+		     midi_builder->obj_mutexattr);
+
   midi_builder->data = NULL;
   midi_builder->length = 0;
 
@@ -825,12 +842,25 @@ ags_midi_builder_set_property(GObject *gobject,
 {
   AgsMidiBuilder *midi_builder;
 
+  pthread_mutex_t *midi_builder_mutex;
+
   midi_builder = AGS_MIDI_BUILDER(gobject);
+
+  /* get midi builder mutex */
+  pthread_mutex_lock(ags_midi_builder_get_class_mutex());
+  
+  midi_builder_mutex = midi_builder->obj_mutex;
+  
+  pthread_mutex_unlock(ags_midi_builder_get_class_mutex());
   
   switch(prop_id){
   case PROP_FILE:
     {
+      pthread_mutex_lock(midi_builder_mutex);
+
       midi_builder->file = g_value_get_pointer(value);
+
+      pthread_mutex_unlock(midi_builder_mutex);
     }
     break;
   default:
@@ -847,13 +877,26 @@ ags_midi_builder_get_property(GObject *gobject,
 {
   AgsMidiBuilder *midi_builder;
 
+  pthread_mutex_t *midi_builder_mutex;
+
   midi_builder = AGS_MIDI_BUILDER(gobject);
+
+  /* get midi builder mutex */
+  pthread_mutex_lock(ags_midi_builder_get_class_mutex());
+  
+  midi_builder_mutex = midi_builder->obj_mutex;
+  
+  pthread_mutex_unlock(ags_midi_builder_get_class_mutex());
   
   switch(prop_id){
   case PROP_FILE:
     {
+      pthread_mutex_lock(midi_builder_mutex);
+
       g_value_set_pointer(value,
 			  midi_builder->file);
+
+      pthread_mutex_unlock(midi_builder_mutex);
     }
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID(gobject, prop_id, param_spec);
@@ -864,9 +907,35 @@ ags_midi_builder_get_property(GObject *gobject,
 void
 ags_midi_builder_finalize(GObject *gobject)
 {
+  AgsMidiBuilder *midi_builder;
+    
+  midi_builder = (AgsMidiBuilder *) gobject;
+
+  pthread_mutex_destroy(midi_builder->obj_mutex);
+  free(midi_builder->obj_mutex);
+
+  pthread_mutexattr_destroy(midi_builder->obj_mutexattr);
+  free(midi_builder->obj_mutexattr);
+
+  /* call parent */
   G_OBJECT_CLASS(ags_midi_builder_parent_class)->finalize(gobject);
 }
 
+/**
+ * ags_midi_builder_get_class_mutex:
+ * 
+ * Use this function's returned mutex to access mutex fields.
+ *
+ * Returns: the class mutex
+ * 
+ * Since: 2.0.0
+ */
+pthread_mutex_t*
+ags_midi_builder_get_class_mutex()
+{
+  return(&ags_midi_builder_class_mutex);
+}
+
 /**
  * ags_midi_builder_header_alloc:
  * 
@@ -874,7 +943,7 @@ ags_midi_builder_finalize(GObject *gobject)
  * 
  * Returns: the newly allocated #AgsMidiBuilderHeader-struct
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 AgsMidiBuilderHeader*
 ags_midi_builder_header_alloc()
@@ -903,7 +972,7 @@ ags_midi_builder_header_alloc()
  * 
  * Free MIDI builder header.
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_midi_builder_header_free(AgsMidiBuilderHeader *midi_builder_header)
@@ -922,7 +991,7 @@ ags_midi_builder_header_free(AgsMidiBuilderHeader *midi_builder_header)
  * 
  * Returns: the newly allocated #AgsMidiBuilderTrack-struct
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 AgsMidiBuilderTrack*
 ags_midi_builder_track_alloc()
@@ -948,7 +1017,7 @@ ags_midi_builder_track_alloc()
  * 
  * Free MIDI builder track.
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_midi_builder_track_free(AgsMidiBuilderTrack *midi_builder_track)
@@ -978,7 +1047,7 @@ ags_midi_builder_track_free(AgsMidiBuilderTrack *midi_builder_track)
  * 
  * Returns: the matching #GList-struct
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 GList*
 ags_midi_builder_track_find_delta_time_with_track_name(GList *midi_builder_track,
@@ -1006,7 +1075,7 @@ ags_midi_builder_track_find_delta_time_with_track_name(GList *midi_builder_track
  * 
  * Insert MIDI message.
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_midi_builder_track_insert_midi_message(AgsMidiBuilderTrack *midi_builder_track,
@@ -1072,7 +1141,7 @@ ags_midi_builder_track_insert_midi_message(AgsMidiBuilderTrack *midi_builder_tra
  * 
  * Returns: the buffer position before @delta_time
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 unsigned char*
 ags_midi_builder_track_get_delta_time_offset(AgsMidiBuilderTrack *midi_builder_track,
@@ -1137,7 +1206,7 @@ ags_midi_builder_real_midi_putc(AgsMidiBuilder *midi_builder,
  *
  * Put char in MIDI file.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_midi_builder_midi_putc(AgsMidiBuilder *midi_builder,
@@ -1169,7 +1238,7 @@ ags_midi_builder_real_on_error(AgsMidiBuilder *midi_builder,
  *
  * Triggered as an error occurs.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_midi_builder_on_error(AgsMidiBuilder *midi_builder,
@@ -1231,7 +1300,7 @@ ags_midi_builder_real_append_header(AgsMidiBuilder *midi_builder,
  *
  * Appends MIDI header to @midi_builder.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_midi_builder_append_header(AgsMidiBuilder *midi_builder,
@@ -1299,7 +1368,7 @@ ags_midi_builder_real_append_track(AgsMidiBuilder *midi_builder,
  *
  * Append a track called @track_name to @midi_builder.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_midi_builder_append_track(AgsMidiBuilder *midi_builder,
@@ -1347,7 +1416,7 @@ ags_midi_builder_real_append_key_on(AgsMidiBuilder *midi_builder,
  *
  * Append key-on for @note to @midi_builder with key dynamics @velocity, at @delta_time.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_midi_builder_append_key_on(AgsMidiBuilder *midi_builder,
@@ -1401,7 +1470,7 @@ ags_midi_builder_real_append_key_off(AgsMidiBuilder *midi_builder,
  *
  * Append key-off for @note to @midi_builder with key dynamics @velocity, at @delta_time.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_midi_builder_append_key_off(AgsMidiBuilder *midi_builder,
@@ -1456,7 +1525,7 @@ ags_midi_builder_real_append_key_pressure(AgsMidiBuilder *midi_builder,
  *
  * Append key-pressure for @note to @midi_builder with key dynamics @pressure, at @delta_time.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_midi_builder_append_key_pressure(AgsMidiBuilder *midi_builder,
@@ -1510,7 +1579,7 @@ ags_midi_builder_real_append_change_parameter(AgsMidiBuilder *midi_builder,
  *
  * Appends change parameter.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_midi_builder_append_change_parameter(AgsMidiBuilder *midi_builder,
@@ -1564,7 +1633,7 @@ ags_midi_builder_real_append_change_pitch_bend(AgsMidiBuilder *midi_builder,
  *
  * Change pitch bend.
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_midi_builder_append_change_pitch_bend(AgsMidiBuilder *midi_builder,
@@ -1615,7 +1684,7 @@ ags_midi_builder_real_append_change_program(AgsMidiBuilder *midi_builder,
  *
  * Appends change program.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_midi_builder_append_change_program(AgsMidiBuilder *midi_builder,
@@ -1664,7 +1733,7 @@ ags_midi_builder_real_append_change_pressure(AgsMidiBuilder *midi_builder,
  * 
  * Appends change pressure.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_midi_builder_append_change_pressure(AgsMidiBuilder *midi_builder,
@@ -1711,7 +1780,7 @@ ags_midi_builder_real_append_sysex(AgsMidiBuilder *midi_builder,
  *
  * Appends sysex data.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_midi_builder_append_sysex(AgsMidiBuilder *midi_builder,
@@ -1758,7 +1827,7 @@ ags_midi_builder_real_append_quarter_frame(AgsMidiBuilder *midi_builder,
  *
  * Appends quarter frame.
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_midi_builder_append_quarter_frame(AgsMidiBuilder *midi_builder,
@@ -1803,7 +1872,7 @@ ags_midi_builder_real_append_song_position(AgsMidiBuilder *midi_builder,
  * 
  * Appends song position.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_midi_builder_append_song_position(AgsMidiBuilder *midi_builder,
@@ -1847,7 +1916,7 @@ ags_midi_builder_real_append_song_select(AgsMidiBuilder *midi_builder,
  * 
  * Appends song select.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_midi_builder_append_song_select(AgsMidiBuilder *midi_builder,
@@ -1888,7 +1957,7 @@ ags_midi_builder_real_append_tune_request(AgsMidiBuilder *midi_builder,
  * 
  * Appends tune request.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_midi_builder_append_tune_request(AgsMidiBuilder *midi_builder,
@@ -1930,7 +1999,7 @@ ags_midi_builder_real_append_sequence_number(AgsMidiBuilder *midi_builder,
  * 
  * Appends sequence number.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_midi_builder_append_sequence_number(AgsMidiBuilder *midi_builder,
@@ -1978,7 +2047,7 @@ ags_midi_builder_real_append_smtpe(AgsMidiBuilder *midi_builder,
  * 
  * Appends smtpe.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_midi_builder_append_smtpe(AgsMidiBuilder *midi_builder,
@@ -2023,7 +2092,7 @@ ags_midi_builder_real_append_tempo(AgsMidiBuilder *midi_builder,
  * 
  * Appends tempo.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_midi_builder_append_tempo(AgsMidiBuilder *midi_builder,
@@ -2070,7 +2139,7 @@ ags_midi_builder_real_append_time_signature(AgsMidiBuilder *midi_builder,
  *
  * Appends time signature.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_midi_builder_append_time_signature(AgsMidiBuilder *midi_builder,
@@ -2115,7 +2184,7 @@ ags_midi_builder_real_append_key_signature(AgsMidiBuilder *midi_builder,
  *
  * Appends key signature.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_midi_builder_append_key_signature(AgsMidiBuilder *midi_builder,
@@ -2161,7 +2230,7 @@ ags_midi_builder_real_append_sequencer_meta_event(AgsMidiBuilder *midi_builder,
  * 
  * Appends sequencer meta event.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_midi_builder_append_sequencer_meta_event(AgsMidiBuilder *midi_builder,
@@ -2212,7 +2281,7 @@ ags_midi_builder_real_append_text_event(AgsMidiBuilder *midi_builder,
  *
  * Appends text event.
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_midi_builder_append_text_event(AgsMidiBuilder *midi_builder,
@@ -2236,7 +2305,7 @@ ags_midi_builder_append_text_event(AgsMidiBuilder *midi_builder,
  * 
  * Append from XML node @node.
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_midi_builder_append_xml_node(AgsMidiBuilder *midi_builder,
@@ -3032,7 +3101,7 @@ ags_midi_builder_append_xml_node(AgsMidiBuilder *midi_builder,
  *
  * Builds from XML document @doc.
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_midi_builder_from_xml_doc(AgsMidiBuilder *midi_builder,
@@ -3075,7 +3144,7 @@ ags_midi_builder_from_xml_doc(AgsMidiBuilder *midi_builder,
  *
  * Build the MIDI data.
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_midi_builder_build(AgsMidiBuilder *midi_builder)
@@ -3152,11 +3221,11 @@ ags_midi_builder_build(AgsMidiBuilder *midi_builder)
  * ags_midi_builder_new:
  * @file: the FILE handle
  * 
- * Creates a new #AgsMidiBuilder
+ * Creates a new instance of #AgsMidiBuilder
  *
- * Returns: an #AgsMidiBuilder
+ * Returns: the new #AgsMidiBuilder
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 AgsMidiBuilder*
 ags_midi_builder_new(FILE *file)
