@@ -1,5 +1,5 @@
 /* GSequencer - Advanced GTK Sequencer
- * Copyright (C) 2005-2015 Joël Krähemann
+ * Copyright (C) 2005-2018 Joël Krähemann
  *
  * This file is part of GSequencer.
  *
@@ -56,13 +56,15 @@ enum{
 
 static gpointer ags_midi_file_parent_class = NULL;
 
+static pthread_mutex_t ags_midi_file_class_mutex = PTHREAD_MUTEX_INITIALIZER;
+
 GType
 ags_midi_file_get_type(void)
 {
   static volatile gsize g_define_type_id__volatile = 0;
 
   if(g_once_init_enter (&g_define_type_id__volatile)){
-    GType ags_type_midi_file;
+    GType ags_type_midi_file = 0;
 
     static const GTypeInfo ags_midi_file_info = {
       sizeof (AgsMidiFileClass),
@@ -80,7 +82,7 @@ ags_midi_file_get_type(void)
 						"AgsMidiFile", &ags_midi_file_info,
 						0);
 
-    g_once_init_leave (&g_define_type_id__volatile, ags_type_midi_file);
+    g_once_init_leave(&g_define_type_id__volatile, ags_type_midi_file);
   }
 
   return g_define_type_id__volatile;
@@ -108,7 +110,7 @@ ags_midi_file_class_init(AgsMidiFileClass *midi_file)
    *
    * The assigned filename to perform input/output on.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_string("filename",
 				   i18n_pspec("assigned filename"),
@@ -120,11 +122,32 @@ ags_midi_file_class_init(AgsMidiFileClass *midi_file)
 				  param_spec);
 }
 
+GQuark
+ags_midi_file_error_quark()
+{
+  return(g_quark_from_static_string("ags-midi-file-error-quark"));
+}
+
 void
 ags_midi_file_init(AgsMidiFile *midi_file)
 {
   midi_file->flags = 0;
-  
+
+  /* midi file mutex */
+  midi_file->obj_mutexattr = (pthread_mutexattr_t *) malloc(sizeof(pthread_mutexattr_t));
+  pthread_mutexattr_init(midi_file->obj_mutexattr);
+  pthread_mutexattr_settype(midi_file->obj_mutexattr,
+			    PTHREAD_MUTEX_RECURSIVE);
+
+#ifdef __linux__
+  pthread_mutexattr_setprotocol(midi_file->obj_mutexattr,
+				PTHREAD_PRIO_INHERIT);
+#endif
+
+  midi_file->obj_mutex =  (pthread_mutex_t *) malloc(sizeof(pthread_mutex_t));
+  pthread_mutex_init(midi_file->obj_mutex,
+		     midi_file->obj_mutexattr);
+
   midi_file->file = NULL;
   midi_file->filename = NULL;
   
@@ -142,6 +165,9 @@ ags_midi_file_init(AgsMidiFile *midi_file)
   midi_file->track = NULL;
 
   midi_file->current_track = NULL;
+
+  midi_file->notation = NULL;
+  midi_file->midi = NULL;
 }
 
 void
@@ -152,7 +178,16 @@ ags_midi_file_set_property(GObject *gobject,
 {
   AgsMidiFile *midi_file;
 
+  pthread_mutex_t *midi_file_mutex;
+
   midi_file = AGS_MIDI_FILE(gobject);
+
+  /* get midi file mutex */
+  pthread_mutex_lock(ags_midi_file_get_class_mutex());
+  
+  midi_file_mutex = midi_file->obj_mutex;
+  
+  pthread_mutex_unlock(ags_midi_file_get_class_mutex());
   
   switch(prop_id){
   case PROP_FILENAME:
@@ -161,7 +196,11 @@ ags_midi_file_set_property(GObject *gobject,
 
       filename = g_value_get_string(value);
       
+      pthread_mutex_lock(midi_file_mutex);
+
       if(filename == midi_file->filename){
+	pthread_mutex_unlock(midi_file_mutex);
+	
 	return;
       }
 
@@ -170,6 +209,8 @@ ags_midi_file_set_property(GObject *gobject,
       }
 
       midi_file->filename = g_strdup(filename);
+
+      pthread_mutex_unlock(midi_file_mutex);
     }
     break;
   default:
@@ -186,13 +227,26 @@ ags_midi_file_get_property(GObject *gobject,
 {
   AgsMidiFile *midi_file;
 
+  pthread_mutex_t *midi_file_mutex;
+
   midi_file = AGS_MIDI_FILE(gobject);
+
+  /* get midi file mutex */
+  pthread_mutex_lock(ags_midi_file_get_class_mutex());
+  
+  midi_file_mutex = midi_file->obj_mutex;
+  
+  pthread_mutex_unlock(ags_midi_file_get_class_mutex());
   
   switch(prop_id){
   case PROP_FILENAME:
     {
+      pthread_mutex_lock(midi_file_mutex);
+
       g_value_set_string(value,
 			 midi_file->filename);
+
+      pthread_mutex_unlock(midi_file_mutex);
     }
     break;
   default:
@@ -209,28 +263,61 @@ ags_midi_file_finalize(GObject *gobject)
   midi_file = (AgsMidiFile *) gobject;
   
   g_free(midi_file->filename);
-  
+
+  pthread_mutex_destroy(midi_file->obj_mutex);
+  free(midi_file->obj_mutex);
+
+  pthread_mutexattr_destroy(midi_file->obj_mutexattr);
+  free(midi_file->obj_mutexattr);
+
+  /* call parent */
   G_OBJECT_CLASS(ags_midi_file_parent_class)->finalize(gobject);
 }
 
+/**
+ * ags_midi_file_get_class_mutex:
+ * 
+ * Use this function's returned mutex to access mutex fields.
+ *
+ * Returns: the class mutex
+ * 
+ * Since: 2.0.0
+ */
+pthread_mutex_t*
+ags_midi_file_get_class_mutex()
+{
+  return(&ags_midi_file_class_mutex);
+}
+
 /**
  * ags_midi_file_open:
  * @midi_file: the #AgsMidiFile
+ * @filename: the filename
  *
  * Opens a MIDI file read-only.
  *
  * Returns: %TRUE on success, otherwise %FALSE
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 gboolean
-ags_midi_file_open(AgsMidiFile *midi_file)
+ags_midi_file_open(AgsMidiFile *midi_file,
+		   gchar *filename)
 {
-  if(midi_file == NULL ||
+  if(!AGS_IS_MIDI_FILE(midi_file) ||
      midi_file->filename == NULL){
     return(FALSE);
   }
 
+  g_object_set(midi_file,
+	       "filename", filename,
+	       NULL);
+  
+  if(!g_file_test(filename,
+		  (G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR))){
+    return(FALSE);
+  }
+
   midi_file->file = fopen(midi_file->filename,
 			  "r");
 
@@ -247,13 +334,13 @@ ags_midi_file_open(AgsMidiFile *midi_file)
  *
  * Returns: %TRUE on success, otherwise %FALSE
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 gboolean
 ags_midi_file_open_from_data(AgsMidiFile *midi_file,
 			     unsigned char *data, guint buffer_length)
 {
-  if(midi_file == NULL){
+  if(!AGS_IS_MIDI_FILE(midi_file)){
     return(FALSE);
   }
 
@@ -266,18 +353,32 @@ ags_midi_file_open_from_data(AgsMidiFile *midi_file,
 /**
  * ags_midi_file_rw_open:
  * @midi_file: the #AgsMidiFile
+ * @filename: the filename
+ * @create: %TRUE create file if not exists, else if %FALSE return
  * 
  * Opens a MIDI file with read-write permission.
  *
  * Returns: %TRUE on success, otherwise %FALSE
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 gboolean
-ags_midi_file_rw_open(AgsMidiFile *midi_file)
+ags_midi_file_rw_open(AgsMidiFile *midi_file,
+		      gchar *filename,
+		      gboolean create)
 {
-  if(midi_file == NULL ||
-     midi_file->filename == NULL){
+  if(!AGS_IS_MIDI_FILE(midi_file) ||
+     filename == NULL){
+    return(FALSE);
+  }
+
+  g_object_set(midi_file,
+	       "filename", filename,
+	       NULL);
+  
+  if(!create &&
+     !g_file_test(filename,
+		  (G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR))){
     return(FALSE);
   }
   
@@ -293,12 +394,12 @@ ags_midi_file_rw_open(AgsMidiFile *midi_file)
  *
  * Closes the file stream.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_midi_file_close(AgsMidiFile *midi_file)
 {
-  if(midi_file == NULL ||
+  if(!AGS_IS_MIDI_FILE(midi_file) ||
      midi_file->file == NULL){
     return;
   }
@@ -316,7 +417,7 @@ ags_midi_file_close(AgsMidiFile *midi_file)
  *
  * Returns: the data array just read
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 unsigned char*
 ags_midi_file_read(AgsMidiFile *midi_file)
@@ -325,7 +426,7 @@ ags_midi_file_read(AgsMidiFile *midi_file)
 
   size_t n_read;
   
-  if(midi_file == NULL ||
+  if(!AGS_IS_MIDI_FILE(midi_file) ||
      midi_file->filename == NULL ||
      midi_file->file == NULL){
     return(NULL);
@@ -333,6 +434,10 @@ ags_midi_file_read(AgsMidiFile *midi_file)
   
   stat(midi_file->filename, &sb);
 
+  if(sb.st_size == 0){
+    return(NULL);
+  }
+  
   midi_file->buffer_length = sb.st_size + 1;
   midi_file->buffer = (unsigned char *) malloc(midi_file->buffer_length * sizeof(unsigned char));
   midi_file->buffer[sb.st_size] = EOF;
@@ -355,7 +460,7 @@ ags_midi_file_read(AgsMidiFile *midi_file)
  *
  * Writes @data to the file stream and to internal buffer, reallocates it if necessary.
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_midi_file_write(AgsMidiFile *midi_file,
@@ -363,7 +468,7 @@ ags_midi_file_write(AgsMidiFile *midi_file,
 {
   unsigned char *start;
   
-  if(midi_file == NULL ||
+  if(!AGS_IS_MIDI_FILE(midi_file) ||
      midi_file->file == NULL){
     return;
   }
@@ -396,12 +501,12 @@ ags_midi_file_write(AgsMidiFile *midi_file,
  *
  * Seeks the file stream's offset.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_midi_file_seek(AgsMidiFile *midi_file, guint position, gint whence)
 {
-  if(midi_file == NULL ||
+  if(!AGS_IS_MIDI_FILE(midi_file) ||
      midi_file->file == NULL){
     return;
   }
@@ -415,12 +520,12 @@ ags_midi_file_seek(AgsMidiFile *midi_file, guint position, gint whence)
  * 
  * Flushes file stream's data buffer to disc.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_midi_file_flush(AgsMidiFile *midi_file)
 {
-  if(midi_file == NULL ||
+  if(!AGS_IS_MIDI_FILE(midi_file) ||
      midi_file->file == NULL){
     return;
   }
@@ -428,29 +533,85 @@ ags_midi_file_flush(AgsMidiFile *midi_file)
   fflush(midi_file->file);
 }
 
+/**
+ * ags_midi_file_read_byte:
+ * @midi_file: the #AgsMidiFile
+ * @error: the #GError pointer return location
+ *  
+ * Reads a unsigned char quantity.
+ *
+ * Returns: the current value at file's iteration pointer
+ * 
+ * Since: 2.0.0
+ */
+unsigned char
+ags_midi_file_read_byte(AgsMidiFile *midi_file,
+			GError **error)
+{
+  unsigned char value;
+
+  if(!AGS_IS_MIDI_FILE(midi_file)){
+    return(0x0);
+  }
+
+  if(midi_file->iter + 1 < midi_file->buffer + midi_file->buffer_length){
+    value = (midi_file->iter[0]);
+    
+    midi_file->iter += 1;
+  }else{
+    if(error != NULL){
+      g_set_error(error,
+		  AGS_MIDI_FILE_ERROR,
+		  AGS_MIDI_FILE_ERROR_PREMATURE_EOF,
+		  "no more data available in file buffer");
+    }
+    
+    return(0x0);
+  }
+  
+  return(value);
+}
+
 /**
  * ags_midi_file_read_gint16:
  * @midi_file: the #AgsMidiFile
+ * @error: the #GError pointer return location
  *  
  * Reads a gint16 quantity.
  *
  * Returns: the current value at file's iteration pointer
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 gint16
-ags_midi_file_read_gint16(AgsMidiFile *midi_file)
+ags_midi_file_read_gint16(AgsMidiFile *midi_file,
+			  GError **error)
 {
   unsigned char str[2];
   gint16 value = 0;
 
-  str[0] = (midi_file->iter[0]);
-  str[1] = (midi_file->iter[1]);
+  if(!AGS_IS_MIDI_FILE(midi_file)){
+    return(0x0);
+  }
 
-  midi_file->iter += 2;
-  
-  value = (str[0] & 0xff);
-  value = (value<<8) + (str[1] & 0xff);
+  if(midi_file->iter + 2 < midi_file->buffer + midi_file->buffer_length){
+    str[0] = (midi_file->iter[0]);
+    str[1] = (midi_file->iter[1]);
+    
+    midi_file->iter += 2;
+    
+    value = (str[0] & 0xff);
+    value = (value<<8) + (str[1] & 0xff);
+  }else{
+    if(error != NULL){
+      g_set_error(error,
+		  AGS_MIDI_FILE_ERROR,
+		  AGS_MIDI_FILE_ERROR_PREMATURE_EOF,
+		  "no more data available in file buffer");
+    }
+    
+    return(0x0);
+  }
   
   return(value);
 }
@@ -458,29 +619,46 @@ ags_midi_file_read_gint16(AgsMidiFile *midi_file)
 /**
  * ags_midi_file_read_gint24:
  * @midi_file: the #AgsMidiFile
+ * @error: the #GError pointer return location
  *  
  * Reads a 24-bit quantity.
  *
  * Returns: the current value at file's iteration pointer
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 gint32
-ags_midi_file_read_gint24(AgsMidiFile *midi_file)
+ags_midi_file_read_gint24(AgsMidiFile *midi_file,
+			  GError **error)
 {
   unsigned char str[4];
   gint32 value = 0;
-  
-  str[0] = (unsigned char) 0x00;
-  str[1] = (midi_file->iter[0]);
-  str[2] = (midi_file->iter[1]);
-  str[3] = (midi_file->iter[2]);
 
-  midi_file->iter += 3;
+  if(!AGS_IS_MIDI_FILE(midi_file)){
+    return(0x0);
+  }
   
-  value = (value<<8) + (str[1] & 0xff);
-  value = (value<<8) + (str[2] & 0xff);
-  value = (value<<8) + (str[3] & 0xff);
+  if(midi_file->iter + 3 < midi_file->buffer + midi_file->buffer_length){
+    str[0] = (unsigned char) 0x00;
+    str[1] = (midi_file->iter[0]);
+    str[2] = (midi_file->iter[1]);
+    str[3] = (midi_file->iter[2]);
+
+    midi_file->iter += 3;
+  
+    value = (value<<8) + (str[1] & 0xff);
+    value = (value<<8) + (str[2] & 0xff);
+    value = (value<<8) + (str[3] & 0xff);
+  }else{  
+    if(error != NULL){
+      g_set_error(error,
+		  AGS_MIDI_FILE_ERROR,
+		  AGS_MIDI_FILE_ERROR_PREMATURE_EOF,
+		  "no more data available in file buffer");
+    }
+    
+    return(0x0);
+  }
   
   return(value);
 }
@@ -488,30 +666,47 @@ ags_midi_file_read_gint24(AgsMidiFile *midi_file)
 /**
  * ags_midi_file_read_gint32:
  * @midi_file: the #AgsMidiFile
+ * @error: the #GError pointer return location
  *  
  * Reads a gint32 quantity.
  *
  * Returns: the current value at file's iteration pointer
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 gint32
-ags_midi_file_read_gint32(AgsMidiFile *midi_file)
+ags_midi_file_read_gint32(AgsMidiFile *midi_file,
+			  GError **error)
 {
   unsigned char str[4];
   gint32 value;
   
-  str[0] = (midi_file->iter[0]);
-  str[1] = (midi_file->iter[1]);
-  str[2] = (midi_file->iter[2]);
-  str[3] = (midi_file->iter[3]);
+  if(!AGS_IS_MIDI_FILE(midi_file)){
+    return(0x0);
+  }
   
-  midi_file->iter += 4;
+  if(midi_file->iter + 4 < midi_file->buffer + midi_file->buffer_length){
+    str[0] = (midi_file->iter[0]);
+    str[1] = (midi_file->iter[1]);
+    str[2] = (midi_file->iter[2]);
+    str[3] = (midi_file->iter[3]);
   
-  value = (str[0] & 0xff);
-  value = (value<<8) + (str[1] & 0xff);
-  value = (value<<8) + (str[2] & 0xff);
-  value = (value<<8) + (str[3] & 0xff);
+    midi_file->iter += 4;
+  
+    value = (str[0] & 0xff);
+    value = (value<<8) + (str[1] & 0xff);
+    value = (value<<8) + (str[2] & 0xff);
+    value = (value<<8) + (str[3] & 0xff);
+  }else{
+    if(error != NULL){
+      g_set_error(error,
+		  AGS_MIDI_FILE_ERROR,
+		  AGS_MIDI_FILE_ERROR_PREMATURE_EOF,
+		  "no more data available in file buffer");
+    }
+    
+    return(0x0);
+  }
   
   return(value);
 }
@@ -519,35 +714,64 @@ ags_midi_file_read_gint32(AgsMidiFile *midi_file)
 /**
  * ags_midi_file_read_varlength:
  * @midi_file: the #AgsMidiFile
+ * @error: the #GError pointer return location
  *  
  * Reads a variable length quantity.
  *
  * Returns: the current value at file's iteration pointer
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 long
-ags_midi_file_read_varlength(AgsMidiFile *midi_file)
+ags_midi_file_read_varlength(AgsMidiFile *midi_file,
+			     GError **error)
 {
   long value;
   guint i;
   unsigned char c;
+  gboolean success;
   
-  c = midi_file->iter[0];
-  value = c;
-  i = 1;
+  if(!AGS_IS_MIDI_FILE(midi_file)){
+    return(0x0);
+  }
 
-  midi_file->iter += 1;
+  success = TRUE;
   
-  if(c & 0x80){
+  if(midi_file->iter + 1 < midi_file->buffer + midi_file->buffer_length){
+    c = midi_file->iter[0];
+    value = c;
+    i = 1;
+
+    midi_file->iter += 1;
+  }else{
+    success = FALSE;
+  }
+  
+  if((c & 0x80) &&
+     success){
     value &= 0x7F;
    
     do{
-      //TODO:JK: unsafe
-      value = (value << 7) + ((c = (midi_file->iter[0])) & 0x7F);
-      i++;
-      midi_file->iter += 1;
-    }while(c & 0x80);
+      if(midi_file->iter + 1 < midi_file->buffer + midi_file->buffer_length){
+	//TODO:JK: unsafe
+	value = (value << 7) + ((c = (midi_file->iter[0])) & 0x7F);
+	i++;
+	midi_file->iter += 1;
+      }else{
+	success = FALSE;
+      }
+    }while((c & 0x80) && success);
+  }
+
+  if(!success){
+    if(error != NULL){
+      g_set_error(error,
+		  AGS_MIDI_FILE_ERROR,
+		  AGS_MIDI_FILE_ERROR_PREMATURE_EOF,
+		  "no more data available in file buffer");
+    }
+    
+    return(0x0);
   }
   
   return(value);
@@ -557,21 +781,38 @@ ags_midi_file_read_varlength(AgsMidiFile *midi_file)
  * ags_midi_file_read_text:
  * @midi_file: the #AgsMidiFile
  * @length: the number of bytes to be read, or as long valid string for -1
+ * @error: the #GError pointer return location
  *  
  * Reads a string.
  *
  * Returns: the string at file's iteration pointer
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 unsigned char*
 ags_midi_file_read_text(AgsMidiFile *midi_file,
-			gint length)
+			gint length,
+			GError **error)
 {
   unsigned char *text;
   gchar c;
   guint i;
+  
+  if(!AGS_IS_MIDI_FILE(midi_file)){
+    return(NULL);
+  }
 
+  if(midi_file->iter + length >= midi_file->buffer + midi_file->buffer_length){
+    if(error != NULL){
+      g_set_error(error,
+		  AGS_MIDI_FILE_ERROR,
+		  AGS_MIDI_FILE_ERROR_PREMATURE_EOF,
+		  "no more data available in file buffer");
+    }
+
+    return(NULL);
+  }
+  
   text = (unsigned char *) malloc((AGS_MIDI_FILE_MAX_TEXT_LENGTH + 1) * sizeof(unsigned char));
   memset(text, 0, AGS_MIDI_FILE_MAX_TEXT_LENGTH * sizeof(unsigned char));
   i = 0;
@@ -595,6 +836,34 @@ ags_midi_file_read_text(AgsMidiFile *midi_file,
   return(text);
 }
 
+/**
+ * ags_midi_file_write_byte:
+ * @midi_file: the #AgsMidiFile
+ * @val: the value to write
+ *
+ * Writes a unsigned char quantity to internal buffer.
+ *
+ * Since: 2.0.0
+ */
+void
+ags_midi_file_write_byte(AgsMidiFile *midi_file, unsigned char val)
+{
+  if(midi_file->iter + 1 >= (midi_file->buffer + midi_file->buffer_length)){
+    size_t new_length;
+
+    new_length = (midi_file->iter + 1) - (midi_file->buffer + midi_file->buffer_length);
+    
+    midi_file->buffer = (unsigned char *) realloc(midi_file->buffer,
+						  new_length * sizeof(unsigned char));
+
+    midi_file->buffer_length = new_length;
+  }
+
+  midi_file->iter[0] = val;
+  
+  midi_file->iter += 1;
+}
+
 /**
  * ags_midi_file_write_gint16:
  * @midi_file: the #AgsMidiFile
@@ -602,15 +871,15 @@ ags_midi_file_read_text(AgsMidiFile *midi_file,
  *
  * Writes a gint16 quantity to internal buffer.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_midi_file_write_gint16(AgsMidiFile *midi_file, gint16 val)
 {
-  if(&(midi_file->iter[2]) >= &(midi_file->buffer[midi_file->buffer_length])){
+  if((midi_file->iter + 2) >= (midi_file->buffer + midi_file->buffer_length)){
     size_t new_length;
 
-    new_length = &(midi_file->iter[2]) - &(midi_file->buffer[midi_file->buffer_length]);
+    new_length = (midi_file->iter + 2) - (midi_file->buffer + midi_file->buffer_length);
     
     midi_file->buffer = (unsigned char *) realloc(midi_file->buffer,
 						  new_length * sizeof(unsigned char));
@@ -631,15 +900,15 @@ ags_midi_file_write_gint16(AgsMidiFile *midi_file, gint16 val)
  *
  * Writes a 24-bit quantity to internal buffer.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_midi_file_write_gint24(AgsMidiFile *midi_file, gint32 val)
 {
-  if(&(midi_file->iter[3]) >= &(midi_file->buffer[midi_file->buffer_length])){
+  if((midi_file->iter + 3) >= (midi_file->buffer + midi_file->buffer_length)){
     size_t new_length;
 
-    new_length = &(midi_file->iter[3]) - &(midi_file->buffer[midi_file->buffer_length]);
+    new_length = (midi_file->iter + 3) - (midi_file->buffer + midi_file->buffer_length);
     
     midi_file->buffer = (unsigned char *) realloc(midi_file->buffer,
 						  new_length * sizeof(unsigned char));
@@ -661,15 +930,15 @@ ags_midi_file_write_gint24(AgsMidiFile *midi_file, gint32 val)
  *
  * Writes a gint32 quantity to internal buffer.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_midi_file_write_gint32(AgsMidiFile *midi_file, gint32 val)
 {
-  if(&(midi_file->iter[4]) >= &(midi_file->buffer[midi_file->buffer_length])){
+  if((midi_file->iter + 4) >= (midi_file->buffer + midi_file->buffer_length)){
     size_t new_length;
 
-    new_length = &(midi_file->iter[4]) - &(midi_file->buffer[midi_file->buffer_length]);
+    new_length = (midi_file->iter + 4) - (midi_file->buffer + midi_file->buffer_length);
     
     midi_file->buffer = (unsigned char *) realloc(midi_file->buffer,
 						  new_length * sizeof(unsigned char));
@@ -692,7 +961,7 @@ ags_midi_file_write_gint32(AgsMidiFile *midi_file, gint32 val)
  *
  * Writes a variable length quantity to internal buffer.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_midi_file_write_varlength(AgsMidiFile *midi_file, long val)
@@ -712,10 +981,10 @@ ags_midi_file_write_varlength(AgsMidiFile *midi_file, long val)
   }while(0x80 & c);
 
   /* realloc buffer if needed */
-  if(&(midi_file->iter[i]) >= &(midi_file->buffer[midi_file->buffer_length])){
+  if((midi_file->iter + i) >= (midi_file->buffer + midi_file->buffer_length)){
     size_t new_length;
 
-    new_length = &(midi_file->iter[i]) - &(midi_file->buffer[midi_file->buffer_length]);
+    new_length = (midi_file->iter + i) - (midi_file->buffer + midi_file->buffer_length);
     
     midi_file->buffer = (unsigned char *) realloc(midi_file->buffer,
 						  new_length * sizeof(unsigned char));
@@ -739,7 +1008,7 @@ ags_midi_file_write_varlength(AgsMidiFile *midi_file, long val)
  *
  * Writes a string to internal buffer up to length bytes.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_midi_file_write_text(AgsMidiFile *midi_file,
@@ -751,10 +1020,10 @@ ags_midi_file_write_text(AgsMidiFile *midi_file,
     return;
   }
 
-  if(&(midi_file->iter[length]) >= &(midi_file->buffer[midi_file->buffer_length])){
+  if((midi_file->iter + length) >= (midi_file->buffer + midi_file->buffer_length)){
     size_t new_length;
 
-    new_length = &(midi_file->iter[length]) - &(midi_file->buffer[midi_file->buffer_length]);
+    new_length = (midi_file->iter + length) - (midi_file->buffer + midi_file->buffer_length);
     midi_file->buffer = (unsigned char *) realloc(midi_file->buffer,
 						  new_length * sizeof(unsigned char));
 
@@ -772,16 +1041,18 @@ ags_midi_file_write_text(AgsMidiFile *midi_file,
  * ags_midi_file_read_header:
  * @midi_file: the #AgsMidiFile
  * @buffer_length: pointer to return buffer length or %NULL
+ * @error: the #GError pointer return location
  *
  * Reads the MIDI file's header and positions internal buffer pointer just behind it.
  *
  * Returns: the header's bytes
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 unsigned char*
 ags_midi_file_read_header(AgsMidiFile *midi_file,
-			  guint *buffer_length)
+			  guint *buffer_length,
+			  GError **error)
 {
   static gchar header[] = "MThd";
 
@@ -791,7 +1062,9 @@ ags_midi_file_read_header(AgsMidiFile *midi_file,
   guint n;
   gchar c;
 
-  if(midi_file == NULL){
+  GError *local_error;
+  
+  if(!AGS_IS_MIDI_FILE(midi_file)){
     if(buffer_length != NULL){
       *buffer_length = 0;
     }
@@ -821,10 +1094,21 @@ ags_midi_file_read_header(AgsMidiFile *midi_file,
   length += 4;
 
   /* get some values */
-  midi_file->offset = (guint) ags_midi_file_read_gint32(midi_file);
-  midi_file->format = (guint) ags_midi_file_read_gint16(midi_file);
-  midi_file->count = (guint) ags_midi_file_read_gint16(midi_file);
-  midi_file->division = (guint) ags_midi_file_read_gint16(midi_file);
+  local_error = NULL;
+  midi_file->offset = (guint) ags_midi_file_read_gint32(midi_file,
+							&local_error);
+  
+  local_error = NULL;
+  midi_file->format = (guint) ags_midi_file_read_gint16(midi_file,
+							&local_error);
+
+  local_error = NULL;
+  midi_file->count = (guint) ags_midi_file_read_gint16(midi_file,
+						       &local_error);
+
+  local_error = NULL;
+  midi_file->division = (guint) ags_midi_file_read_gint16(midi_file,
+							  &local_error);
 
   if((midi_file->division) & 0x8000){
     /* SMPTE */
@@ -852,7 +1136,7 @@ ags_midi_file_read_header(AgsMidiFile *midi_file,
  *
  * Write header bytes.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_midi_file_write_header(AgsMidiFile *midi_file,
@@ -860,14 +1144,14 @@ ags_midi_file_write_header(AgsMidiFile *midi_file,
 {
   guint i;
   
-  if(midi_file == NULL){
+  if(!AGS_IS_MIDI_FILE(midi_file)){
     return;
   }
 
-  if(&(midi_file->iter[length]) >= &(midi_file->buffer[midi_file->buffer_length])){
+  if((midi_file->iter + length) >= (midi_file->buffer + midi_file->buffer_length)){
     size_t new_length;
 
-    new_length = &(midi_file->iter[length]) - &(midi_file->buffer[midi_file->buffer_length]);
+    new_length = (midi_file->iter + length) - (midi_file->buffer + midi_file->buffer_length);
     midi_file->buffer = (unsigned char *) realloc(midi_file->buffer,
 						  new_length * sizeof(unsigned char));
 
@@ -881,9 +1165,22 @@ ags_midi_file_write_header(AgsMidiFile *midi_file,
   midi_file->iter += length;
 }
 
+/**
+ * ags_midi_file_read_header:
+ * @midi_file: the #AgsMidiFile
+ * @buffer_length: pointer to return buffer length or %NULL
+ * @error: the #GError pointer return location
+ *
+ * Reads the MIDI file's track data.
+ *
+ * Returns: the track's bytes
+ *
+ * Since: 2.0.0
+ */
 unsigned char*
 ags_midi_file_read_track_data(AgsMidiFile *midi_file,
-			      guint *buffer_length)
+			      guint *buffer_length,
+			      GError **error)
 {
   gchar *track_name;
   unsigned char *data, *start;
@@ -896,8 +1193,10 @@ ags_midi_file_read_track_data(AgsMidiFile *midi_file,
   gboolean end_of_track;
 
   static gchar track[] = "MTrk";
+
+  GError *local_error;
   
-  if(midi_file == NULL){
+  if(!AGS_IS_MIDI_FILE(midi_file)){
     if(buffer_length != NULL){
       *buffer_length = 0;
     }
@@ -927,12 +1226,16 @@ ags_midi_file_read_track_data(AgsMidiFile *midi_file,
   start = midi_file->iter - 4;
   
   /* offset */
-  ags_midi_file_read_gint32(midi_file);
+  local_error = NULL;
+  ags_midi_file_read_gint32(midi_file,
+			    &local_error);
 
   end_of_track = FALSE;
   
   while(!end_of_track){
-    delta_time = ags_midi_file_read_varlength(midi_file);
+    local_error = NULL;
+    delta_time = ags_midi_file_read_varlength(midi_file,
+					      &local_error);
     
     status = midi_file->iter[0];
     midi_file->iter += 1;
@@ -1033,9 +1336,14 @@ ags_midi_file_read_track_data(AgsMidiFile *midi_file,
 	      guint text_length;
 	      
 	      /* These are all text events */
-	      text_length = ags_midi_file_read_varlength(midi_file);
+	      local_error = NULL;
+	      text_length = ags_midi_file_read_varlength(midi_file,
+							 &local_error);
+
+	      local_error = NULL;
 	      text = ags_midi_file_read_text(midi_file,
-					     text_length);
+					     text_length,
+					     &local_error);
 
 	      g_free(text);
 	    }
@@ -1143,14 +1451,14 @@ ags_midi_file_write_track_data(AgsMidiFile *midi_file,
 {
   guint i;
   
-  if(midi_file == NULL){
+  if(!AGS_IS_MIDI_FILE(midi_file)){
     return;
   }
 
-  if(&(midi_file->iter[length]) >= &(midi_file->buffer[midi_file->buffer_length])){
+  if((midi_file->iter + length) >= (midi_file->buffer + midi_file->buffer_length)){
     size_t new_length;
 
-    new_length = &(midi_file->iter[length]) - &(midi_file->buffer[midi_file->buffer_length]);
+    new_length = (midi_file->iter + length) - (midi_file->buffer + midi_file->buffer_length);
     midi_file->buffer = (unsigned char *) realloc(midi_file->buffer,
 						  new_length * sizeof(unsigned char));
 
@@ -1164,6 +1472,28 @@ ags_midi_file_write_track_data(AgsMidiFile *midi_file,
   midi_file->iter += length;
 }
 
+void
+ags_midi_file_read_notation(AgsMidiFile *midi_file)
+{
+  //TODO:JK: implement me
+}
+
+void
+ags_mid_file_read_midi(AgsMidiFile *midi_file)
+{
+  //TODO:JK: implement me
+}
+
+/**
+ * ags_midi_file_new:
+ * @filename: the filename
+ *
+ * Create a new instance of #AgsMidiFile
+ * 
+ * Returns: the new #AgsMidiFile
+ *
+ * Since: 2.0.0
+ */
 AgsMidiFile*
 ags_midi_file_new(gchar *filename)
 {
