@@ -183,11 +183,10 @@ ags_pulse_client_class_init(AgsPulseClientClass *pulse_client)
    * 
    * Since: 2.0.0
    */
-  param_spec = g_param_spec_object("device",
-				   i18n_pspec("assigned device"),
-				   i18n_pspec("The assigned device"),
-				   G_TYPE_OBJECT,
-				   G_PARAM_READABLE | G_PARAM_WRITABLE);
+  param_spec = g_param_spec_pointer("device",
+				    i18n_pspec("assigned device"),
+				    i18n_pspec("The assigned device"),
+				    G_PARAM_READABLE | G_PARAM_WRITABLE);
   g_object_class_install_property(gobject,
 				  PROP_DEVICE,
 				  param_spec);
@@ -199,11 +198,10 @@ ags_pulse_client_class_init(AgsPulseClientClass *pulse_client)
    * 
    * Since: 2.0.0
    */
-  param_spec = g_param_spec_object("port",
-				   i18n_pspec("assigned port"),
-				   i18n_pspec("The assigned port"),
-				   G_TYPE_OBJECT,
-				   G_PARAM_READABLE | G_PARAM_WRITABLE);
+  param_spec = g_param_spec_pointer("port",
+				    i18n_pspec("assigned port"),
+				    i18n_pspec("The assigned port"),
+				    G_PARAM_READABLE | G_PARAM_WRITABLE);
   g_object_class_install_property(gobject,
 				  PROP_PORT,
 				  param_spec);
@@ -337,7 +335,7 @@ ags_pulse_client_set_property(GObject *gobject,
     {
       GObject *device;
 
-      device = (GObject *) g_value_get_object(value);
+      device = (GObject *) g_value_get_pointer(value);
 
       pthread_mutex_lock(pulse_client_mutex);
 
@@ -361,7 +359,7 @@ ags_pulse_client_set_property(GObject *gobject,
     {
       GObject *port;
 
-      port = (GObject *) g_value_get_object(value);
+      port = (GObject *) g_value_get_pointer(value);
 
       pthread_mutex_lock(pulse_client_mutex);
 
