@@ -81,11 +81,9 @@ static guint sequencer_thread_signals[LAST_SIGNAL];
 GType
 ags_sequencer_thread_get_type()
 {
-  static volatile gsize g_define_type_id__volatile = 0;
-
-  if(g_once_init_enter (&g_define_type_id__volatile)){
-    GType ags_type_sequencer_thread;
+  static GType ags_type_sequencer_thread = 0;
 
+  if(!ags_type_sequencer_thread){
     static const GTypeInfo ags_sequencer_thread_info = {
       sizeof (AgsSequencerThreadClass),
       NULL, /* base_init */
@@ -112,11 +110,9 @@ ags_sequencer_thread_get_type()
     g_type_add_interface_static(ags_type_sequencer_thread,
 				AGS_TYPE_CONNECTABLE,
 				&ags_connectable_interface_info);
-
-    g_once_init_leave (&g_define_type_id__volatile, ags_type_sequencer_thread);
   }
-
-  return g_define_type_id__volatile;
+  
+  return (ags_type_sequencer_thread);
 }
 
 void
@@ -142,7 +138,7 @@ ags_sequencer_thread_class_init(AgsSequencerThreadClass *sequencer_thread)
    *
    * The assigned #AgsSequencer.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_object("sequencer",
 				   i18n_pspec("sequencer assigned to"),
@@ -494,7 +490,7 @@ ags_sequencer_thread_interval_timeout(AgsSequencerThread *sequencer_thread)
  * Returns: the matching #AgsSequencerThread, if not
  * found %NULL.
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 AgsSequencerThread*
 ags_sequencer_thread_find_sequencer(AgsSequencerThread *sequencer_thread,
@@ -521,11 +517,11 @@ ags_sequencer_thread_find_sequencer(AgsSequencerThread *sequencer_thread,
  * ags_sequencer_thread_new:
  * @sequencer: the #AgsSequencer
  *
- * Create a new #AgsSequencerThread.
+ * Create a new instance of #AgsSequencerThread.
  *
  * Returns: the new #AgsSequencerThread
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 AgsSequencerThread*
 ags_sequencer_thread_new(GObject *sequencer)
