@@ -19,8 +19,6 @@
 
 #include <ags/server/security/ags_password_store_manager.h>
 
-#include <pthread.h>
-
 void ags_password_store_manager_class_init(AgsPasswordStoreManagerClass *password_store_manager);
 void ags_password_store_manager_init (AgsPasswordStoreManager *password_store_manager);
 void ags_password_store_manager_finalize(GObject *gobject);
@@ -42,9 +40,11 @@ AgsPasswordStoreManager *ags_password_store_manager = NULL;
 GType
 ags_password_store_manager_get_type (void)
 {
-  static GType ags_type_password_store_manager = 0;
+  static volatile gsize g_define_type_id__volatile = 0;
+
+  if(g_once_init_enter (&g_define_type_id__volatile)){
+    GType ags_type_password_store_manager = 0;
 
-  if(!ags_type_password_store_manager){
     static const GTypeInfo ags_password_store_manager_info = {
       sizeof (AgsPasswordStoreManagerClass),
       NULL, /* base_init */
@@ -58,12 +58,14 @@ ags_password_store_manager_get_type (void)
     };
 
     ags_type_password_store_manager = g_type_register_static(G_TYPE_OBJECT,
-							     "AgsPasswordStoreManager\0",
+							     "AgsPasswordStoreManager",
 							     &ags_password_store_manager_info,
 							     0);
+
+    g_once_init_leave(&g_define_type_id__volatile, ags_type_password_store_manager);
   }
 
-  return (ags_type_password_store_manager);
+  return g_define_type_id__volatile;
 }
 
 void
@@ -152,7 +154,7 @@ ags_password_store_manager_check_password(AgsPasswordStoreManager *password_stor
  *
  * Returns: the #AgsPasswordStoreManager
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 AgsPasswordStoreManager*
 ags_password_store_manager_get_instance()
@@ -179,7 +181,7 @@ ags_password_store_manager_get_instance()
  *
  * Returns: a new #AgsPasswordStoreManager
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 AgsPasswordStoreManager*
 ags_password_store_manager_new()
