@@ -119,7 +119,7 @@ ags_audio_application_context_test_dispose()
   CU_ASSERT(audio_application_context->server == NULL);
   CU_ASSERT(audio_application_context->soundcard == NULL);
   CU_ASSERT(audio_application_context->sequencer == NULL);
-  CU_ASSERT(audio_application_context->distributed_manager == NULL);
+  CU_ASSERT(audio_application_context->sound_server == NULL);
 }
 
 void
