@@ -1,5 +1,5 @@
 /* GSequencer - Advanced GTK Sequencer
- * Copyright (C) 2005-2017 Joël Krähemann
+ * Copyright (C) 2005-2018 Joël Krähemann
  *
  * This file is part of GSequencer.
  *
@@ -25,10 +25,10 @@
 #include <ags/object/ags_async_queue.h>
 #include <ags/object/ags_marshal.h>
 
-#include <ags/thread/ags_mutex_manager.h>
-#include <ags/thread/ags_condition_manager.h>
 #include <ags/thread/ags_returnable_thread.h>
 
+#include <ags/audio/thread/ags_channel_thread.h>
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <unistd.h>
@@ -111,22 +111,24 @@ static pthread_key_t ags_thread_key;
 /* Once-only initialisation of the key */
 static pthread_once_t ags_thread_key_once = PTHREAD_ONCE_INIT;
 
+static pthread_mutex_t ags_thread_class_mutex = PTHREAD_MUTEX_INITIALIZER;
+
 GType
 ags_thread_get_type()
 {
   static volatile gsize g_define_type_id__volatile = 0;
 
   if(g_once_init_enter (&g_define_type_id__volatile)){
-    GType ags_type_thread;
-    
+    GType ags_type_thread = 0;
+
     const GTypeInfo ags_thread_info = {
-      sizeof(AgsThreadClass),
+      sizeof (AgsThreadClass),
       NULL, /* base_init */
       NULL, /* base_finalize */
       (GClassInitFunc) ags_thread_class_init,
       NULL, /* class_finalize */
       NULL, /* class_data */
-      sizeof(AgsThread),
+      sizeof (AgsThread),
       0,    /* n_preallocs */
       (GInstanceInitFunc) ags_thread_init,
     };
@@ -146,7 +148,7 @@ ags_thread_get_type()
 				AGS_TYPE_CONNECTABLE,
 				&ags_connectable_interface_info);
 
-    g_once_init_leave (&g_define_type_id__volatile, ags_type_thread);
+    g_once_init_leave(&g_define_type_id__volatile, ags_type_thread);
   }
 
   return g_define_type_id__volatile;
@@ -174,7 +176,7 @@ ags_thread_class_init(AgsThreadClass *thread)
    *
    * The frequency to run at in Hz.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_double("frequency",
 				   i18n_pspec("frequency as JIFFIE"),
@@ -192,7 +194,7 @@ ags_thread_class_init(AgsThreadClass *thread)
    *
    * The max-frequency to run at in Hz.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_double("max-precision",
 				   i18n_pspec("max precision as JIFFIE"),
@@ -225,7 +227,7 @@ ags_thread_class_init(AgsThreadClass *thread)
    *
    * Returns: the number of cycles to perform
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   thread_signals[CLOCK] =
     g_signal_new("clock",
@@ -233,7 +235,7 @@ ags_thread_class_init(AgsThreadClass *thread)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET (AgsThreadClass, clock),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_UINT__VOID,
+		 ags_cclosure_marshal_UINT__VOID,
 		 G_TYPE_UINT, 0);
 
 
@@ -243,7 +245,7 @@ ags_thread_class_init(AgsThreadClass *thread)
    *
    * The ::start() signal is invoked as thread started.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   thread_signals[START] =
     g_signal_new("start",
@@ -260,7 +262,7 @@ ags_thread_class_init(AgsThreadClass *thread)
    *
    * The ::run() signal is invoked during run loop.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   thread_signals[RUN] =
     g_signal_new("run",
@@ -277,7 +279,7 @@ ags_thread_class_init(AgsThreadClass *thread)
    *
    * The ::suspend() signal is invoked during suspending.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   thread_signals[SUSPEND] =
     g_signal_new("suspend",
@@ -294,7 +296,7 @@ ags_thread_class_init(AgsThreadClass *thread)
    *
    * The ::resume() signal is invoked during resuming.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   thread_signals[RESUME] =
     g_signal_new("resume",
@@ -312,7 +314,7 @@ ags_thread_class_init(AgsThreadClass *thread)
    * The ::timelock() signal is invoked as standard compution
    * time exceeded.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   thread_signals[TIMELOCK] =
     g_signal_new("timelock",
@@ -329,7 +331,7 @@ ags_thread_class_init(AgsThreadClass *thread)
    *
    * The ::stop() signal is invoked as @thread stopped.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   thread_signals[STOP] =
     g_signal_new("stop",
@@ -351,7 +353,7 @@ ags_thread_class_init(AgsThreadClass *thread)
    *
    * Returns: the time spent
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   thread_signals[INTERRUPTED] =
     g_signal_new("interrupted",
@@ -359,7 +361,7 @@ ags_thread_class_init(AgsThreadClass *thread)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET (AgsThreadClass, interrupted),
 		 NULL, NULL,
-		 g_cclosure_user_marshal_UINT__INT_UINT_POINTER,
+		 ags_cclosure_marshal_UINT__INT_UINT_POINTER,
 		 G_TYPE_UINT, 3,
 		 G_TYPE_INT, G_TYPE_UINT, G_TYPE_POINTER);
 }
@@ -376,9 +378,6 @@ ags_thread_connectable_interface_init(AgsConnectableInterface *connectable)
 void
 ags_thread_init(AgsThread *thread)
 {
-  AgsMutexManager *mutex_manager;
-  AgsConditionManager *condition_manager;
-
   AgsConfig *config;
   
   gchar *str;
@@ -392,7 +391,7 @@ ags_thread_init(AgsThread *thread)
 
   config = ags_config_get_instance();
 
-  /* insert audio loop mutex */
+  /* thread mutex */
   thread->obj_mutexattr =
     attr = (pthread_mutexattr_t *) malloc(sizeof(pthread_mutexattr_t));
 
@@ -414,33 +413,12 @@ ags_thread_init(AgsThread *thread)
   pthread_mutex_init(mutex,
 		     attr);
   
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
-  
-  pthread_mutex_lock(application_mutex);
-
-  ags_mutex_manager_insert(mutex_manager,
-			   (GObject *) thread,
-			   mutex);
-  
-  pthread_mutex_unlock(application_mutex);
-
   /* the condition */
   thread->obj_condattr = NULL;
 
   thread->obj_cond =
     cond = (pthread_cond_t *) malloc(sizeof(pthread_cond_t));
   pthread_cond_init(cond, NULL);
-
-  condition_manager = ags_condition_manager_get_instance();
-  
-  pthread_mutex_lock(application_mutex);
-
-  ags_condition_manager_insert(condition_manager,
-			       (GObject *) thread,
-			       cond);
-  
-  pthread_mutex_unlock(application_mutex);
   
   /* fields */
   g_atomic_int_set(&(thread->flags),
@@ -585,16 +563,29 @@ ags_thread_set_property(GObject *gobject,
 {
   AgsThread *thread;
 
+  pthread_mutex_t *thread_mutex;
+
   thread = AGS_THREAD(gobject);
 
+  /* get thread mutex */
+  pthread_mutex_lock(ags_thread_get_class_mutex());
+  
+  thread_mutex = thread->obj_mutex;
+  
+  pthread_mutex_unlock(ags_thread_get_class_mutex());
+
   switch(prop_id){
   case PROP_FREQUENCY:
     {
       gdouble freq;
 
+      pthread_mutex_lock(thread_mutex);
+
       freq = g_value_get_double(value);
 
       if(freq == thread->freq){
+	pthread_mutex_unlock(thread_mutex);
+
 	return;
       }
 
@@ -609,6 +600,8 @@ ags_thread_set_property(GObject *gobject,
       }else{
 	thread->tic_delay = 0;
       }
+
+      pthread_mutex_unlock(thread_mutex);
     }
     break;
   case PROP_MAX_PRECISION:
@@ -616,9 +609,13 @@ ags_thread_set_property(GObject *gobject,
       gdouble max_precision;
       gdouble old_max_precision;
 
+      pthread_mutex_lock(thread_mutex);
+
       max_precision = g_value_get_double(value);
 
       if(max_precision == thread->max_precision){
+	pthread_mutex_unlock(thread_mutex);
+
 	return;
       }
 
@@ -637,6 +634,8 @@ ags_thread_set_property(GObject *gobject,
       }else{
 	thread->tic_delay = 0;
       }
+
+      pthread_mutex_unlock(thread_mutex);
     }
     break;
   default:
@@ -653,17 +652,34 @@ ags_thread_get_property(GObject *gobject,
 {
   AgsThread *thread;
 
+  pthread_mutex_t *thread_mutex;
+
   thread = AGS_THREAD(gobject);
 
+  /* get thread mutex */
+  pthread_mutex_lock(ags_thread_get_class_mutex());
+  
+  thread_mutex = thread->obj_mutex;
+  
+  pthread_mutex_unlock(ags_thread_get_class_mutex());
+
   switch(prop_id){
   case PROP_FREQUENCY:
     {
+      pthread_mutex_lock(thread_mutex);
+
       g_value_set_double(value, thread->freq);
+
+      pthread_mutex_unlock(thread_mutex);
     }
     break;
   case PROP_MAX_PRECISION:
     {
+      pthread_mutex_lock(thread_mutex);
+
       g_value_set_double(value, thread->max_precision);
+
+      pthread_mutex_unlock(thread_mutex);
     }
     break;
   default:
@@ -706,15 +722,12 @@ void
 ags_thread_finalize(GObject *gobject)
 {
   AgsThread *thread, *parent;
-  AgsMutexManager *mutex_manager;
-  AgsConditionManager *condition_manager;
   
   gboolean running;
   gboolean do_exit;
   
   pthread_t *thread_ptr;
   pthread_attr_t *thread_attr;
-  pthread_mutex_t *application_mutex;
 
   void *stackaddr;
   size_t stacksize;
@@ -750,21 +763,8 @@ ags_thread_finalize(GObject *gobject)
   }
 
   /*  */
-  mutex_manager = ags_mutex_manager_get_instance();
-  condition_manager = ags_condition_manager_get_instance();
-
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
-  
-  pthread_mutex_lock(application_mutex);
-
-  ags_mutex_manager_remove(mutex_manager,
-			   (GObject *) thread);
-
   pthread_mutexattr_destroy(thread->obj_mutexattr);
   free(thread->obj_mutexattr);
-
-  ags_condition_manager_remove(condition_manager,
-			       (GObject *) thread);
   
   /*  */  
   free(thread->computing_time);
@@ -831,8 +831,6 @@ ags_thread_finalize(GObject *gobject)
     pthread_detach(*thread_ptr);
   }
   
-  pthread_mutex_unlock(application_mutex);
-
   pthread_attr_destroy(thread_attr);
   free(thread_attr);
 
@@ -848,6 +846,147 @@ ags_thread_finalize(GObject *gobject)
   }
 }
 
+/**
+ * ags_thread_get_class_mutex:
+ * 
+ * Use this function's returned mutex to access mutex fields.
+ *
+ * Returns: the class mutex
+ * 
+ * Since: 2.0.0
+ */
+pthread_mutex_t*
+ags_thread_get_class_mutex()
+{
+  return(&ags_thread_class_mutex);
+}
+
+/**
+ * ags_thread_test_flags:
+ * @thread: the #AgsThread
+ * @flags: the flags
+ *
+ * Test @flags to be set on @thread.
+ * 
+ * Returns: %TRUE if flags are set, else %FALSE
+ *
+ * Since: 2.0.0
+ */
+gboolean
+ags_thread_test_flags(AgsThread *thread, guint flags)
+{
+  gboolean retval;  
+
+  if(!AGS_IS_THREAD(thread)){
+    return(FALSE);
+  }
+  
+  retval = ((flags & (g_atomic_int_get(&(thread->flags)))) != 0) ? TRUE: FALSE;
+    
+  return(retval);
+}
+
+/**
+ * ags_thread_set_flags:
+ * @thread: the #AgsThread
+ * @flags: the flags
+ *
+ * Set flags.
+ * 
+ * Since: 2.0.0
+ */
+void
+ags_thread_set_flags(AgsThread *thread, guint flags)
+{
+  if(!AGS_IS_THREAD(thread)){
+    return;
+  }
+
+  g_atomic_int_or(&(thread->flags), flags);
+}
+
+/**
+ * ags_thread_unset_flags:
+ * @thread: the #AgsThread
+ * @flags: the flags
+ *
+ * Unset flags.
+ * 
+ * Since: 2.0.0
+ */
+void
+ags_thread_unset_flags(AgsThread *thread, guint flags)
+{
+  if(!AGS_IS_THREAD(thread)){
+    return;
+  }
+
+  g_atomic_int_and(&(thread->flags), (~flags));
+}
+
+/**
+ * ags_thread_test_sync_flags:
+ * @thread: the #AgsThread
+ * @sync_flags: the syn flags
+ *
+ * Test @sync_flags to be set on @thread.
+ * 
+ * Returns: %TRUE if sync flags are set, else %FALSE
+ *
+ * Since: 2.0.0
+ */
+gboolean
+ags_thread_test_sync_flags(AgsThread *thread, guint sync_flags)
+{
+  gboolean retval;  
+
+  if(!AGS_IS_THREAD(thread)){
+    return(FALSE);
+  }
+  
+  retval = ((sync_flags & (g_atomic_int_get(&(thread->sync_flags)))) != 0) ? TRUE: FALSE;
+    
+  return(retval);
+}
+
+/**
+ * ags_thread_set_sync_flags:
+ * @thread: the #AgsThread
+ * @sync_flags: the sync flags
+ *
+ * Set sync flags.
+ * 
+ * Since: 2.0.0
+ */
+void
+ags_thread_set_sync_flags(AgsThread *thread, guint sync_flags)
+{
+  if(!AGS_IS_THREAD(thread)){
+    return;
+  }
+
+  g_atomic_int_or(&(thread->sync_flags), sync_flags);
+}
+
+/**
+ * ags_thread_unset_sync_flags:
+ * @thread: the #AgsThread
+ * @sync_flags: the sync flags
+ *
+ * Unset sync flags.
+ * 
+ * Since: 2.0.0
+ */
+void
+ags_thread_unset_sync_flags(AgsThread *thread, guint sync_flags)
+{
+  if(!AGS_IS_THREAD(thread)){
+    return;
+  }
+
+  g_atomic_int_and(&(thread->sync_flags), (~sync_flags));
+}
+
 void
 ags_thread_resume_handler(int sig)
 {
@@ -894,7 +1033,7 @@ ags_thread_suspend_handler(int sig)
  * Unsets AGS_THREAD_WAIT_0, AGS_THREAD_WAIT_1 or AGS_THREAD_WAIT_2.
  * Additionaly the thread is woken up by this function if waiting.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_thread_set_sync(AgsThread *thread, guint tic)
@@ -1012,7 +1151,7 @@ ags_thread_set_sync(AgsThread *thread, guint tic)
  * 
  * Calls ags_thread_set_sync() on all threads.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_thread_set_sync_all(AgsThread *thread, guint tic)
@@ -1028,6 +1167,11 @@ ags_thread_set_sync_all(AgsThread *thread, guint tic)
     /* reset chaos tree */
     g_atomic_int_and(&(thread->flags),
 		    ~AGS_THREAD_IS_CHAOS_TREE);
+
+    if((AGS_THREAD_MARK_SYNCED & (g_atomic_int_get(&(thread->sync_flags)))) != 0){
+      g_atomic_int_or(&(thread->sync_flags),
+		      AGS_THREAD_SYNCED);
+    }
     
     /* descend */
     child = g_atomic_pointer_get(&(thread->children));
@@ -1078,7 +1222,7 @@ ags_thread_set_sync_all(AgsThread *thread, guint tic)
  * Reset all threads. E.g. suspended threads. A synchronization
  * stage after #AgsAsyncQueue run.
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_thread_reset_all(AgsThread *thread)
@@ -1152,30 +1296,23 @@ ags_thread_reset_all(AgsThread *thread)
  * 
  * Locks the threads own mutex and sets the appropriate flag.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_thread_lock(AgsThread *thread)
 {
-  AgsMutexManager *mutex_manager;
-
-  pthread_mutex_t *application_mutex;
   pthread_mutex_t *mutex;
   
-  if(thread == NULL){
+  if(!AGS_IS_THREAD(thread)){
     return;
   }
 
-  /* lookup mutices */
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
+  /* mutex */
+  pthread_mutex_lock(ags_thread_get_class_mutex());
 
-  pthread_mutex_lock(application_mutex);
-
-  mutex = ags_mutex_manager_lookup(mutex_manager,
-				   (GObject *) thread);
+  mutex = thread->obj_mutex;
   
-  pthread_mutex_unlock(application_mutex);
+  pthread_mutex_unlock(ags_thread_get_class_mutex());
 
   /* lock */
   pthread_mutex_lock(mutex);
@@ -1193,30 +1330,23 @@ ags_thread_lock(AgsThread *thread)
  *
  * Returns: %TRUE on success, otherwise %FALSE
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 gboolean
 ags_thread_trylock(AgsThread *thread)
 {
-  AgsMutexManager *mutex_manager;
-
-  pthread_mutex_t *application_mutex;
   pthread_mutex_t *mutex;
   
-  if(thread == NULL){
+  if(!AGS_IS_THREAD(thread)){
     return(FALSE);
   }
 
   /* lookup mutices */
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
+  pthread_mutex_lock(ags_thread_get_class_mutex());
 
-  pthread_mutex_lock(application_mutex);
-
-  mutex = ags_mutex_manager_lookup(mutex_manager,
-				   (GObject *) thread);
+  mutex = thread->obj_mutex;
   
-  pthread_mutex_unlock(application_mutex);
+  pthread_mutex_unlock(ags_thread_get_class_mutex());
 
   /* lock */
   if(pthread_mutex_trylock(mutex) != 0){      
@@ -1235,30 +1365,23 @@ ags_thread_trylock(AgsThread *thread)
  *
  * Unlocks the threads own mutex and unsets the appropriate flag.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_thread_unlock(AgsThread *thread)
 {
-  AgsMutexManager *mutex_manager;
-
-  pthread_mutex_t *application_mutex;
   pthread_mutex_t *mutex;
   
-  if(thread == NULL){
+  if(!AGS_IS_THREAD(thread)){
     return;
   }
 
-  /* lookup mutices */
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
+  /* mutex */
+  pthread_mutex_lock(ags_thread_get_class_mutex());
 
-  pthread_mutex_lock(application_mutex);
-
-  mutex = ags_mutex_manager_lookup(mutex_manager,
-				   (GObject *) thread);
+  mutex = thread->obj_mutex;
   
-  pthread_mutex_unlock(application_mutex);
+  pthread_mutex_unlock(ags_thread_get_class_mutex());
 
   /* unlock */
   g_atomic_int_and(&(thread->flags),
@@ -1275,7 +1398,7 @@ ags_thread_unlock(AgsThread *thread)
  *
  * Returns: the toplevevel #AgsThread
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 AgsThread*
 ags_thread_get_toplevel(AgsThread *thread)
@@ -1299,7 +1422,7 @@ ags_thread_get_toplevel(AgsThread *thread)
  *
  * Returns: the very first #AgsThread within same tree level
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 AgsThread*
 ags_thread_first(AgsThread *thread)
@@ -1323,7 +1446,7 @@ ags_thread_first(AgsThread *thread)
  *
  * Returns: the very last @AgsThread within same tree level
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 AgsThread*
 ags_thread_last(AgsThread *thread)
@@ -1346,7 +1469,7 @@ ags_thread_last(AgsThread *thread)
  * 
  * Remove child of thread.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_thread_remove_child(AgsThread *thread, AgsThread *child)
@@ -1398,7 +1521,7 @@ ags_thread_remove_child(AgsThread *thread, AgsThread *child)
  * 
  * Add child to thread.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_thread_add_child(AgsThread *thread, AgsThread *child)
@@ -1416,7 +1539,7 @@ ags_thread_add_child(AgsThread *thread, AgsThread *child)
  * 
  * Add child to thread.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_thread_add_child_extended(AgsThread *thread, AgsThread *child,
@@ -1435,7 +1558,7 @@ ags_thread_add_child_extended(AgsThread *thread, AgsThread *child,
   if(AGS_IS_MAIN_LOOP(main_loop)){
     pthread_mutex_lock(ags_main_loop_get_tree_lock(AGS_MAIN_LOOP(main_loop)));
 
-    g_signal_connect(G_OBJECT(main_loop), "interrupt",
+    g_signal_connect(AGS_MAIN_LOOP(main_loop), "interrupt",
 		     G_CALLBACK(ags_thread_interrupt_callback), child);
   }
   
@@ -1501,7 +1624,7 @@ ags_thread_add_child_extended(AgsThread *thread, AgsThread *child,
  *
  * Returns: %TRUE if locked otherwise %FALSE
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 gboolean
 ags_thread_parental_is_locked(AgsThread *thread, AgsThread *parent)
@@ -1534,7 +1657,7 @@ ags_thread_parental_is_locked(AgsThread *thread, AgsThread *parent)
  *
  * Returns: %TRUE if locked otherwise %FALSE
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 gboolean
 ags_thread_sibling_is_locked(AgsThread *thread)
@@ -1568,7 +1691,7 @@ ags_thread_sibling_is_locked(AgsThread *thread)
  *
  * Returns: %TRUE if locked otherwise %FALSE
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 gboolean
 ags_thread_children_is_locked(AgsThread *thread)
@@ -1803,7 +1926,7 @@ ags_thread_is_tree_ready(AgsThread *thread,
  *
  * Returns: next matching #AgsThread
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 AgsThread*
 ags_thread_next_parent_locked(AgsThread *thread, AgsThread *parent)
@@ -1835,7 +1958,7 @@ ags_thread_next_parent_locked(AgsThread *thread, AgsThread *parent)
  *
  * Returns: next matching #AgsThread
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 AgsThread*
 ags_thread_next_sibling_locked(AgsThread *thread)
@@ -1873,7 +1996,7 @@ ags_thread_next_sibling_locked(AgsThread *thread)
  *
  * Returns: next matching #AgsThread
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 AgsThread*
 ags_thread_next_children_locked(AgsThread *thread)
@@ -1916,7 +2039,7 @@ ags_thread_next_children_locked(AgsThread *thread)
  *
  * Lock parent tree structure.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_thread_lock_parent(AgsThread *thread, AgsThread *parent)
@@ -1946,7 +2069,7 @@ ags_thread_lock_parent(AgsThread *thread, AgsThread *parent)
  *
  * Lock sibling tree structure.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_thread_lock_sibling(AgsThread *thread)
@@ -1982,7 +2105,7 @@ ags_thread_lock_sibling(AgsThread *thread)
  *
  * Lock child tree structure.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_thread_lock_children(AgsThread *thread)
@@ -2025,7 +2148,7 @@ ags_thread_lock_all(AgsThread *thread)
  *
  * Unlock parent tree structure.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_thread_unlock_parent(AgsThread *thread, AgsThread *parent)
@@ -2062,7 +2185,7 @@ ags_thread_unlock_parent(AgsThread *thread, AgsThread *parent)
  *
  * Unlock sibling tree structure.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_thread_unlock_sibling(AgsThread *thread)
@@ -2105,7 +2228,7 @@ ags_thread_unlock_sibling(AgsThread *thread)
  *
  * Unlock child tree structure.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_thread_unlock_children(AgsThread *thread)
@@ -2167,7 +2290,7 @@ ags_thread_unlock_all(AgsThread *thread)
  *
  * Wait on parent tree structure.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_thread_wait_parent(AgsThread *thread, AgsThread *parent)
@@ -2205,7 +2328,7 @@ ags_thread_wait_parent(AgsThread *thread, AgsThread *parent)
  *
  * Wait on sibling tree structure.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_thread_wait_sibling(AgsThread *thread)
@@ -2249,7 +2372,7 @@ ags_thread_wait_sibling(AgsThread *thread)
  *
  * Wait on child tree structure.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_thread_wait_children(AgsThread *thread)
@@ -2307,7 +2430,7 @@ ags_thread_wait_children(AgsThread *thread)
  *
  * Signals the tree in higher levels.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_thread_signal_parent(AgsThread *thread, AgsThread *parent,
@@ -2337,7 +2460,7 @@ ags_thread_signal_parent(AgsThread *thread, AgsThread *parent,
  *
  * Signals the tree on same level.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_thread_signal_sibling(AgsThread *thread, gboolean broadcast)
@@ -2366,7 +2489,7 @@ ags_thread_signal_sibling(AgsThread *thread, gboolean broadcast)
  *
  * Signals the tree in lower levels.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_thread_signal_children(AgsThread *thread, gboolean broadcast)
@@ -2404,7 +2527,6 @@ guint
 ags_thread_real_clock(AgsThread *thread)
 {
   AgsThread *main_loop, *async_queue;
-  AgsMutexManager *mutex_manager;
 
 #ifdef __APPLE__
   clock_serv_t cclock;
@@ -2433,6 +2555,9 @@ ags_thread_real_clock(AgsThread *thread)
     pthread_mutex_lock(ags_main_loop_get_tree_lock(AGS_MAIN_LOOP(main_loop)));
 
     if((AGS_THREAD_INITIAL_RUN & (g_atomic_int_get(&(thread->flags)))) != 0){
+      g_atomic_int_or(&(thread->sync_flags),
+		      AGS_THREAD_MARK_SYNCED);
+      
       thread->current_tic = ags_main_loop_get_tic(AGS_MAIN_LOOP(main_loop));
 
       if((AGS_THREAD_START_SYNCED_FREQ & (g_atomic_int_get(&(thread->flags)))) != 0 &&
@@ -2623,18 +2748,13 @@ ags_thread_real_clock(AgsThread *thread)
     async_queue = NULL;
   }
   
-  /* lookup mutices */
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
+  /* mutex */
+  pthread_mutex_lock(ags_thread_get_class_mutex());
 
-  pthread_mutex_lock(application_mutex);
+  main_loop_mutex = main_loop->obj_mutex;
+  mutex = thread->obj_mutex;
 
-  main_loop_mutex = ags_mutex_manager_lookup(mutex_manager,
-					     (GObject *) main_loop);
-  mutex = ags_mutex_manager_lookup(mutex_manager,
-				   (GObject *) thread);
-  
-  pthread_mutex_unlock(application_mutex);
+  pthread_mutex_unlock(ags_thread_get_class_mutex());
 
   /* calculate main loop delay */
   pthread_mutex_lock(main_loop_mutex);
@@ -2820,7 +2940,7 @@ ags_thread_real_clock(AgsThread *thread)
  *
  * Returns: the cycles to be performed
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 guint
 ags_thread_clock(AgsThread *thread)
@@ -2867,6 +2987,10 @@ ags_thread_real_start(AgsThread *thread)
 		      AGS_THREAD_WAIT_1 |
 		      AGS_THREAD_WAIT_2)));
 
+  g_atomic_int_and(&(thread->sync_flags),
+		   (~(AGS_THREAD_MARK_SYNCED |
+		      AGS_THREAD_SYNCED)));
+
   g_atomic_int_and(&(thread->sync_flags),
 		   (~(AGS_THREAD_SYNCED_FREQ)));
 
@@ -2894,34 +3018,34 @@ ags_thread_real_start(AgsThread *thread)
  *
  * Add @child to @thread's start queue.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_thread_add_start_queue(AgsThread *thread,
 			   AgsThread *child)
 {
-  GList *start_queue;
+  if(!AGS_IS_THREAD(thread) ||
+     !AGS_IS_THREAD(child)){
+    return;
+  }
   
-  pthread_mutex_lock(thread->start_mutex);
+  pthread_mutex_lock(thread->mutex);
   
-  start_queue = g_atomic_pointer_get(&(thread->start_queue));
-  start_queue = g_list_prepend(start_queue,
-			       child);
-
   g_atomic_pointer_set(&(thread->start_queue),
-		       start_queue);
+		       g_list_prepend(g_atomic_pointer_get(&(thread->start_queue)),
+				      child));
   
-  pthread_mutex_unlock(thread->start_mutex);
+  pthread_mutex_unlock(thread->mutex);
 }
 
 /**
- * ags_thread_add_start_queue:
+ * ags_thread_add_start_queue_all:
  * @thread: the #AgsThread
  * @child: the children as #GList-struct containing #AgsThread to start
  *
  * Add @child to @thread's start queue.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_thread_add_start_queue_all(AgsThread *thread,
@@ -2929,16 +3053,40 @@ ags_thread_add_start_queue_all(AgsThread *thread,
 {
   GList *start_queue;
   
-  pthread_mutex_lock(thread->start_mutex);
+  pthread_mutex_lock(thread->mutex);
   
   start_queue = g_atomic_pointer_get(&(thread->start_queue));
-  start_queue = g_list_concat(start_queue,
-			      g_list_copy(child));
 
-  g_atomic_pointer_set(&(thread->start_queue),
-		       start_queue);
+  if(start_queue == NULL){
+    g_atomic_pointer_set(&(thread->start_queue),
+			 g_list_copy(child));
+  }else{
+    start_queue = g_list_concat(start_queue,
+				g_list_copy(child));
+    
+    g_atomic_pointer_set(&(thread->start_queue),
+			 start_queue);
+  }
+    
+  pthread_mutex_unlock(thread->mutex);
+}
+
+/**
+ * ags_thread_is_running:
+ * @thread: the #AgsThread instance
+ *
+ * Query running flag.
+ *
+ * Since: 2.0.0
+ */
+gboolean
+ags_thread_is_running(AgsThread *thread)
+{
+  if(!AGS_IS_THREAD(thread)){
+    return(FALSE);
+  }
   
-  pthread_mutex_unlock(thread->start_mutex);
+  return(((AGS_THREAD_RUNNING & (g_atomic_int_get(&(thread->flags)))) ? TRUE: FALSE));
 }
 
 /**
@@ -2947,7 +3095,7 @@ ags_thread_add_start_queue_all(AgsThread *thread,
  *
  * Start the thread.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_thread_start(AgsThread *thread)
@@ -2980,7 +3128,6 @@ ags_thread_loop(void *ptr)
 {
   AgsThread *thread, *main_loop;
   AgsThread *async_queue;
-  AgsMutexManager *mutex_manager;
   AgsThread *queued_thread;
   
   GList *start_start_queue, *start_queue, *start_queue_next;
@@ -2989,7 +3136,6 @@ ags_thread_loop(void *ptr)
   guint i, i_stop;
   gboolean wait_for_parent, wait_for_sibling, wait_for_children;
   
-  pthread_mutex_t *application_mutex;
   pthread_mutex_t *mutex;
   
   thread = (AgsThread *) ptr;
@@ -3003,17 +3149,14 @@ ags_thread_loop(void *ptr)
   if(!AGS_IS_MAIN_LOOP(main_loop)){
     main_loop = NULL;
   }
-  
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
-  
-  pthread_mutex_lock(application_mutex);
 
-  mutex = ags_mutex_manager_lookup(mutex_manager,
-				   (GObject *) thread);
-  
-  pthread_mutex_unlock(application_mutex);
+  /* mutex */
+  pthread_mutex_lock(ags_thread_get_class_mutex());
 
+  mutex = thread->obj_mutex;
+  
+  pthread_mutex_unlock(ags_thread_get_class_mutex());
+  
   if(main_loop != NULL){
     pthread_mutex_lock(ags_main_loop_get_tree_lock(AGS_MAIN_LOOP(main_loop)));
     
@@ -3060,7 +3203,7 @@ ags_thread_loop(void *ptr)
 			 NULL);
 
     pthread_mutex_unlock(mutex);
-
+    
     while(start_queue != NULL){
       start_queue_next = start_queue->next;
       
@@ -3129,11 +3272,11 @@ ags_thread_loop(void *ptr)
     g_list_free(start_start_queue);
     
     /* notify start */
-    /* signal AgsAudioLoop */
+    /* signal AgsMainLoop */
     pthread_mutex_lock(thread->start_mutex);
       
     g_atomic_int_set(&(thread->start_done),
-		     TRUE);
+		     TRUE);    
       
     if(g_atomic_int_get(&(thread->start_wait)) == TRUE){
       pthread_cond_broadcast(thread->start_cond);
@@ -3483,7 +3626,7 @@ ags_thread_loop(void *ptr)
  * Only for internal use of ags_thread_loop but you may want to set the your very own
  * class function namely your thread's routine.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_thread_run(AgsThread *thread)
@@ -3621,7 +3764,7 @@ ags_thread_real_stop(AgsThread *thread)
  * 
  * Stop the threads loop by unsetting AGS_THREAD_RUNNING flag.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_thread_stop(AgsThread *thread)
@@ -3634,7 +3777,7 @@ ags_thread_stop(AgsThread *thread)
   if((AGS_THREAD_RUNNING & (g_atomic_int_get(&(thread->flags)))) == 0){
     return;
   }
-
+  
   g_object_ref(G_OBJECT(thread));
   g_signal_emit(G_OBJECT(thread),
 		thread_signal, 0);
@@ -3652,7 +3795,7 @@ ags_thread_stop(AgsThread *thread)
  * 
  * Returns: the time spent
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 guint
 ags_thread_interrupted(AgsThread *thread,
@@ -3691,7 +3834,7 @@ ags_thread_interrupted(AgsThread *thread,
  *
  * Performs hangcheck of thread.
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_thread_hangcheck(AgsThread *thread)
@@ -3849,7 +3992,7 @@ ags_thread_self(void)
  *
  * Returns: the first ordered thread
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 AgsThread*
 ags_thread_chaos_tree(AgsThread *thread)
@@ -3872,7 +4015,7 @@ ags_thread_chaos_tree(AgsThread *thread)
  *
  * Returns: %TRUE if tree is in chaos, otherwise %FALSE
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 gboolean
 ags_thread_is_chaos_tree(AgsThread *thread,
@@ -3940,12 +4083,12 @@ ags_thread_is_chaos_tree(AgsThread *thread,
  * ags_thread_new:
  * @data: an #GObject
  *
- * Create a new #AgsThread you may provide a #gpointer as @data
+ * Create a new instance of #AgsThread you may provide a #gpointer as @data
  * to your thread routine.
  *
  * Returns: the new #AgsThread
  * 
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 AgsThread*
 ags_thread_new(gpointer data)
